@@ -50,6 +50,14 @@ extensions = [
     "bzlink",
 ]
 
+myst_enable_extensions = [
+    "colon_fence",
+    "smartquotes",
+    "deflist",
+    "html_admonition",
+    "fieldlist",
+]
+
 # JSDoc must run successfully for dirs specified, so running
 # tree-wide (the default) will not work currently.
 # When adding more paths to this list, please ensure that they are not
@@ -58,11 +66,13 @@ js_source_path = [
     "../browser/components/backup",
     "../browser/components/backup/actors",
     "../browser/components/backup/resources",
+    "../browser/components/customizableui",
     "../browser/components/extensions",
     "../browser/components/migration",
     "../browser/components/migration/content",
     "../browser/components/uitour",
     "../browser/components/urlbar",
+    "../js/xpconnect/loader",
     "../remote/marionette",
     "../testing/mochitest/BrowserTestUtils",
     "../testing/mochitest/tests/SimpleTest/SimpleTest.js",
@@ -72,13 +82,14 @@ js_source_path = [
     "../toolkit/actors",
     "../toolkit/components/extensions",
     "../toolkit/components/extensions/parent",
-    "../toolkit/components/featuregates",
-    "../toolkit/components/ml/content/ONNXPipeline.mjs",
+    "../toolkit/components/ml/content/backends/ONNXPipeline.mjs",
+    "../toolkit/modules/BrowserUtils.sys.mjs",
     "../toolkit/mozapps/extensions",
     "../toolkit/components/prompts/src",
     "../toolkit/components/pictureinpicture",
     "../toolkit/components/pictureinpicture/content",
     "../toolkit/components/search",
+    "../toolkit/components/uniffi-bindgen-gecko-js/components/generated",
 ]
 root_for_relative_js_paths = ".."
 jsdoc_config_path = "jsdoc.json"
@@ -125,11 +136,12 @@ if on_rtd:
 else:
     # We only need to set the RTD theme when not on RTD because the RTD
     # environment handles this otherwise.
-    import sphinx_rtd_theme
 
     html_theme = "sphinx_rtd_theme"
-    html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
 
+# As we parse the error messages and they can be translated, force
+# the english locale
+os.environ["LANG"] = "C"
 
 html_static_path = ["_static"]
 htmlhelp_basename = "FirefoxTreeDocs"
