@@ -15,6 +15,7 @@
 #include "mozilla/mozalloc.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/ServoCSSParser.h"
+#include "mozilla/StaticPrefs_browser.h"
 #include "mozilla/StaticPrefs_editor.h"
 #include "mozilla/dom/Document.h"
 #include "mozilla/dom/Element.h"
@@ -171,8 +172,7 @@ static void ProcessMarginRightValue(const nsAString* aInputString,
   }
 }
 
-#define CSS_EQUIV_TABLE_NONE \
-  { CSSEditUtils::eCSSEditableProperty_NONE, 0 }
+#define CSS_EQUIV_TABLE_NONE {CSSEditUtils::eCSSEditableProperty_NONE, 0}
 
 const CSSEditUtils::CSSEquivTable boldEquivTable[] = {
     {CSSEditUtils::eCSSEditableProperty_font_weight, true, false, ProcessBValue,
@@ -615,27 +615,26 @@ CSSEditUtils::RemoveCSSInlineStyleWithTransaction(
 
 // static
 void CSSEditUtils::GetDefaultBackgroundColor(nsAString& aColor) {
+  aColor.AssignLiteral("#ffffff");  // Default to white
+
   if (MOZ_UNLIKELY(StaticPrefs::editor_use_custom_colors())) {
-    nsresult rv = Preferences::GetString("editor.background_color", aColor);
+    DebugOnly<nsresult> rv =
+        Preferences::GetString("editor.background_color", aColor);
     // XXX Why don't you validate the pref value?
-    if (NS_FAILED(rv)) {
-      NS_WARNING("failed to get editor.background_color");
-      aColor.AssignLiteral("#ffffff");  // Default to white
-    }
+    NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
+                         "failed to get editor.background_color");
     return;
   }
 
-  if (Preferences::GetBool("browser.display.use_system_colors", false)) {
+  if (StaticPrefs::browser_display_document_color_use() != 2) {
     return;
   }
 
-  nsresult rv =
+  DebugOnly<nsresult> rv =
       Preferences::GetString("browser.display.background_color", aColor);
   // XXX Why don't you validate the pref value?
-  if (NS_FAILED(rv)) {
-    NS_WARNING("failed to get browser.display.background_color");
-    aColor.AssignLiteral("#ffffff");  // Default to white
-  }
+  NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
+                       "failed to get browser.display.background_color");
 }
 
 // static
