@@ -7,14 +7,14 @@
 #define PlaceholderTransaction_h
 
 #include "EditAggregateTransaction.h"
+#include "EditorForwards.h"
+#include "SelectionState.h"
+
 #include "mozilla/Maybe.h"
-#include "mozilla/SelectionState.h"
 #include "mozilla/WeakPtr.h"
 
 namespace mozilla {
 
-class EditorBase;
-
 /**
  * An aggregate transaction that knows how to absorb all subsequent
  * transactions with the same name.  This transaction does not "Do" anything.
@@ -51,7 +51,6 @@ class PlaceholderTransaction final : public EditAggregateTransaction,
 
   NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(PlaceholderTransaction,
                                            EditAggregateTransaction)
-  // ------------ EditAggregateTransaction -----------------------
 
   NS_DECL_EDITTRANSACTIONBASE
   NS_DECL_EDITTRANSACTIONBASE_GETASMETHODS_OVERRIDE(PlaceholderTransaction)
@@ -59,7 +58,7 @@ class PlaceholderTransaction final : public EditAggregateTransaction,
   MOZ_CAN_RUN_SCRIPT NS_IMETHOD RedoTransaction() override;
   NS_IMETHOD Merge(nsITransaction* aTransaction, bool* aDidMerge) override;
 
-  bool StartSelectionEquals(SelectionState& aSelectionState);
+  void AppendChild(EditTransactionBase& aTransaction);
 
   nsresult EndPlaceHolderBatch();
 
