@@ -11,6 +11,7 @@
 #include "mozilla/dom/Comment.h"
 #include "mozilla/dom/CustomElementRegistry.h"
 #include "mozilla/dom/DocGroup.h"
+#include "mozilla/dom/DocumentFragment.h"
 #include "mozilla/dom/DocumentType.h"
 #include "mozilla/dom/Element.h"
 #include "mozilla/dom/LinkStyle.h"
@@ -18,12 +19,14 @@
 #include "mozilla/dom/HTMLImageElement.h"
 #include "mozilla/dom/HTMLTemplateElement.h"
 #include "mozilla/dom/MutationObservers.h"
+#include "mozilla/dom/ShadowRoot.h"
 #include "mozilla/dom/Text.h"
 #include "nsAttrName.h"
 #include "nsContentCreatorFunctions.h"
 #include "nsContentUtils.h"
 #include "nsDocElementCreatedNotificationRunner.h"
 #include "nsEscape.h"
+#include "nsGenericHTMLElement.h"
 #include "nsHtml5AutoPauseUpdate.h"
 #include "nsHtml5DocumentMode.h"
 #include "nsHtml5HtmlAttributes.h"
@@ -39,10 +42,10 @@
 #include "nsIURI.h"
 #include "nsNetUtil.h"
 #include "nsTextNode.h"
+#include "js/ColumnNumber.h"  // JS::ColumnNumberOneOrigin
 
 using namespace mozilla;
 using namespace mozilla::dom;
-using mozilla::dom::Document;
 
 /**
  * Helper class that opens a notification batch if the current doc
@@ -135,13 +138,18 @@ nsHtml5TreeOperation::~nsHtml5TreeOperation() {
 
     void operator()(const opGetDocumentFragmentForTemplate& aOperation) {}
 
+    void operator()(const opSetDocumentFragmentForTemplate& aOperation) {}
+
+    void operator()(const opGetShadowRootFromHost& aOperation) {}
+
     void operator()(const opGetFosterParent& aOperation) {}
 
     void operator()(const opMarkAsBroken& aOperation) {}
 
-    void operator()(const opRunScript& aOperation) {}
+    void operator()(const opRunScriptThatMayDocumentWriteOrBlock& aOperation) {}
 
-    void operator()(const opRunScriptAsyncDefer& aOperation) {}
+    void operator()(
+        const opRunScriptThatCannotDocumentWriteOrBlock& aOperation) {}
 
     void operator()(const opPreventScriptExecution& aOperation) {}
 
@@ -195,6 +203,8 @@ nsHtml5TreeOperation::~nsHtml5TreeOperation() {
 
     void operator()(const opAddLineNumberId& aOperation) {}
 
+    void operator()(const opShallowCloneInto& aOperation) {}
+
     void operator()(const opStartLayout& aOperation) {}
 
     void operator()(const opEnableEncodingMenu& aOperation) {}
@@ -208,7 +218,7 @@ nsHtml5TreeOperation::~nsHtml5TreeOperation() {
 }
 
 nsresult nsHtml5TreeOperation::AppendTextToTextNode(
-    const char16_t* aBuffer, uint32_t aLength, dom::Text* aTextNode,
+    const char16_t* aBuffer, uint32_t aLength, Text* aTextNode,
     nsHtml5DocumentBuilder* aBuilder) {
   MOZ_ASSERT(aTextNode, "Got null text node.");
   MOZ_ASSERT(aBuilder);
@@ -249,9 +259,10 @@ nsresult nsHtml5TreeOperation::Append(nsIContent* aNode, nsIContent* aParent,
   MOZ_ASSERT(aBuilder);
   MOZ_ASSERT(aBuilder->IsInDocUpdate());
   ErrorResult rv;
-  nsHtml5OtherDocUpdate update(aParent->OwnerDoc(), aBuilder->GetDocument());
+  Document* ownerDoc = aParent->OwnerDoc();
+  nsHtml5OtherDocUpdate update(ownerDoc, aBuilder->GetDocument());
   aParent->AppendChildTo(aNode, false, rv);
-  if (!rv.Failed()) {
+  if (!rv.Failed() && !ownerDoc->DOMNotificationsSuspended()) {
     aNode->SetParserHasNotified();
     MutationObservers::NotifyContentAppended(aParent, aNode);
   }
@@ -259,12 +270,12 @@ nsresult nsHtml5TreeOperation::Append(nsIContent* aNode, nsIContent* aParent,
 }
 
 nsresult nsHtml5TreeOperation::Append(nsIContent* aNode, nsIContent* aParent,
-                                      mozilla::dom::FromParser aFromParser,
+                                      FromParser aFromParser,
                                       nsHtml5DocumentBuilder* aBuilder) {
   Maybe<nsHtml5AutoPauseUpdate> autoPause;
-  Maybe<dom::AutoCEReaction> autoCEReaction;
-  dom::DocGroup* docGroup = aParent->OwnerDoc()->GetDocGroup();
-  if (docGroup && aFromParser != mozilla::dom::FROM_PARSER_FRAGMENT) {
+  Maybe<AutoCEReaction> autoCEReaction;
+  DocGroup* docGroup = aParent->OwnerDoc()->GetDocGroup();
+  if (docGroup && aFromParser != FROM_PARSER_FRAGMENT) {
     autoCEReaction.emplace(docGroup->CustomElementReactionsStack(), nullptr);
   }
   nsresult rv = Append(aNode, aParent, aBuilder);
@@ -295,8 +306,10 @@ nsresult nsHtml5TreeOperation::AppendToDocument(
     return rv.StealNSResult();
   }
 
-  aNode->SetParserHasNotified();
-  MutationObservers::NotifyContentInserted(doc, aNode);
+  if (!doc->DOMNotificationsSuspended()) {
+    aNode->SetParserHasNotified();
+    MutationObservers::NotifyContentInserted(doc, aNode);
+  }
 
   NS_ASSERTION(!nsContentUtils::IsSafeToRunScript(),
                "Someone forgot to block scripts");
@@ -385,7 +398,9 @@ nsresult nsHtml5TreeOperation::FosterParent(nsIContent* aNode,
 nsresult nsHtml5TreeOperation::AddAttributes(nsIContent* aNode,
                                              nsHtml5HtmlAttributes* aAttributes,
                                              nsHtml5DocumentBuilder* aBuilder) {
-  dom::Element* node = aNode->AsElement();
+  MOZ_ASSERT(aNode->IsAnyOfHTMLElements(nsGkAtoms::body, nsGkAtoms::html));
+
+  Element* node = aNode->AsElement();
   nsHtml5OtherDocUpdate update(node->OwnerDoc(), aBuilder->GetDocument());
 
   int32_t len = aAttributes->getLength();
@@ -393,7 +408,8 @@ nsresult nsHtml5TreeOperation::AddAttributes(nsIContent* aNode,
     --i;
     nsAtom* localName = aAttributes->getLocalNameNoBoundsCheck(i);
     int32_t nsuri = aAttributes->getURINoBoundsCheck(i);
-    if (!node->HasAttr(nsuri, localName)) {
+    if (!node->HasAttr(nsuri, localName) &&
+        !(nsuri == kNameSpaceID_None && localName == nsGkAtoms::nonce)) {
       nsString value;  // Not Auto, because using it to hold nsStringBuffer*
       aAttributes->getValueNoBoundsCheck(i).ToString(value);
       node->SetAttr(nsuri, localName, aAttributes->getPrefixNoBoundsCheck(i),
@@ -405,49 +421,38 @@ nsresult nsHtml5TreeOperation::AddAttributes(nsIContent* aNode,
 }
 
 void nsHtml5TreeOperation::SetHTMLElementAttributes(
-    dom::Element* aElement, nsAtom* aName, nsHtml5HtmlAttributes* aAttributes) {
+    Element* aElement, nsAtom* aName, nsHtml5HtmlAttributes* aAttributes) {
   int32_t len = aAttributes->getLength();
+  aElement->TryReserveAttributeCount((uint32_t)len);
+  if (aAttributes->getDuplicateAttributeError()) {
+    aElement->SetParserHadDuplicateAttributeError();
+  }
   for (int32_t i = 0; i < len; i++) {
     nsHtml5String val = aAttributes->getValueNoBoundsCheck(i);
     nsAtom* klass = val.MaybeAsAtom();
     if (klass) {
-      aElement->SetSingleClassFromParser(klass);
+      aElement->SetClassAttrFromParser(klass);
     } else {
       nsAtom* localName = aAttributes->getLocalNameNoBoundsCheck(i);
       nsAtom* prefix = aAttributes->getPrefixNoBoundsCheck(i);
       int32_t nsuri = aAttributes->getURINoBoundsCheck(i);
-
       nsString value;  // Not Auto, because using it to hold nsStringBuffer*
       val.ToString(value);
-      if (nsGkAtoms::a == aName && nsGkAtoms::name == localName) {
-        // This is an HTML5-incompliant Geckoism.
-        // Remove when fixing bug 582361
-        NS_ConvertUTF16toUTF8 cname(value);
-        NS_ConvertUTF8toUTF16 uv(nsUnescape(cname.BeginWriting()));
-        aElement->SetAttr(nsuri, localName, prefix, uv, false);
-      } else {
-        aElement->SetAttr(nsuri, localName, prefix, value, false);
-      }
+      aElement->SetAttr(nsuri, localName, prefix, value, false);
     }
   }
 }
 
 nsIContent* nsHtml5TreeOperation::CreateHTMLElement(
-    nsAtom* aName, nsHtml5HtmlAttributes* aAttributes,
-    mozilla::dom::FromParser aFromParser, nsNodeInfoManager* aNodeInfoManager,
-    nsHtml5DocumentBuilder* aBuilder,
-    mozilla::dom::HTMLContentCreatorFunction aCreator) {
-  RefPtr<dom::NodeInfo> nodeInfo = aNodeInfoManager->GetNodeInfo(
+    nsAtom* aName, nsHtml5HtmlAttributes* aAttributes, FromParser aFromParser,
+    nsNodeInfoManager* aNodeInfoManager, nsHtml5DocumentBuilder* aBuilder,
+    HTMLContentCreatorFunction aCreator) {
+  RefPtr<NodeInfo> nodeInfo = aNodeInfoManager->GetNodeInfo(
       aName, nullptr, kNameSpaceID_XHTML, nsINode::ELEMENT_NODE);
   NS_ASSERTION(nodeInfo, "Got null nodeinfo.");
 
-  dom::Element* newContent = nullptr;
   Document* document = nodeInfo->GetDocument();
-  bool willExecuteScript = false;
-  bool isCustomElement = false;
   RefPtr<nsAtom> isAtom;
-  dom::CustomElementDefinition* definition = nullptr;
-
   if (aAttributes) {
     nsHtml5String is = aAttributes->getValue(nsHtml5AttributeName::ATTR_IS);
     if (is) {
@@ -457,114 +462,91 @@ nsIContent* nsHtml5TreeOperation::CreateHTMLElement(
     }
   }
 
-  isCustomElement = (aCreator == NS_NewCustomElement || isAtom);
-  if (isCustomElement && aFromParser != dom::FROM_PARSER_FRAGMENT) {
+  const bool isCustomElement = aCreator == NS_NewCustomElement || isAtom;
+  CustomElementDefinition* customElementDefinition = nullptr;
+  if (isCustomElement && aFromParser != FROM_PARSER_FRAGMENT) {
     RefPtr<nsAtom> tagAtom = nodeInfo->NameAtom();
     RefPtr<nsAtom> typeAtom =
-        (aCreator == NS_NewCustomElement) ? tagAtom : isAtom;
+        aCreator == NS_NewCustomElement ? tagAtom : isAtom;
 
     MOZ_ASSERT(nodeInfo->NameAtom()->Equals(nodeInfo->LocalName()));
-    definition = nsContentUtils::LookupCustomElementDefinition(
+    customElementDefinition = nsContentUtils::LookupCustomElementDefinition(
         document, nodeInfo->NameAtom(), nodeInfo->NamespaceID(), typeAtom);
-
-    if (definition) {
-      willExecuteScript = true;
-    }
   }
 
-  if (willExecuteScript) {  // This will cause custom element constructors to
-                            // run
-    mozilla::dom::AutoSetThrowOnDynamicMarkupInsertionCounter
+  auto DoCreateElement = [&](HTMLContentCreatorFunction aCreator) -> Element* {
+    nsCOMPtr<Element> newElement;
+    if (aCreator) {
+      newElement = aCreator(nodeInfo.forget(), aFromParser);
+    } else {
+      NS_NewHTMLElement(getter_AddRefs(newElement), nodeInfo.forget(),
+                        aFromParser, isAtom, customElementDefinition);
+    }
+
+    MOZ_ASSERT(newElement, "Element creation created null pointer.");
+    Element* element = newElement.get();
+    aBuilder->HoldElement(newElement.forget());
+
+    if (auto* linkStyle = LinkStyle::FromNode(*element)) {
+      linkStyle->DisableUpdates();
+    }
+
+    if (!aAttributes) {
+      return element;
+    }
+
+    SetHTMLElementAttributes(element, aName, aAttributes);
+    return element;
+  };
+
+  if (customElementDefinition) {
+    // This will cause custom element constructors to run.
+    AutoSetThrowOnDynamicMarkupInsertionCounter
         throwOnDynamicMarkupInsertionCounter(document);
     nsHtml5AutoPauseUpdate autoPauseContentUpdate(aBuilder);
-    { nsAutoMicroTask mt; }
-    dom::AutoCEReaction autoCEReaction(
+    {
+      nsAutoMicroTask mt;
+    }
+    AutoCEReaction autoCEReaction(
         document->GetDocGroup()->CustomElementReactionsStack(), nullptr);
-
-    nsCOMPtr<dom::Element> newElement;
-    NS_NewHTMLElement(getter_AddRefs(newElement), nodeInfo.forget(),
-                      aFromParser, isAtom, definition);
-
-    MOZ_ASSERT(newElement, "Element creation created null pointer.");
-    newContent = newElement;
-    aBuilder->HoldElement(newElement.forget());
-
-    if (MOZ_UNLIKELY(aName == nsGkAtoms::style || aName == nsGkAtoms::link)) {
-      if (auto* linkStyle = dom::LinkStyle::FromNode(*newContent)) {
-        linkStyle->SetEnableUpdates(false);
-      }
-    }
-
-    if (!aAttributes) {
-      return newContent;
-    }
-
-    SetHTMLElementAttributes(newContent, aName, aAttributes);
-  } else {
-    nsCOMPtr<dom::Element> newElement;
-
-    if (isCustomElement) {
-      NS_NewHTMLElement(getter_AddRefs(newElement), nodeInfo.forget(),
-                        aFromParser, isAtom, definition);
-    } else {
-      newElement = aCreator(nodeInfo.forget(), aFromParser);
-    }
-
-    MOZ_ASSERT(newElement, "Element creation created null pointer.");
-
-    newContent = newElement;
-    aBuilder->HoldElement(newElement.forget());
-
-    if (MOZ_UNLIKELY(aName == nsGkAtoms::style || aName == nsGkAtoms::link)) {
-      if (auto* linkStyle = dom::LinkStyle::FromNode(*newContent)) {
-        linkStyle->SetEnableUpdates(false);
-      }
-    }
-
-    if (!aAttributes) {
-      return newContent;
-    }
-
-    SetHTMLElementAttributes(newContent, aName, aAttributes);
+    return DoCreateElement(nullptr);
   }
-
-  return newContent;
+  return DoCreateElement(isCustomElement ? nullptr : aCreator);
 }
 
 nsIContent* nsHtml5TreeOperation::CreateSVGElement(
-    nsAtom* aName, nsHtml5HtmlAttributes* aAttributes,
-    mozilla::dom::FromParser aFromParser, nsNodeInfoManager* aNodeInfoManager,
-    nsHtml5DocumentBuilder* aBuilder,
-    mozilla::dom::SVGContentCreatorFunction aCreator) {
+    nsAtom* aName, nsHtml5HtmlAttributes* aAttributes, FromParser aFromParser,
+    nsNodeInfoManager* aNodeInfoManager, nsHtml5DocumentBuilder* aBuilder,
+    SVGContentCreatorFunction aCreator) {
   nsCOMPtr<nsIContent> newElement;
   if (MOZ_LIKELY(aNodeInfoManager->SVGEnabled())) {
-    RefPtr<dom::NodeInfo> nodeInfo = aNodeInfoManager->GetNodeInfo(
+    RefPtr<NodeInfo> nodeInfo = aNodeInfoManager->GetNodeInfo(
         aName, nullptr, kNameSpaceID_SVG, nsINode::ELEMENT_NODE);
     MOZ_ASSERT(nodeInfo, "Got null nodeinfo.");
 
-    mozilla::DebugOnly<nsresult> rv =
+    DebugOnly<nsresult> rv =
         aCreator(getter_AddRefs(newElement), nodeInfo.forget(), aFromParser);
     MOZ_ASSERT(NS_SUCCEEDED(rv) && newElement);
   } else {
-    RefPtr<dom::NodeInfo> nodeInfo = aNodeInfoManager->GetNodeInfo(
+    RefPtr<NodeInfo> nodeInfo = aNodeInfoManager->GetNodeInfo(
         aName, nullptr, kNameSpaceID_disabled_SVG, nsINode::ELEMENT_NODE);
     MOZ_ASSERT(nodeInfo, "Got null nodeinfo.");
 
     // The mismatch between NS_NewXMLElement and SVGContentCreatorFunction
     // argument types is annoying.
-    nsCOMPtr<dom::Element> xmlElement;
-    mozilla::DebugOnly<nsresult> rv =
+    nsCOMPtr<Element> xmlElement;
+    DebugOnly<nsresult> rv =
         NS_NewXMLElement(getter_AddRefs(xmlElement), nodeInfo.forget());
     MOZ_ASSERT(NS_SUCCEEDED(rv) && xmlElement);
     newElement = xmlElement;
   }
 
-  dom::Element* newContent = newElement->AsElement();
+  Element* newContent = newElement->AsElement();
   aBuilder->HoldElement(newElement.forget());
 
   if (MOZ_UNLIKELY(aName == nsGkAtoms::style)) {
-    if (auto* linkStyle = dom::LinkStyle::FromNode(*newContent)) {
-      linkStyle->SetEnableUpdates(false);
+    if (auto* linkStyle = LinkStyle::FromNode(*newContent)) {
+      linkStyle->DisableUpdates();
     }
   }
 
@@ -572,12 +554,16 @@ nsIContent* nsHtml5TreeOperation::CreateSVGElement(
     return newContent;
   }
 
+  if (aAttributes->getDuplicateAttributeError()) {
+    newContent->SetParserHadDuplicateAttributeError();
+  }
+
   int32_t len = aAttributes->getLength();
   for (int32_t i = 0; i < len; i++) {
     nsHtml5String val = aAttributes->getValueNoBoundsCheck(i);
     nsAtom* klass = val.MaybeAsAtom();
     if (klass) {
-      newContent->SetSingleClassFromParser(klass);
+      newContent->SetClassAttrFromParser(klass);
     } else {
       nsAtom* localName = aAttributes->getLocalNameNoBoundsCheck(i);
       nsAtom* prefix = aAttributes->getPrefixNoBoundsCheck(i);
@@ -594,38 +580,42 @@ nsIContent* nsHtml5TreeOperation::CreateSVGElement(
 nsIContent* nsHtml5TreeOperation::CreateMathMLElement(
     nsAtom* aName, nsHtml5HtmlAttributes* aAttributes,
     nsNodeInfoManager* aNodeInfoManager, nsHtml5DocumentBuilder* aBuilder) {
-  nsCOMPtr<dom::Element> newElement;
+  nsCOMPtr<Element> newElement;
   if (MOZ_LIKELY(aNodeInfoManager->MathMLEnabled())) {
-    RefPtr<dom::NodeInfo> nodeInfo = aNodeInfoManager->GetNodeInfo(
+    RefPtr<NodeInfo> nodeInfo = aNodeInfoManager->GetNodeInfo(
         aName, nullptr, kNameSpaceID_MathML, nsINode::ELEMENT_NODE);
     NS_ASSERTION(nodeInfo, "Got null nodeinfo.");
 
-    mozilla::DebugOnly<nsresult> rv =
+    DebugOnly<nsresult> rv =
         NS_NewMathMLElement(getter_AddRefs(newElement), nodeInfo.forget());
     MOZ_ASSERT(NS_SUCCEEDED(rv) && newElement);
   } else {
-    RefPtr<dom::NodeInfo> nodeInfo = aNodeInfoManager->GetNodeInfo(
+    RefPtr<NodeInfo> nodeInfo = aNodeInfoManager->GetNodeInfo(
         aName, nullptr, kNameSpaceID_disabled_MathML, nsINode::ELEMENT_NODE);
     NS_ASSERTION(nodeInfo, "Got null nodeinfo.");
 
-    mozilla::DebugOnly<nsresult> rv =
+    DebugOnly<nsresult> rv =
         NS_NewXMLElement(getter_AddRefs(newElement), nodeInfo.forget());
     MOZ_ASSERT(NS_SUCCEEDED(rv) && newElement);
   }
 
-  dom::Element* newContent = newElement;
+  Element* newContent = newElement;
   aBuilder->HoldElement(newElement.forget());
 
   if (!aAttributes) {
     return newContent;
   }
 
+  if (aAttributes->getDuplicateAttributeError()) {
+    newContent->SetParserHadDuplicateAttributeError();
+  }
+
   int32_t len = aAttributes->getLength();
   for (int32_t i = 0; i < len; i++) {
     nsHtml5String val = aAttributes->getValueNoBoundsCheck(i);
     nsAtom* klass = val.MaybeAsAtom();
     if (klass) {
-      newContent->SetSingleClassFromParser(klass);
+      newContent->SetClassAttrFromParser(klass);
     } else {
       nsAtom* localName = aAttributes->getLocalNameNoBoundsCheck(i);
       nsAtom* prefix = aAttributes->getPrefixNoBoundsCheck(i);
@@ -639,21 +629,20 @@ nsIContent* nsHtml5TreeOperation::CreateMathMLElement(
   return newContent;
 }
 
-void nsHtml5TreeOperation::SetFormElement(nsIContent* aNode,
+void nsHtml5TreeOperation::SetFormElement(nsIContent* aNode, nsIContent* aForm,
                                           nsIContent* aParent) {
-  RefPtr<dom::HTMLFormElement> formElement =
-      dom::HTMLFormElement::FromNodeOrNull(aParent);
+  RefPtr formElement = HTMLFormElement::FromNodeOrNull(aForm);
   NS_ASSERTION(formElement,
                "The form element doesn't implement HTMLFormElement.");
-  nsCOMPtr<nsIFormControl> formControl(do_QueryInterface(aNode));
+  nsCOMPtr<nsIFormControl> formControl = nsIFormControl::FromNodeOrNull(aNode);
   if (formControl &&
       formControl->ControlType() !=
           FormControlType::FormAssociatedCustomElement &&
-      !aNode->AsElement()->HasAttr(kNameSpaceID_None, nsGkAtoms::form)) {
+      !aNode->AsElement()->HasAttr(nsGkAtoms::form) &&
+      aForm->SubtreeRoot() == aParent->SubtreeRoot()) {
     formControl->SetForm(formElement);
-  } else if (HTMLImageElement* domImageElement =
-                 dom::HTMLImageElement::FromNodeOrNull(aNode)) {
-    domImageElement->SetForm(formElement);
+  } else if (auto* image = HTMLImageElement::FromNodeOrNull(aNode)) {
+    image->SetForm(formElement);
   }
 }
 
@@ -698,8 +687,7 @@ nsresult nsHtml5TreeOperation::AppendComment(nsIContent* aParent,
                                              char16_t* aBuffer, int32_t aLength,
                                              nsHtml5DocumentBuilder* aBuilder) {
   nsNodeInfoManager* nodeInfoManager = aParent->OwnerDoc()->NodeInfoManager();
-  RefPtr<dom::Comment> comment =
-      new (nodeInfoManager) dom::Comment(nodeInfoManager);
+  RefPtr<Comment> comment = new (nodeInfoManager) Comment(nodeInfoManager);
   NS_ASSERTION(comment, "Infallible malloc failed?");
   nsresult rv = comment->SetText(aBuffer, aLength, false);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -709,8 +697,8 @@ nsresult nsHtml5TreeOperation::AppendComment(nsIContent* aParent,
 
 nsresult nsHtml5TreeOperation::AppendCommentToDocument(
     char16_t* aBuffer, int32_t aLength, nsHtml5DocumentBuilder* aBuilder) {
-  RefPtr<dom::Comment> comment = new (aBuilder->GetNodeInfoManager())
-      dom::Comment(aBuilder->GetNodeInfoManager());
+  RefPtr<Comment> comment = new (aBuilder->GetNodeInfoManager())
+      Comment(aBuilder->GetNodeInfoManager());
   NS_ASSERTION(comment, "Infallible malloc failed?");
   nsresult rv = comment->SetText(aBuffer, aLength, false);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -723,7 +711,7 @@ nsresult nsHtml5TreeOperation::AppendDoctypeToDocument(
     nsHtml5DocumentBuilder* aBuilder) {
   // Adapted from nsXMLContentSink
   // Create a new doctype node
-  RefPtr<dom::DocumentType> docType =
+  RefPtr<DocumentType> docType =
       NS_NewDOMDocumentType(aBuilder->GetNodeInfoManager(), aName, aPublicId,
                             aSystemId, VoidString());
   return AppendToDocument(docType, aBuilder);
@@ -731,10 +719,14 @@ nsresult nsHtml5TreeOperation::AppendDoctypeToDocument(
 
 nsIContent* nsHtml5TreeOperation::GetDocumentFragmentForTemplate(
     nsIContent* aNode) {
-  dom::HTMLTemplateElement* tempElem =
-      static_cast<dom::HTMLTemplateElement*>(aNode);
-  RefPtr<dom::DocumentFragment> frag = tempElem->Content();
-  return frag;
+  auto* tempElem = static_cast<HTMLTemplateElement*>(aNode);
+  return tempElem->Content();
+}
+
+void nsHtml5TreeOperation::SetDocumentFragmentForTemplate(
+    nsIContent* aNode, nsIContent* aDocumentFragment) {
+  auto* tempElem = static_cast<HTMLTemplateElement*>(aNode);
+  tempElem->SetContent(static_cast<DocumentFragment*>(aDocumentFragment));
 }
 
 nsIContent* nsHtml5TreeOperation::GetFosterParent(nsIContent* aTable,
@@ -763,8 +755,7 @@ void nsHtml5TreeOperation::DoneCreatingElement(nsIContent* aNode) {
 
 void nsHtml5TreeOperation::SvgLoad(nsIContent* aNode) {
   nsCOMPtr<nsIRunnable> event = new nsHtml5SVGLoadDispatcher(aNode);
-  if (NS_FAILED(
-          aNode->OwnerDoc()->Dispatch(TaskCategory::Network, event.forget()))) {
+  if (NS_FAILED(aNode->OwnerDoc()->Dispatch(event.forget()))) {
     NS_WARNING("failed to dispatch svg load dispatcher");
   }
 }
@@ -836,7 +827,7 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
 
     nsresult operator()(const opCreateHTMLElement& aOperation) {
       nsIContent** target = aOperation.mContent;
-      mozilla::dom::HTMLContentCreatorFunction creator = aOperation.mCreator;
+      HTMLContentCreatorFunction creator = aOperation.mCreator;
       nsAtom* name = aOperation.mName;
       nsHtml5HtmlAttributes* attributes = aOperation.mAttributes;
       nsIContent* intendedParent =
@@ -855,7 +846,7 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
 
     nsresult operator()(const opCreateSVGElement& aOperation) {
       nsIContent** target = aOperation.mContent;
-      mozilla::dom::SVGContentCreatorFunction creator = aOperation.mCreator;
+      SVGContentCreatorFunction creator = aOperation.mCreator;
       nsAtom* name = aOperation.mName;
       nsHtml5HtmlAttributes* attributes = aOperation.mAttributes;
       nsIContent* intendedParent =
@@ -891,7 +882,8 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
     }
 
     nsresult operator()(const opSetFormElement& aOperation) {
-      SetFormElement(*(aOperation.mContent), *(aOperation.mFormElement));
+      SetFormElement(*(aOperation.mContent), *(aOperation.mFormElement),
+                     *(aOperation.mIntendedParent));
       return NS_OK;
     }
 
@@ -938,6 +930,37 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       return NS_OK;
     }
 
+    nsresult operator()(const opSetDocumentFragmentForTemplate& aOperation) {
+      SetDocumentFragmentForTemplate(*aOperation.mTemplate,
+                                     *aOperation.mFragment);
+      return NS_OK;
+    }
+
+    nsresult operator()(const opGetShadowRootFromHost& aOperation) {
+      nsIContent* root = nsContentUtils::AttachDeclarativeShadowRoot(
+          *aOperation.mHost, aOperation.mShadowRootMode,
+          aOperation.mShadowRootIsClonable,
+          aOperation.mShadowRootIsSerializable,
+          aOperation.mShadowRootDelegatesFocus);
+      if (root) {
+        *aOperation.mFragHandle = root;
+        return NS_OK;
+      }
+
+      // We failed to attach a new shadow root, so instead attach a template
+      // element and return its content.
+      nsHtml5TreeOperation::Append(*aOperation.mTemplateNode, *aOperation.mHost,
+                                   mBuilder);
+      *aOperation.mFragHandle =
+          static_cast<HTMLTemplateElement*>(*aOperation.mTemplateNode)
+              ->Content();
+      nsContentUtils::LogSimpleConsoleError(
+          u"Failed to attach Declarative Shadow DOM."_ns, "DOM"_ns,
+          mBuilder->GetDocument()->IsInPrivateBrowsing(),
+          mBuilder->GetDocument()->IsInChromeDocShell());
+      return NS_OK;
+    }
+
     nsresult operator()(const opGetFosterParent& aOperation) {
       nsIContent* table = *(aOperation.mTable);
       nsIContent* stackParent = *(aOperation.mStackParent);
@@ -950,7 +973,8 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       return aOperation.mResult;
     }
 
-    nsresult operator()(const opRunScript& aOperation) {
+    nsresult operator()(
+        const opRunScriptThatMayDocumentWriteOrBlock& aOperation) {
       nsIContent* node = *(aOperation.mElement);
       nsAHtml5TreeBuilderState* snapshot = aOperation.mBuilderState;
       if (snapshot) {
@@ -961,8 +985,9 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       return NS_OK;
     }
 
-    nsresult operator()(const opRunScriptAsyncDefer& aOperation) {
-      mBuilder->RunScript(*(aOperation.mElement));
+    nsresult operator()(
+        const opRunScriptThatCannotDocumentWriteOrBlock& aOperation) {
+      mBuilder->RunScript(*(aOperation.mElement), false);
       return NS_OK;
     }
 
@@ -1016,7 +1041,7 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
 
     nsresult operator()(const opSetStyleLineNumber& aOperation) {
       nsIContent* node = *(aOperation.mContent);
-      if (auto* linkStyle = dom::LinkStyle::FromNode(*node)) {
+      if (auto* linkStyle = LinkStyle::FromNode(*node)) {
         linkStyle->SetLineNumber(aOperation.mLineNumber);
       } else {
         MOZ_ASSERT(nsNameSpaceManager::GetInstance()->mSVGDisabled,
@@ -1031,7 +1056,8 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       nsCOMPtr<nsIScriptElement> sele = do_QueryInterface(node);
       if (sele) {
         sele->SetScriptLineNumber(aOperation.mLineNumber);
-        sele->SetScriptColumnNumber(aOperation.mColumnNumber);
+        sele->SetScriptColumnNumber(
+            JS::ColumnNumberOneOrigin(aOperation.mColumnNumber));
         sele->FreezeExecutionAttrs(node->OwnerDoc());
       } else {
         MOZ_ASSERT(nsNameSpaceManager::GetInstance()->mSVGDisabled,
@@ -1064,7 +1090,7 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       nsDependentString depStr(str);
       // See viewsource.css for the possible classes
       nsAutoString klass;
-      element->GetAttr(kNameSpaceID_None, nsGkAtoms::_class, klass);
+      element->GetAttr(nsGkAtoms::_class, klass);
       if (!klass.IsEmpty()) {
         klass.Append(' ');
         klass.Append(depStr);
@@ -1137,7 +1163,7 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       nsAtom* otherAtom = aOperation.mOther;
       // See viewsource.css for the possible classes in addition to "error".
       nsAutoString klass;
-      element->GetAttr(kNameSpaceID_None, nsGkAtoms::_class, klass);
+      element->GetAttr(nsGkAtoms::_class, klass);
       if (!klass.IsEmpty()) {
         klass.AppendLiteral(" error");
         element->SetAttr(kNameSpaceID_None, nsGkAtoms::_class, klass, true);
@@ -1165,7 +1191,7 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       }
 
       nsAutoString title;
-      element->GetAttr(kNameSpaceID_None, nsGkAtoms::title, title);
+      element->GetAttr(nsGkAtoms::title, title);
       if (!title.IsEmpty()) {
         title.Append('\n');
         title.Append(message);
@@ -1185,6 +1211,19 @@ nsresult nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       return NS_OK;
     }
 
+    nsresult operator()(const opShallowCloneInto& aOperation) {
+      nsIContent* src = *aOperation.mSrc;
+      ErrorResult rv;
+      RefPtr<nsINode> clone = src->CloneNode(false, rv);
+      if (NS_WARN_IF(rv.Failed())) {
+        return rv.StealNSResult();
+      }
+      *aOperation.mDst = clone->AsContent();
+      mBuilder->HoldElement(clone.forget().downcast<nsIContent>());
+      return Append(*aOperation.mDst, *aOperation.mIntendedParent,
+                    aOperation.mFromParser, mBuilder);
+    }
+
     nsresult operator()(const opStartLayout& aOperation) {
       mBuilder->StartLayout(
           mInterrupted);  // this causes a notification flush anyway
