@@ -2,8 +2,6 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-from __future__ import absolute_import, print_function
-
 import contextlib
 import os
 
@@ -78,20 +76,17 @@ class BaseNavigationTestCase(WindowManagerMixin, MarionetteTestCase):
             # TODO: DO NOT USE MOST RECENT WINDOW BUT CURRENT ONE
             return self.marionette.execute_script(
                 """
-              const { AppConstants } = ChromeUtils.import(
-                "resource://gre/modules/AppConstants.jsm"
+              const { AppConstants } = ChromeUtils.importESModule(
+                "resource://gre/modules/AppConstants.sys.mjs"
               );
 
               let win = null;
 
               if (AppConstants.MOZ_APP_NAME == "fennec") {
-                const { Services } = ChromeUtils.import(
-                  "resource://gre/modules/Services.jsm"
-                );
                 win = Services.wm.getMostRecentWindow("navigator:browser");
               } else {
-                const { BrowserWindowTracker } = ChromeUtils.import(
-                  "resource:///modules/BrowserWindowTracker.jsm"
+                const { BrowserWindowTracker } = ChromeUtils.importESModule(
+                  "resource:///modules/BrowserWindowTracker.sys.mjs"
                 );
                 win = BrowserWindowTracker.getTopWindow();
               }
@@ -294,7 +289,7 @@ class TestNavigate(BaseNavigationTestCase):
         self.marionette.navigate("about:robots")
         self.assertFalse(self.is_remote_tab)
 
-    def test_stale_element_after_remoteness_change(self):
+    def test_no_such_element_after_remoteness_change(self):
         self.marionette.navigate(self.test_page_file_url)
         self.assertTrue(self.is_remote_tab)
         elem = self.marionette.find_element(By.ID, "file-url")
@@ -302,20 +297,6 @@ class TestNavigate(BaseNavigationTestCase):
         self.marionette.navigate("about:robots")
         self.assertFalse(self.is_remote_tab)
 
-        # Force a GC to get rid of the replaced browsing context.
-        with self.marionette.using_context("chrome"):
-            self.marionette.execute_async_script(
-                """
-                const resolve = arguments[0];
-
-                var memSrv = Cc["@mozilla.org/memory-reporter-manager;1"]
-                  .getService(Ci.nsIMemoryReporterManager);
-
-                Services.obs.notifyObservers(null, "child-mmu-request", null);
-                memSrv.minimizeMemoryUsage(resolve);
-            """
-            )
-
         with self.assertRaises(errors.StaleElementException):
             elem.click()
 
@@ -336,6 +317,18 @@ class TestNavigate(BaseNavigationTestCase):
         self.marionette.restart()
 
         self.marionette.navigate(inline("<input autofocus>"))
+
+        # Per spec, autofocus candidates will be
+        # flushed by next paint, so we use rAF here to
+        # ensure the candidates are flushed.
+        self.marionette.execute_async_script(
+            """
+        const callback = arguments[arguments.length - 1];
+        window.requestAnimationFrame(function() {
+            window.requestAnimationFrame(callback);
+        });
+        """
+        )
         focus_el = self.marionette.find_element(By.CSS_SELECTOR, ":focus")
         self.assertEqual(self.marionette.get_active_element(), focus_el)
 
@@ -352,7 +345,7 @@ class TestNavigate(BaseNavigationTestCase):
         self.assertFalse(self.is_remote_tab)
 
         with self.marionette.using_context("chrome"):
-            urlbar = self.marionette.find_element(By.ID, "urlbar-input")
+            urlbar = self.marionette.execute_script("return gURLBar.inputField")
             urlbar.send_keys(self.mod_key + "a")
             urlbar.send_keys(self.mod_key + "x")
             urlbar.send_keys("about:support" + Keys.ENTER)
@@ -367,7 +360,7 @@ class TestNavigate(BaseNavigationTestCase):
         self.assertTrue(self.is_remote_tab)
 
         with self.marionette.using_context("chrome"):
-            urlbar = self.marionette.find_element(By.ID, "urlbar-input")
+            urlbar = self.marionette.execute_script("return gURLBar.inputField")
             urlbar.send_keys(self.mod_key + "a")
             urlbar.send_keys(self.mod_key + "x")
             urlbar.send_keys(self.test_page_remote + Keys.ENTER)
@@ -392,12 +385,7 @@ class TestBackForwardNavigation(BaseNavigationTestCase):
 
         def check_page_status(page, expected_history_length):
             if "alert_text" in page:
-                if page["alert_text"] is None:
-                    # navigation auto-dismisses beforeunload prompt
-                    with self.assertRaises(errors.NoAlertPresentException):
-                        Alert(self.marionette).text
-                else:
-                    self.assertEqual(Alert(self.marionette).text, page["alert_text"])
+                self.assertEqual(Alert(self.marionette).text, page["alert_text"])
 
             self.assertEqual(self.marionette.get_url(), page["url"])
             self.assertEqual(self.history_length, expected_history_length)
@@ -449,29 +437,6 @@ class TestBackForwardNavigation(BaseNavigationTestCase):
         self.marionette.go_back()
         self.marionette.go_forward()
 
-    def test_dismissed_beforeunload_prompt(self):
-        url_beforeunload = inline(
-            """
-          <input type="text">
-          <script>
-            window.addEventListener("beforeunload", function (event) {
-              event.preventDefault();
-            });
-          </script>
-        """
-        )
-
-        def modify_page():
-            self.marionette.find_element(By.TAG_NAME, "input").send_keys("foo")
-
-        test_pages = [
-            {"url": inline("<p>foobar</p>"), "alert_text": None},
-            {"url": url_beforeunload, "callback": modify_page},
-            {"url": inline("<p>foobar</p>"), "alert_text": None},
-        ]
-
-        self.run_bfcache_test(test_pages)
-
     def test_data_urls(self):
         test_pages = [
             {"url": inline("<p>foobar</p>")},
@@ -693,26 +658,6 @@ class TestRefresh(BaseNavigationTestCase):
         self.marionette.refresh()
         self.assertEqual(self.test_page_file_url, self.marionette.get_url())
 
-    def test_dismissed_beforeunload_prompt(self):
-        self.marionette.navigate(
-            inline(
-                """
-          <input type="text">
-          <script>
-            window.addEventListener("beforeunload", function (event) {
-              event.preventDefault();
-            });
-          </script>
-        """
-            )
-        )
-        self.marionette.find_element(By.TAG_NAME, "input").send_keys("foo")
-        self.marionette.refresh()
-
-        # navigation auto-dismisses beforeunload prompt
-        with self.assertRaises(errors.NoAlertPresentException):
-            Alert(self.marionette).text
-
     def test_image(self):
         image = self.marionette.absolute_url("black.png")
 
