@@ -9,6 +9,8 @@ const { RemoteSettings } = ChromeUtils.importESModule(
   "resource://services-settings/remote-settings.sys.mjs"
 );
 
+const isAndroid = AppConstants.platform == "android";
+
 XPCOMUtils.defineLazyServiceGetter(
   this,
   "peuService",
@@ -20,6 +22,11 @@ const TEST_REDIRECT_TOP_PAGE =
   TEST_3RD_PARTY_DOMAIN + TEST_PATH + "redirect.sjs?" + TEST_TOP_PAGE;
 const TEST_REDIRECT_3RD_PARTY_PAGE =
   TEST_DOMAIN + TEST_PATH + "redirect.sjs?" + TEST_3RD_PARTY_PARTITIONED_PAGE;
+const TEST_REDIRECT_ANOTHER_3RD_PARTY_PAGE =
+  TEST_ANOTHER_3RD_PARTY_DOMAIN_HTTPS +
+  TEST_PATH +
+  "redirect.sjs?" +
+  TEST_TOP_PAGE_HTTPS;
 
 const COLLECTION_NAME = "partitioning-exempt-urls";
 const EXCEPTION_LIST_PREF_NAME = "privacy.restrict3rdpartystorage.skip_list";
@@ -27,7 +34,7 @@ const EXCEPTION_LIST_PREF_NAME = "privacy.restrict3rdpartystorage.skip_list";
 async function cleanup() {
   Services.prefs.clearUserPref(EXCEPTION_LIST_PREF_NAME);
   await new Promise(resolve => {
-    Services.clearData.deleteData(Ci.nsIClearDataService.CLEAR_ALL, value =>
+    Services.clearData.deleteData(Ci.nsIClearDataService.CLEAR_ALL, () =>
       resolve()
     );
   });
@@ -54,6 +61,8 @@ add_setup(async function () {
     ],
   });
   registerCleanupFunction(cleanup);
+
+  await cleanup();
 });
 
 function executeContentScript(browser, callback, options = {}) {
@@ -118,7 +127,7 @@ function getDataFromFirstParty(browser) {
 function createDataInThirdParty(browser, value) {
   return executeContentScript(browser, writeNetworkCookie, {
     page: TEST_3RD_PARTY_PARTITIONED_PAGE,
-    value,
+    value: value + ";SameSite=None;Secure;Partitioned;",
   });
 }
 function getDataFromThirdParty(browser) {
@@ -132,6 +141,7 @@ async function redirectWithUserInteraction(browser, url, wait = null) {
     browser,
     (content, value) => {
       content.document.userInteractionForTesting();
+      content.document.notifyUserGestureActivation();
 
       let link = content.document.createElement("a");
       link.appendChild(content.document.createTextNode("click me!"));
@@ -146,20 +156,36 @@ async function redirectWithUserInteraction(browser, url, wait = null) {
   await BrowserTestUtils.browserLoaded(browser, false, wait || url);
 }
 
+async function redirectWithoutUserInteraction(browser, url, wait = null) {
+  await executeContentScript(
+    browser,
+    (content, value) => {
+      let link = content.document.createElement("a");
+      link.appendChild(content.document.createTextNode("click me!"));
+      link.href = value;
+      content.document.body.appendChild(link);
+      link.click();
+    },
+    {
+      value: url,
+    }
+  );
+  await BrowserTestUtils.browserLoaded(browser, false, wait || url);
+}
+
 async function checkData(browser, options) {
+  let data;
+
+  // We check if the cookie string includes the expected cookie because the
+  // cookie string might contain both partitioned and unpartitioned cookies at
+  // the same time.
   if ("firstParty" in options) {
-    is(
-      await getDataFromFirstParty(browser),
-      options.firstParty,
-      "correct first-party data"
-    );
+    data = await getDataFromFirstParty(browser);
+    ok(data.includes(options.firstParty), "correct first-party data");
   }
   if ("thirdParty" in options) {
-    is(
-      await getDataFromThirdParty(browser),
-      options.thirdParty,
-      "correct third-party data"
-    );
+    data = await getDataFromThirdParty(browser);
+    ok(data.includes(options.thirdParty), "correct third-party data");
   }
 }
 
@@ -168,7 +194,14 @@ async function runTestRedirectHeuristic(disableHeuristics) {
 
   await SpecialPowers.pushPrefEnv({
     set: [
-      ["privacy.restrict3rdpartystorage.heuristic.recently_visited", true],
+      [
+        "privacy.restrict3rdpartystorage.heuristic.opened_window_after_interaction",
+        false,
+      ],
+      ["privacy.restrict3rdpartystorage.heuristic.window_open", false],
+      ["privacy.restrict3rdpartystorage.heuristic.recently_visited", isAndroid],
+      ["privacy.restrict3rdpartystorage.heuristic.navigation", !isAndroid],
+      ["privacy.restrict3rdpartystorage.heuristic.redirect", false],
       ["privacy.antitracking.enableWebcompat", !disableHeuristics],
     ],
   });
@@ -259,14 +292,156 @@ async function runTestRedirectHeuristic(disableHeuristics) {
   await cleanup();
 }
 
+async function runTestRedirectHeuristicWithoutInteraction() {
+  info("Starting Dynamic FPI Redirect Heuristic test without interaction...");
+
+  await SpecialPowers.pushPrefEnv({
+    set: [
+      ["privacy.restrict3rdpartystorage.heuristic.recently_visited", isAndroid],
+      ["privacy.restrict3rdpartystorage.heuristic.navigation", !isAndroid],
+      ["privacy.antitracking.enableWebcompat", true],
+    ],
+  });
+
+  // mark third-party as tracker
+  await UrlClassifierTestUtils.addTestTrackers();
+
+  info("Creating a new tab");
+  let tab = BrowserTestUtils.addTab(gBrowser, TEST_TOP_PAGE);
+  gBrowser.selectedTab = tab;
+
+  let browser = gBrowser.getBrowserForTab(tab);
+  await BrowserTestUtils.browserLoaded(browser);
+
+  info("initializing...");
+  await checkData(browser, { firstParty: "", thirdParty: "" });
+
+  await Promise.all([
+    createDataInFirstParty(browser, "firstParty"),
+    createDataInThirdParty(browser, "thirdParty"),
+  ]);
+
+  await checkData(browser, {
+    firstParty: "firstParty",
+    thirdParty: "",
+  });
+
+  info("load third-party content as first-party");
+  await redirectWithoutUserInteraction(
+    browser,
+    TEST_REDIRECT_3RD_PARTY_PAGE,
+    TEST_3RD_PARTY_PARTITIONED_PAGE
+  );
+
+  await checkData(browser, { firstParty: "" });
+  await createDataInFirstParty(browser, "heuristicFirstParty");
+  await checkData(browser, { firstParty: "heuristicFirstParty" });
+
+  info("redirect back to first-party page");
+  await redirectWithoutUserInteraction(
+    browser,
+    TEST_REDIRECT_TOP_PAGE,
+    TEST_TOP_PAGE
+  );
+
+  info("third-party tracker should NOT able to access first-party data");
+  await checkData(browser, {
+    firstParty: "firstParty",
+    thirdParty: "",
+  });
+
+  // remove third-party from tracker
+  await UrlClassifierTestUtils.cleanupTestTrackers();
+
+  info("load third-party content as first-party");
+  await redirectWithoutUserInteraction(
+    browser,
+    TEST_REDIRECT_3RD_PARTY_PAGE,
+    TEST_3RD_PARTY_PARTITIONED_PAGE
+  );
+
+  await checkData(browser, {
+    firstParty: "heuristicFirstParty",
+  });
+
+  info("redirect back to first-party page");
+  await redirectWithoutUserInteraction(
+    browser,
+    TEST_REDIRECT_TOP_PAGE,
+    TEST_TOP_PAGE
+  );
+
+  // This heuristic doesn't work in Android because
+  // it doesn't have the session history in the parent
+  // process (yet).
+  info(
+    `third-party page should ${
+      isAndroid ? "" : "not "
+    }be able to access first-party data`
+  );
+  await checkData(browser, {
+    firstParty: "firstParty",
+    thirdParty: isAndroid ? "heuristicFirstParty" : "",
+  });
+
+  info("Removing the tab");
+  BrowserTestUtils.removeTab(tab);
+
+  await SpecialPowers.popPrefEnv();
+
+  await cleanup();
+}
+
+async function runTestRedirectHeuristicWithSameSite() {
+  info("Starting Dynamic FPI Redirect Between Same Site Heuristic test...");
+
+  info("Creating a new tab");
+  let tab = BrowserTestUtils.addTab(gBrowser, TEST_TOP_PAGE_HTTPS);
+  gBrowser.selectedTab = tab;
+
+  let browser = gBrowser.getBrowserForTab(tab);
+  await BrowserTestUtils.browserLoaded(browser);
+
+  info(
+    `Redirecting from ${TEST_DOMAIN_HTTPS} to ${TEST_ANOTHER_3RD_PARTY_DOMAIN_HTTPS}`
+  );
+
+  await redirectWithUserInteraction(
+    browser,
+    TEST_REDIRECT_ANOTHER_3RD_PARTY_PAGE,
+    TEST_TOP_PAGE_HTTPS
+  );
+
+  info("Checking if a permission was set between the redirect");
+
+  const principal = browser.contentPrincipal;
+
+  is(
+    Services.perms.testPermissionFromPrincipal(
+      principal,
+      `3rdPartyStorage^${TEST_ANOTHER_3RD_PARTY_DOMAIN_HTTPS.slice(0, -1)}`
+    ),
+    Services.perms.UNKNOWN_ACTION,
+    "No permission was set for same-site redirect"
+  );
+
+  info("Removing the tab");
+  BrowserTestUtils.removeTab(tab);
+
+  await cleanup();
+}
 add_task(async function testRedirectHeuristic() {
   await runTestRedirectHeuristic(false);
 });
 
+add_task(runTestRedirectHeuristicWithSameSite);
+
 add_task(async function testRedirectHeuristicDisabled() {
   await runTestRedirectHeuristic(true);
 });
 
+add_task(runTestRedirectHeuristicWithoutInteraction);
+
 class UpdateEvent extends EventTarget {}
 function waitForEvent(element, eventName) {
   return new Promise(function (resolve) {
