@@ -9,74 +9,29 @@
 
 "use strict";
 
-// Globals
-
-var { AppConstants } = ChromeUtils.import(
-  "resource://gre/modules/AppConstants.jsm"
+var { Integration } = ChromeUtils.importESModule(
+  "resource://gre/modules/Integration.sys.mjs"
 );
-var { Integration } = ChromeUtils.import(
-  "resource://gre/modules/Integration.jsm"
+var { XPCOMUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/XPCOMUtils.sys.mjs"
 );
-var { XPCOMUtils } = ChromeUtils.import(
-  "resource://gre/modules/XPCOMUtils.jsm"
+const { AppConstants } = ChromeUtils.importESModule(
+  "resource://gre/modules/AppConstants.sys.mjs"
 );
 
-ChromeUtils.defineModuleGetter(
-  this,
-  "DownloadPaths",
-  "resource://gre/modules/DownloadPaths.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "Downloads",
-  "resource://gre/modules/Downloads.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "FileUtils",
-  "resource://gre/modules/FileUtils.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "HttpServer",
-  "resource://testing-common/httpd.js"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "NetUtil",
-  "resource://gre/modules/NetUtil.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "PlacesUtils",
-  "resource://gre/modules/PlacesUtils.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "Promise",
-  "resource://gre/modules/Promise.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "Services",
-  "resource://gre/modules/Services.jsm"
-);
-ChromeUtils.defineModuleGetter(this, "OS", "resource://gre/modules/osfile.jsm");
-ChromeUtils.defineModuleGetter(
-  this,
-  "FileTestUtils",
-  "resource://testing-common/FileTestUtils.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "MockRegistrar",
-  "resource://testing-common/MockRegistrar.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "TestUtils",
-  "resource://testing-common/TestUtils.jsm"
-);
+ChromeUtils.defineESModuleGetters(this, {
+  DownloadPaths: "resource://gre/modules/DownloadPaths.sys.mjs",
+  Downloads: "resource://gre/modules/Downloads.sys.mjs",
+  E10SUtils: "resource://gre/modules/E10SUtils.sys.mjs",
+  FileTestUtils: "resource://testing-common/FileTestUtils.sys.mjs",
+  FileUtils: "resource://gre/modules/FileUtils.sys.mjs",
+  HttpServer: "resource://testing-common/httpd.sys.mjs",
+  MockRegistrar: "resource://testing-common/MockRegistrar.sys.mjs",
+  NetUtil: "resource://gre/modules/NetUtil.sys.mjs",
+  PlacesUtils: "resource://gre/modules/PlacesUtils.sys.mjs",
+  TelemetryTestUtils: "resource://testing-common/TelemetryTestUtils.sys.mjs",
+  TestUtils: "resource://testing-common/TestUtils.sys.mjs",
+});
 
 XPCOMUtils.defineLazyServiceGetter(
   this,
@@ -86,10 +41,10 @@ XPCOMUtils.defineLazyServiceGetter(
 );
 
 /* global DownloadIntegration */
-Integration.downloads.defineModuleGetter(
+Integration.downloads.defineESModuleGetter(
   this,
   "DownloadIntegration",
-  "resource://gre/modules/DownloadIntegration.jsm"
+  "resource://gre/modules/DownloadIntegration.sys.mjs"
 );
 
 const ServerSocket = Components.Constructor(
@@ -110,6 +65,12 @@ XPCOMUtils.defineLazyServiceGetter(
   "nsIMIMEService"
 );
 
+const ReferrerInfo = Components.Constructor(
+  "@mozilla.org/referrer-info;1",
+  "nsIReferrerInfo",
+  "init"
+);
+
 const TEST_TARGET_FILE_NAME = "test-download.txt";
 const TEST_STORE_FILE_NAME = "test-downloads.json";
 
@@ -118,59 +79,14 @@ const TEST_STORE_FILE_NAME = "test-downloads.json";
 const TEST_REFERRER_URL = "https://www.example.com/referrer.html";
 
 const TEST_DATA_SHORT = "This test string is downloaded.";
-// Generate using gzipCompressString in TelemetryController.jsm.
+// Generate using gzipCompressString in TelemetryController.sys.mjs.
 const TEST_DATA_SHORT_GZIP_ENCODED_FIRST = [
-  31,
-  139,
-  8,
-  0,
-  0,
-  0,
-  0,
-  0,
-  0,
-  3,
-  11,
-  201,
-  200,
-  44,
-  86,
-  40,
-  73,
-  45,
-  46,
-  81,
-  40,
-  46,
-  41,
-  202,
-  204,
+  31, 139, 8, 0, 0, 0, 0, 0, 0, 3, 11, 201, 200, 44, 86, 40, 73, 45, 46, 81, 40,
+  46, 41, 202, 204,
 ];
 const TEST_DATA_SHORT_GZIP_ENCODED_SECOND = [
-  75,
-  87,
-  0,
-  114,
-  83,
-  242,
-  203,
-  243,
-  114,
-  242,
-  19,
-  83,
-  82,
-  83,
-  244,
-  0,
-  151,
-  222,
-  109,
-  43,
-  31,
-  0,
-  0,
-  0,
+  75, 87, 0, 114, 83, 242, 203, 243, 114, 242, 19, 83, 82, 83, 244, 0, 151, 222,
+  109, 43, 31, 0, 0, 0,
 ];
 const TEST_DATA_SHORT_GZIP_ENCODED = TEST_DATA_SHORT_GZIP_ENCODED_FIRST.concat(
   TEST_DATA_SHORT_GZIP_ENCODED_SECOND
@@ -212,6 +128,18 @@ function getTempFile(leafName) {
   return FileTestUtils.getTempFile(leafName);
 }
 
+/**
+ * Check for file existence.
+ * @param {string} path The file path.
+ */
+async function fileExists(path) {
+  try {
+    return (await IOUtils.stat(path)).type == "regular";
+  } catch (ex) {
+    return false;
+  }
+}
+
 /**
  * Waits for pending events to be processed.
  *
@@ -245,7 +173,7 @@ function promiseTimeout(aTime) {
  *        String containing the URI that will be visited.
  *
  * @return {Promise}
- * @resolves Array [aTime, aTransitionType] from nsINavHistoryObserver.onVisit.
+ * @resolves Array [aTime, aTransitionType] from page-visited places event.
  * @rejects Never.
  */
 function promiseWaitForVisit(aUrl) {
@@ -294,7 +222,8 @@ function promiseNewDownload(aSourceUrl) {
  *        {
  *          isPrivate: Boolean indicating whether the download originated from a
  *                     private window.
- *          referrer: String containing the referrer for the download source.
+ *          referrerInfo: referrerInfo for the download source.
+ *          cookieJarSettings: cookieJarSettings for the download source.
  *          targetFile: nsIFile for the target, or null to use a temporary file.
  *          outPersist: Receives a reference to the created nsIWebBrowserPersist
  *                      instance.
@@ -346,6 +275,16 @@ function promiseStartLegacyDownload(aSourceUrl, aOptions) {
     mimeInfo.preferredApplicationHandler = localHandlerApp;
     mimeInfo.preferredAction = Ci.nsIMIMEInfo.useHelperApp;
   }
+  if (aOptions && aOptions.launcherId) {
+    Assert.ok(mimeInfo != null);
+
+    let gioHandlerApp = Cc["@mozilla.org/gio-service;1"]
+      .getService(Ci.nsIGIOService)
+      .createHandlerAppFromAppId(aOptions.launcherId);
+
+    mimeInfo.preferredApplicationHandler = gioHandlerApp;
+    mimeInfo.preferredAction = Ci.nsIMIMEInfo.useHelperApp;
+  }
 
   if (aOptions && aOptions.launchWhenSucceeded) {
     Assert.ok(mimeInfo != null);
@@ -362,7 +301,7 @@ function promiseStartLegacyDownload(aSourceUrl, aOptions) {
 
   return new Promise(resolve => {
     Downloads.getList(Downloads.ALL)
-      .then(function(aList) {
+      .then(function (aList) {
         // Temporarily register a view that will get notified when the download we
         // are controlling becomes visible in the list of downloads.
         aList
@@ -384,34 +323,39 @@ function promiseStartLegacyDownload(aSourceUrl, aOptions) {
           .catch(do_report_unexpected_exception);
 
         let isPrivate = aOptions && aOptions.isPrivate;
-        let referrer =
-          aOptions && aOptions.referrer
-            ? NetUtil.newURI(aOptions.referrer)
-            : null;
+        let referrerInfo = aOptions ? aOptions.referrerInfo : null;
+        let cookieJarSettings = aOptions ? aOptions.cookieJarSettings : null;
+        let classification =
+          aOptions?.downloadClassification ??
+          Ci.nsITransfer.DOWNLOAD_ACCEPTABLE;
         // Initialize the components so they reference each other.  This will cause
         // the Download object to be created and added to the public downloads.
         transfer.init(
           sourceURI,
+          null,
           NetUtil.newURI(targetFile),
           null,
           mimeInfo,
           null,
           null,
           persist,
-          isPrivate
+          isPrivate,
+          classification,
+          null
         );
         persist.progressListener = transfer;
 
         // Start the actual download process.
-        persist.savePrivacyAwareURI(
+        persist.saveURI(
           sourceURI,
           Services.scriptSecurityManager.getSystemPrincipal(),
           0,
-          referrer,
-          Ci.nsIHttpChannel.REFERRER_POLICY_UNSAFE_URL,
+          referrerInfo,
+          cookieJarSettings,
           null,
           null,
           targetFile,
+          Ci.nsIContentPolicy.TYPE_SAVEAS_DOWNLOAD,
           isPrivate
         );
       })
@@ -440,7 +384,7 @@ function promiseStartExternalHelperAppServiceDownload(aSourceUrl) {
 
   return new Promise(resolve => {
     Downloads.getList(Downloads.PUBLIC)
-      .then(function(aList) {
+      .then(function (aList) {
         // Temporarily register a view that will get notified when the download we
         // are controlling becomes visible in the list of downloads.
         aList
@@ -513,7 +457,7 @@ function promiseStartExternalHelperAppServiceDownload(aSourceUrl) {
 function promiseDownloadMidway(aDownload) {
   return new Promise(resolve => {
     // Wait for the download to reach half of its progress.
-    let onchange = function() {
+    let onchange = function () {
       if (
         !aDownload.stopped &&
         !aDownload.canceled &&
@@ -531,6 +475,64 @@ function promiseDownloadMidway(aDownload) {
   });
 }
 
+/**
+ * Waits for a download to make any amount of progress.
+ *
+ * @param aDownload
+ *        The Download object to wait upon.
+ *
+ * @return {Promise}
+ * @resolves When the download has transfered any number of bytes.
+ * @rejects Never.
+ */
+function promiseDownloadStarted(aDownload) {
+  return new Promise(resolve => {
+    // Wait for the download to transfer some amount of bytes.
+    let onchange = function () {
+      if (
+        !aDownload.stopped &&
+        !aDownload.canceled &&
+        aDownload.currentBytes > 0
+      ) {
+        aDownload.onchange = null;
+        resolve();
+      }
+    };
+
+    // Register for the notification, but also call the function directly in
+    // case the download already reached the expected progress.
+    aDownload.onchange = onchange;
+    onchange();
+  });
+}
+
+/**
+ * Waits for a download to finish.
+ *
+ * @param aDownload
+ *        The Download object to wait upon.
+ *
+ * @return {Promise}
+ * @resolves When the download succeeded or errored.
+ * @rejects Never.
+ */
+function promiseDownloadFinished(aDownload) {
+  return new Promise(resolve => {
+    // Wait for the download to finish.
+    let onchange = function () {
+      if (aDownload.succeeded || aDownload.error) {
+        aDownload.onchange = null;
+        resolve();
+      }
+    };
+
+    // Register for the notification, but also call the function directly in
+    // case the download already reached the expected progress.
+    aDownload.onchange = onchange;
+    onchange();
+  });
+}
+
 /**
  * Waits for a download to finish, in case it has not finished already.
  *
@@ -589,43 +591,193 @@ function promiseNewList(aIsPrivate) {
  * @resolves When the operation completes.
  * @rejects Never.
  */
-function promiseVerifyContents(aPath, aExpectedContents) {
-  return (async function() {
-    let file = new FileUtils.File(aPath);
+async function promiseVerifyContents(aPath, aExpectedContents) {
+  let file = new FileUtils.File(aPath);
 
-    if (!(await OS.File.exists(aPath))) {
-      do_throw("File does not exist: " + aPath);
-    }
+  if (!(await IOUtils.exists(aPath))) {
+    do_throw("File does not exist: " + aPath);
+  }
 
-    if ((await OS.File.stat(aPath)).size == 0) {
-      do_throw("File is empty: " + aPath);
-    }
+  if ((await IOUtils.stat(aPath)).size == 0) {
+    do_throw("File is empty: " + aPath);
+  }
 
-    await new Promise(resolve => {
-      NetUtil.asyncFetch(
-        { uri: NetUtil.newURI(file), loadUsingSystemPrincipal: true },
-        function(aInputStream, aStatus) {
-          Assert.ok(Components.isSuccessCode(aStatus));
-          let contents = NetUtil.readInputStreamToString(
-            aInputStream,
-            aInputStream.available()
-          );
-          if (
-            contents.length > TEST_DATA_SHORT.length * 2 ||
-            /[^\x20-\x7E]/.test(contents)
-          ) {
-            // Do not print the entire content string to the test log.
-            Assert.equal(contents.length, aExpectedContents.length);
-            Assert.ok(contents == aExpectedContents);
-          } else {
-            // Print the string if it is short and made of printable characters.
-            Assert.equal(contents, aExpectedContents);
-          }
-          resolve();
+  await new Promise(resolve => {
+    NetUtil.asyncFetch(
+      { uri: NetUtil.newURI(file), loadUsingSystemPrincipal: true },
+      function (aInputStream, aStatus) {
+        Assert.ok(Components.isSuccessCode(aStatus));
+        let contents = NetUtil.readInputStreamToString(
+          aInputStream,
+          aInputStream.available()
+        );
+        if (
+          contents.length > TEST_DATA_SHORT.length * 2 ||
+          /[^\x20-\x7E]/.test(contents)
+        ) {
+          // Do not print the entire content string to the test log.
+          Assert.equal(contents.length, aExpectedContents.length);
+          Assert.ok(contents == aExpectedContents);
+        } else {
+          // Print the string if it is short and made of printable characters.
+          Assert.equal(contents, aExpectedContents);
         }
-      );
+        resolve();
+      }
+    );
+  });
+}
+
+/**
+ * Creates and starts a new download, configured to keep partial data, and
+ * returns only when the first part of "interruptible_resumable.txt" has been
+ * saved to disk.  You must call "continueResponses" to allow the interruptible
+ * request to continue.
+ *
+ * This function uses either DownloadCopySaver or DownloadLegacySaver based on
+ * the current test run.
+ *
+ * @param aOptions
+ *        An optional object used to control the behavior of this function.
+ *        You may pass an object with a subset of the following fields:
+ *        {
+ *          useLegacySaver: Boolean indicating whether to launch a legacy download.
+ *        }
+ *
+ * @return {Promise}
+ * @resolves The newly created Download object, still in progress.
+ * @rejects JavaScript exception.
+ */
+async function promiseStartDownload_tryToKeepPartialData({
+  useLegacySaver = false,
+} = {}) {
+  mustInterruptResponses();
+
+  // Start a new download and configure it to keep partially downloaded data.
+  let download;
+  if (!useLegacySaver) {
+    let targetFilePath = getTempFile(TEST_TARGET_FILE_NAME).path;
+    download = await Downloads.createDownload({
+      source: httpUrl("interruptible_resumable.txt"),
+      target: {
+        path: targetFilePath,
+        partFilePath: targetFilePath + ".part",
+      },
     });
-  })();
+    download.tryToKeepPartialData = true;
+    download.start().catch(() => {});
+  } else {
+    // Start a download using nsIExternalHelperAppService, that is configured
+    // to keep partially downloaded data by default.
+    download = await promiseStartExternalHelperAppServiceDownload();
+  }
+
+  await promiseDownloadMidway(download);
+  await promisePartFileReady(download);
+
+  return download;
+}
+
+/**
+ * This function should be called after the progress notification for a download
+ * is received, and waits for the worker thread of BackgroundFileSaver to
+ * receive the data to be written to the ".part" file on disk.
+ *
+ * @return {Promise}
+ * @resolves When the ".part" file has been written to disk.
+ * @rejects JavaScript exception.
+ */
+async function promisePartFileReady(aDownload) {
+  // We don't have control over the file output code in BackgroundFileSaver.
+  // After we receive the download progress notification, we may only check
+  // that the ".part" file has been created, while its size cannot be
+  // determined because the file is currently open.
+  try {
+    do {
+      await promiseTimeout(50);
+    } while (!(await IOUtils.exists(aDownload.target.partFilePath)));
+  } catch (ex) {
+    if (!DOMException.isInstance(ex)) {
+      throw ex;
+    }
+    // This indicates that the file has been created and cannot be accessed.
+    // The specific error might vary with the platform.
+    info("IOUtils exception while checking existence: " + ex.name);
+    // Wait some more time to allow the write to complete.
+    await promiseTimeout(100);
+  }
+}
+
+/**
+ * Create a download which will be reputation blocked.
+ *
+ * @param options
+ *        {
+ *           keepPartialData: bool,
+ *           keepBlockedData: bool,
+ *           useLegacySaver: bool,
+ *           verdict: nsIApplicationReputationService value indicating the reason for the block,
+ *           expectedError: Downloads.Error value indicating the expected error,
+ *        }
+ * @return {Promise}
+ * @resolves The reputation blocked download.
+ * @rejects JavaScript exception.
+ */
+async function promiseBlockedDownload({
+  keepPartialData,
+  keepBlockedData,
+  useLegacySaver,
+  verdict = Ci.nsIApplicationReputationService.VERDICT_UNCOMMON,
+  expectedError = Downloads.Error.BLOCK_VERDICT_UNCOMMON,
+} = {}) {
+  let blockFn = () => ({
+    shouldBlockForReputationCheck: () =>
+      Promise.resolve({
+        shouldBlock: true,
+        verdict,
+      }),
+    shouldKeepBlockedData: () => Promise.resolve(keepBlockedData),
+  });
+
+  Integration.downloads.register(blockFn);
+  function cleanup() {
+    Integration.downloads.unregister(blockFn);
+  }
+  registerCleanupFunction(cleanup);
+
+  let download;
+
+  try {
+    if (keepPartialData) {
+      download = await promiseStartDownload_tryToKeepPartialData({
+        useLegacySaver,
+      });
+      continueResponses();
+    } else if (useLegacySaver) {
+      download = await promiseStartLegacyDownload();
+    } else {
+      download = await promiseNewDownload();
+      await download.start();
+      do_throw("The download should have blocked.");
+    }
+
+    await promiseDownloadStopped(download);
+    do_throw("The download should have blocked.");
+  } catch (ex) {
+    if (!(ex instanceof Downloads.Error) || !ex.becauseBlocked) {
+      throw ex;
+    }
+    Assert.ok(ex.becauseBlockedByReputationCheck);
+    Assert.equal(ex.reputationCheckVerdict, expectedError);
+    Assert.ok(download.error.becauseBlockedByReputationCheck);
+    Assert.equal(download.error.reputationCheckVerdict, expectedError);
+  }
+
+  Assert.ok(download.stopped);
+  Assert.ok(!download.succeeded);
+
+  cleanup();
+  return download;
 }
 
 /**
@@ -648,7 +800,7 @@ function startFakeServer() {
 /**
  * This is an internal reference that should not be used directly by tests.
  */
-var _gDeferResponses = Promise.defer();
+var _gDeferResponses = Promise.withResolvers();
 
 /**
  * Ensures that all the interruptible requests started after this function is
@@ -676,7 +828,7 @@ function mustInterruptResponses() {
   _gDeferResponses.resolve();
 
   info("Interruptible responses will be blocked midway.");
-  _gDeferResponses = Promise.defer();
+  _gDeferResponses = Promise.withResolvers();
 }
 
 /**
@@ -700,7 +852,7 @@ function continueResponses() {
  *        the server, when the continueResponses function is called.
  */
 function registerInterruptibleHandler(aPath, aFirstPartFn, aSecondPartFn) {
-  gHttpServer.registerPathHandler(aPath, function(aRequest, aResponse) {
+  gHttpServer.registerPathHandler(aPath, function (aRequest, aResponse) {
     info("Interruptible request started.");
 
     // Process the first part of the response.
@@ -714,7 +866,7 @@ function registerInterruptibleHandler(aPath, aFirstPartFn, aSecondPartFn) {
         aResponse.finish();
         info("Interruptible request finished.");
       })
-      .catch(Cu.reportError);
+      .catch(console.error);
   });
 }
 
@@ -728,16 +880,41 @@ function isValidDate(aDate) {
   return aDate && aDate.getTime && !isNaN(aDate.getTime());
 }
 
+/**
+ * Check actual ReferrerInfo is the same as expected.
+ * Because the actual download's referrer info's computedReferrer is computed
+ * from referrerPolicy and originalReferrer and is non-null, and the expected
+ * referrer info was constructed in isolation and therefore the computedReferrer
+ * is null, it isn't possible to use equals here. */
+function checkEqualReferrerInfos(aActualInfo, aExpectedInfo) {
+  Assert.equal(
+    !!aExpectedInfo.originalReferrer,
+    !!aActualInfo.originalReferrer
+  );
+  if (aExpectedInfo.originalReferrer && aActualInfo.originalReferrer) {
+    Assert.equal(
+      aExpectedInfo.originalReferrer.spec,
+      aActualInfo.originalReferrer.spec
+    );
+  }
+
+  Assert.equal(aExpectedInfo.sendReferrer, aActualInfo.sendReferrer);
+  Assert.equal(aExpectedInfo.referrerPolicy, aActualInfo.referrerPolicy);
+}
+
 /**
  * Waits for the download annotations to be set for the given page, required
  * because the addDownload method will add these to the database asynchronously.
  */
-function waitForAnnotation(sourceUriSpec, annotationName) {
+function waitForAnnotation(sourceUriSpec, annotationName, optionalValue) {
   return TestUtils.waitForCondition(async () => {
     let pageInfo = await PlacesUtils.history.fetch(sourceUriSpec, {
       includeAnnotations: true,
     });
-    return pageInfo && pageInfo.annotations.has(annotationName);
+    if (optionalValue) {
+      return pageInfo?.annotations.get(annotationName) == optionalValue;
+    }
+    return pageInfo?.annotations.has(annotationName);
   }, `Should have found annotation ${annotationName} for ${sourceUriSpec}`);
 }
 
@@ -749,7 +926,7 @@ var gMostRecentFirstBytePos;
 
 // Initialization functions common to all tests
 
-add_task(function test_common_initialize() {
+add_setup(function test_common_initialize() {
   // Start the HTTP server.
   gHttpServer = new HttpServer();
   gHttpServer.registerDirectory("/", do_get_file("../data"));
@@ -770,7 +947,7 @@ add_task(function test_common_initialize() {
     gHttpServer.identity.primaryPort
   );
   Services.prefs.setCharPref("network.dns.localDomains", "www.example.com");
-  registerCleanupFunction(function() {
+  registerCleanupFunction(function () {
     Services.prefs.clearUserPref("network.dns.localDomains");
   });
 
@@ -778,11 +955,22 @@ add_task(function test_common_initialize() {
   // this may block tests that use the interruptible handlers.
   Services.prefs.setBoolPref("browser.cache.disk.enable", false);
   Services.prefs.setBoolPref("browser.cache.memory.enable", false);
-  registerCleanupFunction(function() {
+  registerCleanupFunction(function () {
     Services.prefs.clearUserPref("browser.cache.disk.enable");
     Services.prefs.clearUserPref("browser.cache.memory.enable");
   });
 
+  // Allow relaxing default referrer.
+  Services.prefs.setBoolPref(
+    "network.http.referer.disallowCrossSiteRelaxingDefault",
+    false
+  );
+  registerCleanupFunction(function () {
+    Services.prefs.clearUserPref(
+      "network.http.referer.disallowCrossSiteRelaxingDefault"
+    );
+  });
+
   registerInterruptibleHandler(
     "/interruptible.txt",
     function firstPart(aRequest, aResponse) {
@@ -799,6 +987,17 @@ add_task(function test_common_initialize() {
     }
   );
 
+  registerInterruptibleHandler(
+    "/interruptible_nosize.txt",
+    function firstPart(aRequest, aResponse) {
+      aResponse.setHeader("Content-Type", "text/plain", false);
+      aResponse.write(TEST_DATA_SHORT);
+    },
+    function secondPart(aRequest, aResponse) {
+      aResponse.write(TEST_DATA_SHORT);
+    }
+  );
+
   registerInterruptibleHandler(
     "/interruptible_resumable.txt",
     function firstPart(aRequest, aResponse) {
@@ -862,23 +1061,17 @@ add_task(function test_common_initialize() {
       );
 
       let bos = new BinaryOutputStream(aResponse.bodyOutputStream);
-      bos.writeByteArray(
-        TEST_DATA_SHORT_GZIP_ENCODED_FIRST,
-        TEST_DATA_SHORT_GZIP_ENCODED_FIRST.length
-      );
+      bos.writeByteArray(TEST_DATA_SHORT_GZIP_ENCODED_FIRST);
     },
     function secondPart(aRequest, aResponse) {
       let bos = new BinaryOutputStream(aResponse.bodyOutputStream);
-      bos.writeByteArray(
-        TEST_DATA_SHORT_GZIP_ENCODED_SECOND,
-        TEST_DATA_SHORT_GZIP_ENCODED_SECOND.length
-      );
+      bos.writeByteArray(TEST_DATA_SHORT_GZIP_ENCODED_SECOND);
     }
   );
 
   gHttpServer.registerPathHandler(
     "/shorter-than-content-length-http-1-1.txt",
-    function(aRequest, aResponse) {
+    function (aRequest, aResponse) {
       aResponse.processAsync();
       aResponse.setStatusLine("1.1", 200, "OK");
       aResponse.setHeader("Content-Type", "text/plain", false);
@@ -892,83 +1085,93 @@ add_task(function test_common_initialize() {
     }
   );
 
-  // This URL will emulate being blocked by Windows Parental controls
-  gHttpServer.registerPathHandler("/parentalblocked.zip", function(
-    aRequest,
-    aResponse
-  ) {
-    aResponse.setStatusLine(
-      aRequest.httpVersion,
-      450,
-      "Blocked by Windows Parental Controls"
-    );
+  gHttpServer.registerPathHandler("/busy.txt", function (aRequest, aResponse) {
+    aResponse.setStatusLine("1.1", 504, "Gateway Timeout");
+    aResponse.setHeader("Content-Type", "text/plain", false);
+    aResponse.setHeader("Content-Length", "" + TEST_DATA_SHORT.length, false);
+    aResponse.write(TEST_DATA_SHORT);
   });
 
-  // This URL sends some data followed by an RST packet
-  gHttpServer.registerPathHandler("/netreset.txt", function(
-    aRequest,
-    aResponse
-  ) {
-    info("Starting response that will be aborted.");
-    aResponse.processAsync();
-    aResponse.setHeader("Content-Type", "text/plain", false);
-    aResponse.write(TEST_DATA_SHORT);
-    promiseExecuteSoon()
-      .then(() => {
-        aResponse.abort(null, true);
-        aResponse.finish();
-        info("Aborting response with network reset.");
-      })
-      .then(null, Cu.reportError);
+  gHttpServer.registerPathHandler("/redirect", function (aRequest, aResponse) {
+    aResponse.setStatusLine("1.1", 301, "Moved Permanently");
+    aResponse.setHeader("Location", httpUrl("busy.txt"), false);
+    aResponse.setHeader("Content-Type", "text/javascript", false);
+    aResponse.setHeader("Content-Length", "0", false);
   });
 
+  // This URL will emulate being blocked by Windows Parental controls
+  gHttpServer.registerPathHandler(
+    "/parentalblocked.zip",
+    function (aRequest, aResponse) {
+      aResponse.setStatusLine(
+        aRequest.httpVersion,
+        450,
+        "Blocked by Windows Parental Controls"
+      );
+    }
+  );
+
+  // This URL sends some data followed by an RST packet
+  gHttpServer.registerPathHandler(
+    "/netreset.txt",
+    function (aRequest, aResponse) {
+      info("Starting response that will be aborted.");
+      aResponse.processAsync();
+      aResponse.setHeader("Content-Type", "text/plain", false);
+      aResponse.write(TEST_DATA_SHORT);
+      promiseExecuteSoon()
+        .then(() => {
+          aResponse.abort(null, true);
+          aResponse.finish();
+          info("Aborting response with network reset.");
+        })
+        .then(null, console.error);
+    }
+  );
+
   // During unit tests, most of the functions that require profile access or
   // operating system features will be disabled. Individual tests may override
   // them again to check for specific behaviors.
-  Integration.downloads.register(base => ({
-    __proto__: base,
-    loadPublicDownloadListFromStore: () => Promise.resolve(),
-    shouldKeepBlockedData: () => Promise.resolve(false),
-    shouldBlockForParentalControls: () => Promise.resolve(false),
-    shouldBlockForRuntimePermissions: () => Promise.resolve(false),
-    shouldBlockForReputationCheck: () =>
-      Promise.resolve({
-        shouldBlock: false,
-        verdict: "",
-      }),
-    confirmLaunchExecutable: () => Promise.resolve(),
-    launchFile: () => Promise.resolve(),
-    showContainingDirectory: () => Promise.resolve(),
-    // This flag allows re-enabling the default observers during their tests.
-    allowObservers: false,
-    addListObservers() {
-      return this.allowObservers
-        ? super.addListObservers(...arguments)
-        : Promise.resolve();
-    },
-    // This flag allows re-enabling the download directory logic for its tests.
-    _allowDirectories: false,
-    set allowDirectories(value) {
-      this._allowDirectories = value;
-      // We have to invalidate the previously computed directory path.
-      this._downloadsDirectory = null;
-    },
-    _getDirectory(name) {
-      return super._getDirectory(this._allowDirectories ? name : "TmpD");
-    },
-  }));
+  Integration.downloads.register(base => {
+    let override = {
+      loadPublicDownloadListFromStore: () => Promise.resolve(),
+      shouldKeepBlockedData: () => Promise.resolve(false),
+      shouldBlockForParentalControls: () => Promise.resolve(false),
+      shouldBlockForReputationCheck: () =>
+        Promise.resolve({
+          shouldBlock: false,
+          verdict: Ci.nsIApplicationReputationService.VERDICT_SAFE,
+        }),
+      confirmLaunchExecutable: () => Promise.resolve(),
+      launchFile: () => Promise.resolve(),
+      showContainingDirectory: () => Promise.resolve(),
+      // This flag allows re-enabling the default observers during their tests.
+      allowObservers: false,
+      addListObservers() {
+        return this.allowObservers
+          ? super.addListObservers(...arguments)
+          : Promise.resolve();
+      },
+      // This flag allows re-enabling the download directory logic for its tests.
+      _allowDirectories: false,
+      set allowDirectories(value) {
+        this._allowDirectories = value;
+        // We have to invalidate the previously computed directory path.
+        this._downloadsDirectory = null;
+      },
+      _getDirectory(name) {
+        return super._getDirectory(this._allowDirectories ? name : "TmpD");
+      },
+    };
+    Object.setPrototypeOf(override, base);
+    return override;
+  });
 
   // Make sure that downloads started using nsIExternalHelperAppService are
   // saved to disk without asking for a destination interactively.
   let mock = {
-    QueryInterface: ChromeUtils.generateQI([Ci.nsIHelperAppLauncherDialog]),
-    promptForSaveToFileAsync(
-      aLauncher,
-      aWindowContext,
-      aDefaultFileName,
-      aSuggestedFileExtension,
-      aForcePrompt
-    ) {
+    QueryInterface: ChromeUtils.generateQI(["nsIHelperAppLauncherDialog"]),
+    promptForSaveToFileAsync(aLauncher) {
       // The dialog should create the empty placeholder file.
       let file = getTempFile(TEST_TARGET_FILE_NAME);
       file.create(Ci.nsIFile.NORMAL_FILE_TYPE, FileUtils.PERMS_FILE);
