@@ -31,23 +31,25 @@ add_task(async function test_support_theme_frame() {
   );
 
   Assert.equal(
-    docEl.getAttribute("lwthemetextcolor"),
-    "dark",
-    "LWT text color attribute should be set"
+    docEl.getAttribute("lwtheme-brighttext"),
+    null,
+    "LWT text color attribute should not be set"
   );
 
-  let style = window.getComputedStyle(docEl);
+  let toolbox = document.querySelector("#navigator-toolbox");
+  let toolboxCS = window.getComputedStyle(toolbox);
+
   Assert.ok(
-    style.backgroundImage.includes("face.png"),
-    `The backgroundImage should use face.png. Actual value is: ${style.backgroundImage}`
+    toolboxCS.backgroundImage.includes("face.png"),
+    `The backgroundImage should use face.png. Actual value is: ${toolboxCS.backgroundImage}`
   );
   Assert.equal(
-    style.backgroundColor,
+    getToolboxBackgroundColor(),
     "rgb(" + FRAME_COLOR.join(", ") + ")",
     "Expected correct background color"
   );
   Assert.equal(
-    style.color,
+    toolboxCS.color,
     "rgb(" + TAB_TEXT_COLOR.join(", ") + ")",
     "Expected correct text color"
   );
@@ -62,7 +64,7 @@ add_task(async function test_support_theme_frame() {
   );
 
   Assert.ok(
-    !docEl.hasAttribute("lwthemetextcolor"),
+    !docEl.hasAttribute("lwtheme-brighttext"),
     "LWT text color attribute should not be set"
   );
 });
@@ -92,10 +94,9 @@ add_task(async function test_support_theme_frame_inactive() {
   await extension.startup();
 
   let docEl = window.document.documentElement;
-  let style = window.getComputedStyle(docEl);
 
   Assert.equal(
-    style.backgroundColor,
+    getToolboxBackgroundColor(),
     "rgb(" + FRAME_COLOR.join(", ") + ")",
     "Window background is set to the colors.frame property"
   );
@@ -103,7 +104,7 @@ add_task(async function test_support_theme_frame_inactive() {
   // Now we'll open a new window to see if the inactive browser accent color changed
   let window2 = await BrowserTestUtils.openNewBrowserWindow();
   Assert.equal(
-    style.backgroundColor,
+    getToolboxBackgroundColor(),
     "rgb(" + FRAME_COLOR_INACTIVE.join(", ") + ")",
     `Inactive window background color should be ${FRAME_COLOR_INACTIVE}`
   );
@@ -137,19 +138,16 @@ add_task(async function test_lack_of_theme_frame_inactive() {
   await extension.startup();
 
   let docEl = window.document.documentElement;
-  let style = window.getComputedStyle(docEl);
-
   Assert.equal(
-    style.backgroundColor,
+    getToolboxBackgroundColor(),
     "rgb(" + FRAME_COLOR.join(", ") + ")",
     "Window background is set to the colors.frame property"
   );
 
   // Now we'll open a new window to make sure the inactive browser accent color stayed the same
   let window2 = await BrowserTestUtils.openNewBrowserWindow();
-
   Assert.equal(
-    style.backgroundColor,
+    getToolboxBackgroundColor(),
     "rgb(" + FRAME_COLOR.join(", ") + ")",
     "Inactive window background should not change if colors.frame_inactive isn't set"
   );
