@@ -4,8 +4,6 @@
 
 /* import-globals-from head.js */
 
-const STORAGE_SYNC_PREF = "webextensions.storage.sync.enabled";
-
 // Test implementations and utility functions that are used against multiple
 // storage areas (eg, a test which is run against browser.storage.local and
 // browser.storage.sync, or a test against browser.storage.sync but needs to
@@ -19,7 +17,7 @@ const STORAGE_SYNC_PREF = "webextensions.storage.sync.enabled";
  *        either "local" or "sync" according to what we want to test
  * @param {string} prop
  *        "key" to look up using the storage API
- * @param {Object} value
+ * @param {object} value
  *        "value" to compare against
  */
 async function checkGetImpl(areaName, prop, value) {
@@ -76,91 +74,58 @@ async function checkGetImpl(areaName, prop, value) {
   );
 }
 
-function test_config_flag_needed() {
-  async function testFn() {
-    function background() {
-      let promises = [];
-      let apiTests = [
-        { method: "get", args: ["foo"] },
-        { method: "set", args: [{ foo: "bar" }] },
-        { method: "remove", args: ["foo"] },
-        { method: "clear", args: [] },
-      ];
-      apiTests.forEach(testDef => {
-        promises.push(
-          browser.test.assertRejects(
-            browser.storage.sync[testDef.method](...testDef.args),
-            "Please set webextensions.storage.sync.enabled to true in about:config",
-            `storage.sync.${testDef.method} is behind a flag`
-          )
-        );
-      });
+async function test_storage_after_reload(areaName, { expectPersistency }) {
+  // Just some random extension ID that we can re-use
+  const extensionId = "my-extension-id@1";
 
-      Promise.all(promises).then(() => browser.test.notifyPass("flag needed"));
+  function loadExtension() {
+    async function background(areaName) {
+      browser.test.sendMessage(
+        "initialItems",
+        await browser.storage[areaName].get(null)
+      );
+      await browser.storage[areaName].set({ a: "b" });
+      browser.test.notifyPass("set-works");
     }
 
-    ok(
-      !Services.prefs.getBoolPref(STORAGE_SYNC_PREF, false),
-      "The `${STORAGE_SYNC_PREF}` should be set to false"
-    );
-
-    let extension = ExtensionTestUtils.loadExtension({
+    return ExtensionTestUtils.loadExtension({
       manifest: {
+        browser_specific_settings: { gecko: { id: extensionId } },
         permissions: ["storage"],
       },
-      background,
+      background: `(${background})("${areaName}")`,
     });
-
-    await extension.startup();
-    await extension.awaitFinish("flag needed");
-    await extension.unload();
   }
 
-  return runWithPrefs([[STORAGE_SYNC_PREF, false]], testFn);
+  let extension1 = loadExtension();
+  await extension1.startup();
+
+  Assert.deepEqual(
+    await extension1.awaitMessage("initialItems"),
+    {},
+    "No stored items at first"
+  );
+
+  await extension1.awaitFinish("set-works");
+  await extension1.unload();
+
+  let extension2 = loadExtension();
+  await extension2.startup();
+
+  Assert.deepEqual(
+    await extension2.awaitMessage("initialItems"),
+    expectPersistency ? { a: "b" } : {},
+    `Expect ${areaName} stored items ${
+      expectPersistency ? "to" : "not"
+    } be available after restart`
+  );
+
+  await extension2.awaitFinish("set-works");
+  await extension2.unload();
 }
 
-function test_sync_reloading_extensions_works() {
-  async function testFn() {
-    // Just some random extension ID that we can re-use
-    const extensionId = "my-extension-id@1";
-
-    function loadExtension() {
-      function background() {
-        browser.storage.sync.set({ a: "b" }).then(() => {
-          browser.test.notifyPass("set-works");
-        });
-      }
-
-      return ExtensionTestUtils.loadExtension(
-        {
-          manifest: {
-            permissions: ["storage"],
-          },
-          background: `(${background})()`,
-        },
-        extensionId
-      );
-    }
-
-    ok(
-      Services.prefs.getBoolPref(STORAGE_SYNC_PREF, false),
-      "The `${STORAGE_SYNC_PREF}` should be set to true"
-    );
-
-    let extension1 = loadExtension();
-
-    await extension1.startup();
-    await extension1.awaitFinish("set-works");
-    await extension1.unload();
-
-    let extension2 = loadExtension();
-
-    await extension2.startup();
-    await extension2.awaitFinish("set-works");
-    await extension2.unload();
-  }
-
-  return runWithPrefs([[STORAGE_SYNC_PREF, true]], testFn);
+async function test_sync_reloading_extensions_works() {
+  await test_storage_after_reload("sync", { expectPersistency: true });
 }
 
 async function test_background_page_storage(testAreaName) {
@@ -302,7 +267,7 @@ async function test_background_page_storage(testAreaName) {
       this.testKey1 = "testValue1";
     }
 
-    CustomObj.prototype.toString = function() {
+    CustomObj.prototype.toString = function () {
       return '{"testKey2":"testValue2"}';
     };
 
@@ -519,7 +484,7 @@ async function test_background_page_storage(testAreaName) {
           "customObj keys correct"
         );
 
-        if (areaName === "local") {
+        if (areaName === "local" || areaName === "session") {
           browser.test.assertEq(
             String(date),
             String(obj.date),
@@ -530,7 +495,7 @@ async function test_background_page_storage(testAreaName) {
             obj.regexp.toString(),
             "regexp part correct"
           );
-          // storage.local doesn't call toJSON
+          // storage.local and .session don't use toJSON().
           browser.test.assertEq(
             "testValue1",
             obj.customObj.testKey1,
@@ -658,6 +623,8 @@ async function test_background_page_storage(testAreaName) {
         promise = runTests("local");
       } else if (msg === "test-sync") {
         promise = runTests("sync");
+      } else if (msg === "test-session") {
+        promise = runTests("session");
       }
       promise.then(() => browser.test.sendMessage("test-finished"));
     });
@@ -682,13 +649,13 @@ async function test_background_page_storage(testAreaName) {
   await extension.unload();
 }
 
-function test_storage_sync_requires_real_id() {
+async function test_storage_sync_requires_real_id() {
   async function testFn() {
     async function background() {
       const EXCEPTION_MESSAGE =
-        "The storage API is not available with a temporary addon ID. " +
+        "The storage API will not work with a temporary addon ID. " +
         "Please add an explicit addon ID to your manifest. " +
-        "For more information see https://bugzil.la/1323228.";
+        "For more information see https://mzl.la/3lPk1aE.";
 
       await browser.test.assertRejects(
         browser.storage.sync.set({ foo: "bar" }),
@@ -713,7 +680,7 @@ function test_storage_sync_requires_real_id() {
     await extension.unload();
   }
 
-  return runWithPrefs([[STORAGE_SYNC_PREF, true]], testFn);
+  return await testFn();
 }
 
 // Test for storage areas which don't support getBytesInUse() nor QUOTA
@@ -735,7 +702,7 @@ async function test_background_storage_area_no_bytes_in_use(area) {
   const extensionDef = {
     manifest: {
       permissions: ["storage"],
-      applications: { gecko: { id: EXT_ID } },
+      browser_specific_settings: { gecko: { id: EXT_ID } },
     },
     background: `(${check_storage_area_no_bytes_in_use})("${area}")`,
   };
@@ -758,6 +725,8 @@ async function test_contentscript_storage_area_no_bytes_in_use(area) {
         checkImpl("local");
       } else if (msg === "test-sync") {
         checkImpl("sync");
+      } else if (msg === "test-session") {
+        checkImpl("session");
       } else {
         browser.test.fail(`Unexpected test message received: ${msg}`);
         browser.test.sendMessage("test-complete");
@@ -860,7 +829,7 @@ async function test_background_storage_area_with_bytes_in_use(
   const extensionDef = {
     manifest: {
       permissions: ["storage"],
-      applications: { gecko: { id: EXT_ID } },
+      browser_specific_settings: { gecko: { id: EXT_ID } },
     },
     background: `(${check_storage_area_with_bytes_in_use})("${area}", ${expectQuota})`,
   };
@@ -1185,6 +1154,8 @@ async function testStorageContentScript(checkGet) {
       promise = runTests("local");
     } else if (msg === "test-sync") {
       promise = runTests("sync");
+    } else if (msg === "test-session") {
+      promise = runTests("session");
     }
     promise.then(() => browser.test.sendMessage("test-finished"));
   });
@@ -1225,3 +1196,208 @@ async function test_contentscript_storage(storageType) {
   await extension.unload();
   await contentPage.close();
 }
+
+async function test_storage_empty_events(areaName) {
+  async function background(areaName) {
+    let eventCount = 0;
+
+    browser.storage[areaName].onChanged.addListener(changes => {
+      browser.test.sendMessage("onChanged", [++eventCount, changes]);
+    });
+
+    browser.test.onMessage.addListener(async (method, arg) => {
+      let result = await browser.storage[areaName][method](arg);
+      browser.test.sendMessage("result", result);
+    });
+
+    browser.test.sendMessage("ready");
+  }
+
+  let extension = ExtensionTestUtils.loadExtension({
+    manifest: { permissions: ["storage"] },
+    background: `(${background})("${areaName}")`,
+  });
+
+  await extension.startup();
+  await extension.awaitMessage("ready");
+
+  async function callStorageMethod(method, arg) {
+    info(`call storage.${areaName}.${method}(${JSON.stringify(arg) ?? ""})`);
+    extension.sendMessage(method, arg);
+    await extension.awaitMessage("result");
+  }
+
+  async function expectEvent(expectCount, expectChanges) {
+    equal(
+      JSON.stringify([expectCount, expectChanges]),
+      JSON.stringify(await extension.awaitMessage("onChanged")),
+      "Correct onChanged events count and data in the last changes notified."
+    );
+  }
+
+  await callStorageMethod("set", { alpha: 1 });
+  await expectEvent(1, { alpha: { newValue: 1 } });
+
+  await callStorageMethod("set", {});
+  // Setting nothing doesn't trigger onChanged event.
+
+  await callStorageMethod("set", { beta: 12 });
+  await expectEvent(2, { beta: { newValue: 12 } });
+
+  await callStorageMethod("remove", "alpha");
+  await expectEvent(3, { alpha: { oldValue: 1 } });
+
+  await callStorageMethod("remove", "alpha");
+  // Trying to remove alpha again doesn't trigger onChanged.
+
+  await callStorageMethod("clear");
+  await expectEvent(4, { beta: { oldValue: 12 } });
+
+  await callStorageMethod("clear");
+  // Clear again wothout onChanged. Test will fail on unexpected event/message.
+
+  await extension.unload();
+}
+
+async function test_storage_change_event_page(areaName) {
+  async function testOnChanged(targetIsStorageArea) {
+    function backgroundTestStorageTopNamespace(areaName) {
+      browser.storage.onChanged.addListener((changes, area) => {
+        browser.test.assertEq(area, areaName, "Expected areaName");
+        browser.test.assertEq(
+          JSON.stringify(changes),
+          `{"storageKey":{"newValue":"newStorageValue"}}`,
+          "Expected changes"
+        );
+        browser.test.sendMessage("onChanged_was_fired");
+      });
+    }
+    function backgroundTestStorageAreaNamespace(areaName) {
+      browser.storage[areaName].onChanged.addListener((changes, ...args) => {
+        browser.test.assertEq(args.length, 0, "no more args after changes");
+        browser.test.assertEq(
+          JSON.stringify(changes),
+          `{"storageKey":{"newValue":"newStorageValue"}}`,
+          `Expected changes via ${areaName}.onChanged event`
+        );
+        browser.test.sendMessage("onChanged_was_fired");
+      });
+    }
+    let background, onChangedName;
+    if (targetIsStorageArea) {
+      // Test storage.local.onChanged / storage.sync.onChanged.
+      background = backgroundTestStorageAreaNamespace;
+      onChangedName = `${areaName}.onChanged`;
+    } else {
+      background = backgroundTestStorageTopNamespace;
+      onChangedName = "onChanged";
+    }
+    let extension = ExtensionTestUtils.loadExtension({
+      manifest: {
+        permissions: ["storage"],
+        background: { persistent: false },
+      },
+      background: `(${background})("${areaName}")`,
+      files: {
+        "trigger-change.html": `
+          <!DOCTYPE html><meta charset="utf-8">
+          <script src="trigger-change.js"></script>
+        `,
+        "trigger-change.js": async () => {
+          let areaName = location.search.slice(1);
+          await browser.storage[areaName].set({
+            storageKey: "newStorageValue",
+          });
+          browser.test.sendMessage("tried_to_trigger_change");
+        },
+      },
+    });
+    await extension.startup();
+    assertPersistentListeners(extension, "storage", onChangedName, {
+      primed: false,
+    });
+
+    await extension.terminateBackground();
+    assertPersistentListeners(extension, "storage", onChangedName, {
+      primed: true,
+    });
+
+    // Now trigger the event
+    let contentPage = await ExtensionTestUtils.loadContentPage(
+      `moz-extension://${extension.uuid}/trigger-change.html?${areaName}`
+    );
+    await extension.awaitMessage("tried_to_trigger_change");
+    await contentPage.close();
+    await extension.awaitMessage("onChanged_was_fired");
+
+    assertPersistentListeners(extension, "storage", onChangedName, {
+      primed: false,
+    });
+    await extension.unload();
+  }
+
+  async function testFn() {
+    // Test browser.storage.onChanged.addListener
+    await testOnChanged(/* targetIsStorageArea */ false);
+    // Test browser.storage.local.onChanged.addListener
+    // and browser.storage.sync.onChanged.addListener, depending on areaName.
+    await testOnChanged(/* targetIsStorageArea */ true);
+  }
+
+  return runWithPrefs([["extensions.eventPages.enabled", true]], testFn);
+}
+
+async function test_storage_sync_telemetry_quota(backend, enforced = false) {
+  Services.fog.testResetFOG();
+  let id = "my-extension-id@23";
+
+  // Repeat twice to get the glean events "before" and "after".
+  for (let i = 0; i < 2; i++) {
+    let ext = ExtensionTestUtils.loadExtension({
+      manifest: {
+        browser_specific_settings: { gecko: { id: id } },
+        permissions: ["storage"],
+      },
+      background() {
+        browser.test.onMessage.addListener(async enforced => {
+          await browser.storage.sync.set({
+            a: "1",
+            b: "x".repeat(enforced ? 1_000 : 10_000),
+          });
+          browser.test.notifyPass("done");
+        });
+      },
+    });
+
+    await ext.startup();
+    ext.sendMessage(enforced);
+    await ext.awaitFinish("done");
+    await ext.unload();
+  }
+
+  let events = Glean.extensionsData.syncUsageQuotas.testGetValue();
+  events = events.filter(e => e.extra?.addon_id === id);
+
+  Assert.deepEqual(
+    events[0].extra,
+    {
+      addon_id: id,
+      total_size_bytes: 0,
+      items_count: 0,
+      items_over_quota: 0,
+      backend: backend,
+    },
+    "Expected event values before setting sync storage"
+  );
+  Assert.deepEqual(
+    events[1].extra,
+    {
+      addon_id: id,
+      total_size_bytes: enforced ? 1_007 : 10_007,
+      items_count: 2,
+      items_over_quota: enforced ? 0 : 1,
+      backend: backend,
+    },
+    "Expected event values after setting sync storage"
+  );
+}
