@@ -1,22 +1,26 @@
 "use strict";
 
-do_get_profile();
+ChromeUtils.defineESModuleGetters(this, {
+  ContextualIdentityService:
+    "resource://gre/modules/ContextualIdentityService.sys.mjs",
+  ExtensionPreferencesManager:
+    "resource://gre/modules/ExtensionPreferencesManager.sys.mjs",
+  AddonManager: "resource://gre/modules/AddonManager.sys.mjs",
+});
 
-function waitForPrefChange(pref) {
-  return new Promise(resolve => {
-    function observeChange() {
-      Services.prefs.removeObserver(pref, observeChange);
-      resolve();
-    }
-
-    Services.prefs.addObserver(pref, observeChange);
-  });
-}
+const CONTAINERS_PREF = "privacy.userContext.enabled";
 
 AddonTestUtils.init(this);
+AddonTestUtils.overrideCertDB();
+AddonTestUtils.createAppInfo(
+  "xpcshell@tests.mozilla.org",
+  "XPCShell",
+  "42",
+  "42"
+);
 
 add_task(async function startup() {
-  await ExtensionTestUtils.startAddonManager();
+  await AddonTestUtils.promiseStartupManager();
 });
 
 add_task(async function test_contextualIdentities_without_permissions() {
@@ -32,7 +36,7 @@ add_task(async function test_contextualIdentities_without_permissions() {
     useAddonManager: "temporary",
     background,
     manifest: {
-      applications: {
+      browser_specific_settings: {
         gecko: { id: "testing@thing.com" },
       },
       permissions: [],
@@ -45,7 +49,6 @@ add_task(async function test_contextualIdentities_without_permissions() {
 });
 
 add_task(async function test_contextualIdentity_events() {
-  const CONTAINERS_PREF = "privacy.userContext.enabled";
   async function background() {
     function createOneTimeListener(type) {
       return new Promise((resolve, reject) => {
@@ -140,7 +143,7 @@ add_task(async function test_contextualIdentity_events() {
     background,
     useAddonManager: "temporary",
     manifest: {
-      applications: {
+      browser_specific_settings: {
         gecko: { id: "testing@thing.com" },
       },
       permissions: ["contextualIdentities"],
@@ -157,9 +160,6 @@ add_task(async function test_contextualIdentity_events() {
 });
 
 add_task(async function test_contextualIdentity_with_permissions() {
-  const CONTAINERS_PREF = "privacy.userContext.enabled";
-  const initial = Services.prefs.getBoolPref(CONTAINERS_PREF);
-
   async function background() {
     let ci;
     await browser.test.assertRejects(
@@ -340,7 +340,7 @@ add_task(async function test_contextualIdentity_with_permissions() {
       useAddonManager: "temporary",
       background,
       manifest: {
-        applications: {
+        browser_specific_settings: {
           gecko: { id },
         },
         permissions: ["contextualIdentities"],
@@ -364,23 +364,17 @@ add_task(async function test_contextualIdentity_with_permissions() {
     true,
     "Pref should now be enabled, whatever it's initial state"
   );
-  const prefChange = waitForPrefChange(CONTAINERS_PREF);
   await extension.unload();
-  if (initial === false) {
-    await prefChange;
-  }
   equal(
     Services.prefs.getBoolPref(CONTAINERS_PREF),
-    initial,
-    "Pref should now be initial state"
+    true,
+    "Pref should remain enabled"
   );
 
   Services.prefs.clearUserPref(CONTAINERS_PREF);
 });
 
 add_task(async function test_contextualIdentity_extensions_enable_containers() {
-  const CONTAINERS_PREF = "privacy.userContext.enabled";
-  const initial = Services.prefs.getBoolPref(CONTAINERS_PREF);
   async function background() {
     let ci = await browser.contextualIdentities.get("firefox-container-1");
     browser.test.assertTrue(!!ci, "We have an identity");
@@ -389,16 +383,28 @@ add_task(async function test_contextualIdentity_extensions_enable_containers() {
   }
   function makeExtension(id) {
     return ExtensionTestUtils.loadExtension({
-      useAddonManager: "temporary",
+      useAddonManager: "permanent",
       background,
       manifest: {
-        applications: {
+        browser_specific_settings: {
           gecko: { id },
         },
         permissions: ["contextualIdentities"],
       },
     });
   }
+  async function testSetting(expect, message) {
+    let setting =
+      await ExtensionPreferencesManager.getSetting("privacy.containers");
+    if (expect === null) {
+      equal(setting, null, message);
+    } else {
+      equal(setting.value, expect, message);
+    }
+  }
+  function testPref(expect, message) {
+    equal(Services.prefs.getBoolPref(CONTAINERS_PREF), expect, message);
+  }
 
   let extension = makeExtension("containers-test@mozilla.org");
   await extension.startup();
@@ -408,40 +414,45 @@ add_task(async function test_contextualIdentity_extensions_enable_containers() {
     true,
     "Pref should now be enabled, whatever it's initial state"
   );
-  const prefChange = waitForPrefChange(CONTAINERS_PREF);
   await extension.unload();
-  // If pref was false we should wait for the pref to change back here.
-  if (initial === false) {
-    await prefChange;
-  }
+  await testSetting(null, "setting should be unset");
   equal(
     Services.prefs.getBoolPref(CONTAINERS_PREF),
-    initial,
-    "Pref should now be initial state"
+    true,
+    "Pref should remain enabled"
   );
 
   // Lets set containers explicitly to be off and test we keep it that way after removal
   Services.prefs.setBoolPref(CONTAINERS_PREF, false);
+
   let extension1 = makeExtension("containers-test-1@mozilla.org");
   await extension1.startup();
   await extension1.awaitFinish("contextualIdentities");
+  await testSetting(extension1.id, "setting should be controlled");
+  testPref(true, "Pref should now be enabled, whatever it's initial state");
+
+  // Test that disabling leaves containers on, and that re-enabling with containers off
+  // will re-enable containers.
+  const addon = await AddonManager.getAddonByID(extension1.id);
+  await addon.disable();
+  await testSetting(undefined, "setting should not be an addon");
+  testPref(true, "Pref should remain enabled, whatever it's initial state");
+
+  Services.prefs.setBoolPref(CONTAINERS_PREF, false);
+
+  await addon.enable();
+  await testSetting(extension1.id, "setting should be controlled");
+  testPref(true, "Pref should be enabled");
+
+  await extension1.unload();
+  await testSetting(null, "setting should be unset");
   equal(
     Services.prefs.getBoolPref(CONTAINERS_PREF),
     true,
-    "Pref should now be enabled, whatever it's initial state"
-  );
-  const prefChange1 = waitForPrefChange(CONTAINERS_PREF);
-  await extension1.unload();
-  // We explicitly have a pref that was set off, extensions turned it on
-  // Lets wait until the pref flips back to the user set value of off
-  await prefChange1;
-  equal(
-    Services.prefs.getBoolPref(CONTAINERS_PREF),
-    false,
-    "Pref should now be disabled, whatever it's initial state"
+    "Pref should remain enabled"
   );
 
-  // Lets set containers explicitly to be on and test we keep it that way after removal
+  // Lets set containers explicitly to be on and test we keep it that way after removal.
   Services.prefs.setBoolPref(CONTAINERS_PREF, true);
 
   let extension2 = makeExtension("containers-test-2@mozilla.org");
@@ -452,29 +463,19 @@ add_task(async function test_contextualIdentity_extensions_enable_containers() {
   await extension3.awaitFinish("contextualIdentities");
 
   // Flip the ordering to check it's still enabled
-  equal(
-    Services.prefs.getBoolPref(CONTAINERS_PREF),
-    true,
-    "Pref should now be enabled 1"
-  );
+  await testSetting(extension3.id, "setting should still be controlled by 3");
+  testPref(true, "Pref should now be enabled 1");
   await extension3.unload();
-  equal(
-    Services.prefs.getBoolPref(CONTAINERS_PREF),
-    true,
-    "Pref should now be enabled 2"
-  );
+  await testSetting(extension2.id, "setting should still be controlled by 2");
+  testPref(true, "Pref should now be enabled 2");
   await extension2.unload();
-  equal(
-    Services.prefs.getBoolPref(CONTAINERS_PREF),
-    true,
-    "Pref should now be enabled 3"
-  );
+  await testSetting(null, "setting should be unset");
+  testPref(true, "Pref should now be enabled 3");
 
   Services.prefs.clearUserPref(CONTAINERS_PREF);
 });
 
 add_task(async function test_contextualIdentity_preference_change() {
-  const CONTAINERS_PREF = "privacy.userContext.enabled";
   async function background() {
     let extensionInfo = await browser.management.getSelf();
     if (extensionInfo.version == "1.0.0") {
@@ -503,7 +504,7 @@ add_task(async function test_contextualIdentity_preference_change() {
       background,
       manifest: {
         version,
-        applications: {
+        browser_specific_settings: {
           gecko: { id },
         },
         permissions: ["contextualIdentities"],
@@ -525,9 +526,81 @@ add_task(async function test_contextualIdentity_preference_change() {
   await extension2.startup();
   await extension2.awaitFinish("contextualIdentities");
 
-  const prefChange = waitForPrefChange(CONTAINERS_PREF);
   await extension.unload();
-  await prefChange;
+  equal(
+    Services.prefs.getBoolPref(CONTAINERS_PREF),
+    true,
+    "Pref should remain enabled"
+  );
 
   Services.prefs.clearUserPref(CONTAINERS_PREF);
 });
+
+add_task(
+  { pref_set: [["extensions.eventPages.enabled", true]] },
+  async function test_contextualIdentity_event_page() {
+    let extension = ExtensionTestUtils.loadExtension({
+      useAddonManager: "permanent",
+      manifest: {
+        browser_specific_settings: {
+          gecko: { id: "eventpage@mochitest" },
+        },
+        permissions: ["contextualIdentities"],
+        background: { persistent: false },
+      },
+      background() {
+        browser.contextualIdentities.onCreated.addListener(() => {
+          browser.test.sendMessage("created");
+        });
+        browser.contextualIdentities.onUpdated.addListener(() => {});
+        browser.contextualIdentities.onRemoved.addListener(() => {
+          browser.test.sendMessage("removed");
+        });
+        browser.test.sendMessage("ready");
+      },
+    });
+
+    const EVENTS = ["onCreated", "onUpdated", "onRemoved"];
+
+    await extension.startup();
+    await extension.awaitMessage("ready");
+    for (let event of EVENTS) {
+      assertPersistentListeners(extension, "contextualIdentities", event, {
+        primed: false,
+      });
+    }
+
+    await extension.terminateBackground();
+    for (let event of EVENTS) {
+      assertPersistentListeners(extension, "contextualIdentities", event, {
+        primed: true,
+      });
+    }
+
+    // test events waken background
+    let identity = ContextualIdentityService.create("foobar", "circle", "red");
+
+    await extension.awaitMessage("ready");
+    await extension.awaitMessage("created");
+    for (let event of EVENTS) {
+      assertPersistentListeners(extension, "contextualIdentities", event, {
+        primed: false,
+      });
+    }
+
+    ContextualIdentityService.remove(identity.userContextId);
+    await extension.awaitMessage("removed");
+
+    // check primed listeners after startup
+    await AddonTestUtils.promiseRestartManager();
+    await extension.awaitStartup();
+
+    for (let event of EVENTS) {
+      assertPersistentListeners(extension, "contextualIdentities", event, {
+        primed: true,
+      });
+    }
+
+    await extension.unload();
+  }
+);
