@@ -3,8 +3,6 @@
 "use strict";
 
 add_task(async function test_manifest_incognito() {
-  Services.prefs.setBoolPref("extensions.allowPrivateBrowsingByDefault", false);
-
   let normalized = await ExtensionTestUtils.normalizeManifest({
     incognito: "spanning",
   });
@@ -29,20 +27,48 @@ add_task(async function test_manifest_incognito() {
     "Should have the expected incognito string"
   );
 
+  ExtensionTestUtils.failOnSchemaWarnings(false);
   normalized = await ExtensionTestUtils.normalizeManifest({
-    incognito: "split",
+    incognito: "whatisthis",
   });
+  ExtensionTestUtils.failOnSchemaWarnings(true);
 
   equal(
     normalized.error,
-    'Error processing incognito: Invalid enumeration value "split"',
+    'Error processing incognito: Invalid enumeration value "whatisthis"',
     "Should have an error"
   );
   Assert.deepEqual(normalized.errors, [], "Should not have a warning");
+
+  // Sanity check: Default value of "incognito" is "spanning".
+  normalized = await ExtensionTestUtils.normalizeManifest({});
+  equal(normalized.error, undefined, "Should not have an error");
+  equal(normalized.errors.length, 0, "Should not have warnings");
   equal(
-    normalized.value,
-    undefined,
-    "Invalid incognito string should be undefined"
+    normalized.value.incognito,
+    "spanning",
+    "Should have the expected default value for incognito when unspecified"
+  );
+});
+
+add_task(async function test_manifest_incognito_split_fallback_not_allowed() {
+  ExtensionTestUtils.failOnSchemaWarnings(false);
+  let normalized = await ExtensionTestUtils.normalizeManifest({
+    incognito: "split",
+  });
+  ExtensionTestUtils.failOnSchemaWarnings(true);
+
+  equal(normalized.error, undefined, "Should not have an error");
+  Assert.deepEqual(
+    normalized.errors,
+    [
+      `Warning processing incognito: incognito "split" is unsupported. Falling back to incognito "not_allowed".`,
+    ],
+    "Should have a warning"
+  );
+  equal(
+    normalized.value.incognito,
+    "not_allowed",
+    "incognito:split should fall back to not_allowed by default"
   );
-  Services.prefs.clearUserPref("extensions.allowPrivateBrowsingByDefault");
 });
