@@ -7,71 +7,68 @@
 #include "nsWebBrowserFind.h"
 
 // Only need this for NS_FIND_CONTRACTID,
-// else we could use nsIDOMRange.h and nsIFind.h.
+// else we could use nsRange.h and nsIFind.h.
 #include "nsFind.h"
 
-#include "nsIComponentManager.h"
-#include "nsIScriptSecurityManager.h"
+#include "mozilla/dom/ScriptSettings.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsPIDOMWindow.h"
-#include "nsIURI.h"
 #include "nsIDocShell.h"
-#include "nsIPresShell.h"
 #include "nsPresContext.h"
-#include "nsIDocument.h"
-#include "nsIDOMDocument.h"
+#include "mozilla/dom/Document.h"
 #include "nsISelectionController.h"
-#include "nsISelection.h"
 #include "nsIFrame.h"
-#include "nsITextControlFrame.h"
 #include "nsReadableUtils.h"
 #include "nsIContent.h"
-#include "nsContentCID.h"
-#include "nsIServiceManager.h"
 #include "nsIObserverService.h"
 #include "nsISupportsPrimitives.h"
 #include "nsFind.h"
 #include "nsError.h"
 #include "nsFocusManager.h"
+#include "nsRange.h"
+#include "mozilla/PresShell.h"
 #include "mozilla/Services.h"
 #include "mozilla/dom/Element.h"
-#include "nsISimpleEnumerator.h"
+#include "mozilla/dom/Selection.h"
+#include "nsComponentManagerUtils.h"
 #include "nsContentUtils.h"
 #include "nsGenericHTMLElement.h"
 
 #if DEBUG
-#include "nsIWebNavigation.h"
-#include "nsString.h"
+#  include "nsIWebNavigation.h"
+#  include "nsString.h"
 #endif
 
-nsWebBrowserFind::nsWebBrowserFind()
-  : mFindBackwards(false)
-  , mWrapFind(false)
-  , mEntireWord(false)
-  , mMatchCase(false)
-  , mSearchSubFrames(true)
-  , mSearchParentFrames(true)
-{
-}
+using namespace mozilla;
+using mozilla::dom::Document;
+using mozilla::dom::Element;
+using mozilla::dom::Selection;
 
-nsWebBrowserFind::~nsWebBrowserFind()
-{
-}
+nsWebBrowserFind::nsWebBrowserFind()
+    : mFindBackwards(false),
+      mWrapFind(false),
+      mEntireWord(false),
+      mMatchCase(false),
+      mMatchDiacritics(false),
+      mSearchSubFrames(true),
+      mSearchParentFrames(true) {}
+
+nsWebBrowserFind::~nsWebBrowserFind() = default;
 
 NS_IMPL_ISUPPORTS(nsWebBrowserFind, nsIWebBrowserFind,
                   nsIWebBrowserFindInFrames)
 
 NS_IMETHODIMP
-nsWebBrowserFind::FindNext(bool* aResult)
-{
+nsWebBrowserFind::FindNext(bool* aResult) {
   NS_ENSURE_ARG_POINTER(aResult);
   *aResult = false;
 
   NS_ENSURE_TRUE(CanFindNext(), NS_ERROR_NOT_INITIALIZED);
 
   nsresult rv = NS_OK;
-  nsCOMPtr<nsPIDOMWindowOuter> searchFrame = do_QueryReferent(mCurrentSearchFrame);
+  nsCOMPtr<nsPIDOMWindowOuter> searchFrame =
+      do_QueryReferent(mCurrentSearchFrame);
   NS_ENSURE_TRUE(searchFrame, NS_ERROR_NOT_INITIALIZED);
 
   nsCOMPtr<nsPIDOMWindowOuter> rootFrame = do_QueryReferent(mRootSearchFrame);
@@ -84,10 +81,10 @@ nsWebBrowserFind::FindNext(bool* aResult)
   // this is used by nsTypeAheadFind, which controls find again when it was
   // the last executed find in the current window.
   nsCOMPtr<nsIObserverService> observerSvc =
-    mozilla::services::GetObserverService();
+      mozilla::services::GetObserverService();
   if (observerSvc) {
     nsCOMPtr<nsISupportsInterfacePointer> windowSupportsData =
-      do_CreateInstance(NS_SUPPORTS_INTERFACE_POINTER_CONTRACTID, &rv);
+        do_CreateInstance(NS_SUPPORTS_INTERFACE_POINTER_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
     nsCOMPtr<nsISupports> searchWindowSupports = do_QueryInterface(rootFrame);
     windowSupportsData->SetData(searchWindowSupports);
@@ -111,7 +108,7 @@ nsWebBrowserFind::FindNext(bool* aResult)
     return rv;
   }
   if (*aResult) {
-    return OnFind(searchFrame); // we are done
+    return OnFind(searchFrame);  // we are done
   }
 
   // if we are not searching other frames, return
@@ -124,41 +121,23 @@ nsWebBrowserFind::FindNext(bool* aResult)
     return NS_ERROR_FAILURE;
   }
 
-  int32_t enumDirection = mFindBackwards ? nsIDocShell::ENUMERATE_BACKWARDS :
-                                           nsIDocShell::ENUMERATE_FORWARDS;
+  auto enumDirection = mFindBackwards ? nsIDocShell::ENUMERATE_BACKWARDS
+                                      : nsIDocShell::ENUMERATE_FORWARDS;
 
-  nsCOMPtr<nsISimpleEnumerator> docShellEnumerator;
-  rv = rootDocShell->GetDocShellEnumerator(nsIDocShellTreeItem::typeAll,
-                                           enumDirection,
-                                           getter_AddRefs(docShellEnumerator));
+  nsTArray<RefPtr<nsIDocShell>> docShells;
+  rv = rootDocShell->GetAllDocShellsInSubtree(nsIDocShellTreeItem::typeAll,
+                                              enumDirection, docShells);
   if (NS_FAILED(rv)) {
     return rv;
   }
 
   // remember where we started
-  nsCOMPtr<nsIDocShellTreeItem> startingItem =
-    do_QueryInterface(searchFrame->GetDocShell(), &rv);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  nsCOMPtr<nsIDocShellTreeItem> curItem;
+  nsCOMPtr<nsIDocShellTreeItem> startingItem = searchFrame->GetDocShell();
 
   // XXX We should avoid searching in frameset documents here.
   // We also need to honour mSearchSubFrames and mSearchParentFrames.
-  bool hasMore, doFind = false;
-  while (NS_SUCCEEDED(docShellEnumerator->HasMoreElements(&hasMore)) &&
-         hasMore) {
-    nsCOMPtr<nsISupports> curSupports;
-    rv = docShellEnumerator->GetNext(getter_AddRefs(curSupports));
-    if (NS_FAILED(rv)) {
-      break;
-    }
-    curItem = do_QueryInterface(curSupports, &rv);
-    if (NS_FAILED(rv)) {
-      break;
-    }
-
+  bool doFind = false;
+  for (const auto& curItem : docShells) {
     if (doFind) {
       searchFrame = curItem->GetWindow();
       if (!searchFrame) {
@@ -174,14 +153,14 @@ nsWebBrowserFind::FindNext(bool* aResult)
         return rv;
       }
       if (*aResult) {
-        return OnFind(searchFrame); // we are done
+        return OnFind(searchFrame);  // we are done
       }
 
       OnEndSearchFrame(searchFrame);
     }
 
     if (curItem.get() == startingItem.get()) {
-      doFind = true; // start looking in frames after this one
+      doFind = true;  // start looking in frames after this one
     }
   }
 
@@ -194,28 +173,15 @@ nsWebBrowserFind::FindNext(bool* aResult)
   // From here on, we're wrapping, first through the other frames, then finally
   // from the beginning of the starting frame back to the starting point.
 
-  // because nsISimpleEnumerator is totally lame and isn't resettable, I have to
+  // because nsISimpleEnumerator is bad and isn't resettable, I have to
   // make a new one
-  docShellEnumerator = nullptr;
-  rv = rootDocShell->GetDocShellEnumerator(nsIDocShellTreeItem::typeAll,
-                                           enumDirection,
-                                           getter_AddRefs(docShellEnumerator));
+  rv = rootDocShell->GetAllDocShellsInSubtree(nsIDocShellTreeItem::typeAll,
+                                              enumDirection, docShells);
   if (NS_FAILED(rv)) {
     return rv;
   }
 
-  while (NS_SUCCEEDED(docShellEnumerator->HasMoreElements(&hasMore)) &&
-         hasMore) {
-    nsCOMPtr<nsISupports> curSupports;
-    rv = docShellEnumerator->GetNext(getter_AddRefs(curSupports));
-    if (NS_FAILED(rv)) {
-      break;
-    }
-    curItem = do_QueryInterface(curSupports, &rv);
-    if (NS_FAILED(rv)) {
-      break;
-    }
-
+  for (const auto& curItem : docShells) {
     searchFrame = curItem->GetWindow();
     if (!searchFrame) {
       rv = NS_ERROR_FAILURE;
@@ -230,7 +196,7 @@ nsWebBrowserFind::FindNext(bool* aResult)
         return rv;
       }
       if (*aResult) {
-        return OnFind(searchFrame); // we are done
+        return OnFind(searchFrame);  // we are done
       }
       break;
     }
@@ -244,7 +210,7 @@ nsWebBrowserFind::FindNext(bool* aResult)
       return rv;
     }
     if (*aResult) {
-      return OnFind(searchFrame); // we are done
+      return OnFind(searchFrame);  // we are done
     }
 
     OnEndSearchFrame(searchFrame);
@@ -258,111 +224,95 @@ nsWebBrowserFind::FindNext(bool* aResult)
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::GetSearchString(char16_t** aSearchString)
-{
-  NS_ENSURE_ARG_POINTER(aSearchString);
-  *aSearchString = ToNewUnicode(mSearchString);
+nsWebBrowserFind::GetSearchString(nsAString& aSearchString) {
+  aSearchString = mSearchString;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::SetSearchString(const char16_t* aSearchString)
-{
-  mSearchString.Assign(aSearchString);
+nsWebBrowserFind::SetSearchString(const nsAString& aSearchString) {
+  mSearchString = aSearchString;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::GetFindBackwards(bool* aFindBackwards)
-{
+nsWebBrowserFind::GetFindBackwards(bool* aFindBackwards) {
   NS_ENSURE_ARG_POINTER(aFindBackwards);
   *aFindBackwards = mFindBackwards;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::SetFindBackwards(bool aFindBackwards)
-{
+nsWebBrowserFind::SetFindBackwards(bool aFindBackwards) {
   mFindBackwards = aFindBackwards;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::GetWrapFind(bool* aWrapFind)
-{
+nsWebBrowserFind::GetWrapFind(bool* aWrapFind) {
   NS_ENSURE_ARG_POINTER(aWrapFind);
   *aWrapFind = mWrapFind;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::SetWrapFind(bool aWrapFind)
-{
+nsWebBrowserFind::SetWrapFind(bool aWrapFind) {
   mWrapFind = aWrapFind;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::GetEntireWord(bool* aEntireWord)
-{
+nsWebBrowserFind::GetEntireWord(bool* aEntireWord) {
   NS_ENSURE_ARG_POINTER(aEntireWord);
   *aEntireWord = mEntireWord;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::SetEntireWord(bool aEntireWord)
-{
+nsWebBrowserFind::SetEntireWord(bool aEntireWord) {
   mEntireWord = aEntireWord;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::GetMatchCase(bool* aMatchCase)
-{
+nsWebBrowserFind::GetMatchCase(bool* aMatchCase) {
   NS_ENSURE_ARG_POINTER(aMatchCase);
   *aMatchCase = mMatchCase;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::SetMatchCase(bool aMatchCase)
-{
+nsWebBrowserFind::SetMatchCase(bool aMatchCase) {
   mMatchCase = aMatchCase;
   return NS_OK;
 }
 
-static bool
-IsInNativeAnonymousSubtree(nsIContent* aContent)
-{
-  while (aContent) {
-    nsIContent* bindingParent = aContent->GetBindingParent();
-    if (bindingParent == aContent) {
-      return true;
-    }
-
-    aContent = bindingParent;
-  }
-
-  return false;
+NS_IMETHODIMP
+nsWebBrowserFind::GetMatchDiacritics(bool* aMatchDiacritics) {
+  NS_ENSURE_ARG_POINTER(aMatchDiacritics);
+  *aMatchDiacritics = mMatchDiacritics;
+  return NS_OK;
 }
 
-void
-nsWebBrowserFind::SetSelectionAndScroll(nsPIDOMWindowOuter* aWindow,
-                                        nsIDOMRange* aRange)
-{
-  nsCOMPtr<nsIDocument> doc = aWindow->GetDoc();
+NS_IMETHODIMP
+nsWebBrowserFind::SetMatchDiacritics(bool aMatchDiacritics) {
+  mMatchDiacritics = aMatchDiacritics;
+  return NS_OK;
+}
+
+void nsWebBrowserFind::SetSelectionAndScroll(nsPIDOMWindowOuter* aWindow,
+                                             nsRange* aRange) {
+  RefPtr<Document> doc = aWindow->GetDoc();
   if (!doc) {
     return;
   }
 
-  nsIPresShell* presShell = doc->GetShell();
+  PresShell* presShell = doc->GetPresShell();
   if (!presShell) {
     return;
   }
 
-  nsCOMPtr<nsIDOMNode> node;
-  aRange->GetStartContainer(getter_AddRefs(node));
+  nsCOMPtr<nsINode> node = aRange->GetStartContainer();
   nsCOMPtr<nsIContent> content(do_QueryInterface(node));
   nsIFrame* frame = content->GetPrimaryFrame();
   if (!frame) {
@@ -374,104 +324,67 @@ nsWebBrowserFind::SetSelectionAndScroll(nsPIDOMWindowOuter* aWindow,
 
   // since the match could be an anonymous textnode inside a
   // <textarea> or text <input>, we need to get the outer frame
-  nsITextControlFrame* tcFrame = nullptr;
+  nsIFrame* tcFrame = nullptr;
   for (; content; content = content->GetParent()) {
-    if (!IsInNativeAnonymousSubtree(content)) {
+    if (!content->IsInNativeAnonymousSubtree()) {
       nsIFrame* f = content->GetPrimaryFrame();
       if (!f) {
         return;
       }
-      tcFrame = do_QueryFrame(f);
+      if (f->IsTextInputFrame()) {
+        tcFrame = f;
+      }
       break;
     }
   }
 
-  nsCOMPtr<nsISelection> selection;
-
   selCon->SetDisplaySelection(nsISelectionController::SELECTION_ON);
-  selCon->GetSelection(nsISelectionController::SELECTION_NORMAL,
-                       getter_AddRefs(selection));
-  if (selection) {
-    selection->RemoveAllRanges();
-    selection->AddRange(aRange);
+  RefPtr<Selection> selection =
+      selCon->GetSelection(nsISelectionController::SELECTION_NORMAL);
+  if (!selection) {
+    return;
+  }
+  selection->RemoveAllRanges(IgnoreErrors());
+  selection->AddRangeAndSelectFramesAndNotifyListeners(*aRange, IgnoreErrors());
 
-    nsCOMPtr<nsIFocusManager> fm = do_GetService(FOCUSMANAGER_CONTRACTID);
-    if (fm) {
-      if (tcFrame) {
-        nsCOMPtr<nsIDOMElement> newFocusedElement(do_QueryInterface(content));
-        fm->SetFocus(newFocusedElement, nsIFocusManager::FLAG_NOSCROLL);
-      } else {
-        nsCOMPtr<nsIDOMElement> result;
-        fm->MoveFocus(aWindow, nullptr, nsIFocusManager::MOVEFOCUS_CARET,
-                      nsIFocusManager::FLAG_NOSCROLL, getter_AddRefs(result));
-      }
+  if (RefPtr<nsFocusManager> fm = nsFocusManager::GetFocusManager()) {
+    if (tcFrame) {
+      RefPtr<Element> newFocusedElement = Element::FromNode(content);
+      fm->SetFocus(newFocusedElement, nsIFocusManager::FLAG_NOSCROLL);
+    } else {
+      RefPtr<Element> result;
+      fm->MoveFocus(aWindow, nullptr, nsIFocusManager::MOVEFOCUS_CARET,
+                    nsIFocusManager::FLAG_NOSCROLL, getter_AddRefs(result));
     }
-
-    // Scroll if necessary to make the selection visible:
-    // Must be the last thing to do - bug 242056
-
-    // After ScrollSelectionIntoView(), the pending notifications might be
-    // flushed and PresShell/PresContext/Frames may be dead. See bug 418470.
-    selCon->ScrollSelectionIntoView(
-      nsISelectionController::SELECTION_NORMAL,
-      nsISelectionController::SELECTION_WHOLE_SELECTION,
-      nsISelectionController::SCROLL_CENTER_VERTICALLY |
-        nsISelectionController::SCROLL_SYNCHRONOUS);
   }
+
+  // Scroll if necessary to make the selection visible:
+  // Must be the last thing to do - bug 242056
+
+  // After ScrollSelectionIntoView(), the pending notifications might be
+  // flushed and PresShell/PresContext/Frames may be dead. See bug 418470.
+  // FIXME(emilio): Any reason this couldn't do selection->ScrollIntoView()
+  // directly, rather than re-requesting the selection?
+  selCon->ScrollSelectionIntoView(
+      SelectionType::eNormal, nsISelectionController::SELECTION_WHOLE_SELECTION,
+      ScrollAxis(WhereToScroll::Center), ScrollAxis(), ScrollFlags::None,
+      SelectionScrollMode::SyncFlush);
 }
 
-// Adapted from TextServicesDocument::GetDocumentContentRootNode
-nsresult
-nsWebBrowserFind::GetRootNode(nsIDOMDocument* aDomDoc, nsIDOMNode** aNode)
-{
-  NS_ENSURE_ARG_POINTER(aNode);
-  *aNode = 0;
+nsresult nsWebBrowserFind::SetRangeAroundDocument(nsRange* aSearchRange,
+                                                  nsRange* aStartPt,
+                                                  nsRange* aEndPt,
+                                                  Document* aDoc) {
+  NS_ENSURE_ARG_POINTER(aDoc);
+  uint32_t childCount = aDoc->GetChildCount();
 
-  nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDomDoc);
-  NS_ENSURE_ARG_POINTER(doc);
+  aSearchRange->SetStart(*aDoc, 0, IgnoreErrors());
+  aSearchRange->SetEnd(*aDoc, childCount, IgnoreErrors());
 
-  if (doc->IsHTMLOrXHTML()) {
-    Element* body = doc->GetBody();
-    NS_ENSURE_ARG_POINTER(body);
-    NS_ADDREF(*aNode = body->AsDOMNode());
-    return NS_OK;
-  }
-
-  // For non-HTML documents, the content root node will be the doc element.
-  Element* root = doc->GetDocumentElement();
-  NS_ENSURE_ARG_POINTER(root);
-  NS_ADDREF(*aNode = root->AsDOMNode());
-  return NS_OK;
-}
-
-nsresult
-nsWebBrowserFind::SetRangeAroundDocument(nsIDOMRange* aSearchRange,
-                                         nsIDOMRange* aStartPt,
-                                         nsIDOMRange* aEndPt,
-                                         nsIDOMDocument* aDoc)
-{
-  nsCOMPtr<nsIDOMNode> bodyNode;
-  nsresult rv = GetRootNode(aDoc, getter_AddRefs(bodyNode));
-  nsCOMPtr<nsIContent> bodyContent(do_QueryInterface(bodyNode));
-  NS_ENSURE_SUCCESS(rv, rv);
-  NS_ENSURE_ARG_POINTER(bodyContent);
-
-  uint32_t childCount = bodyContent->GetChildCount();
-
-  aSearchRange->SetStart(bodyNode, 0);
-  aSearchRange->SetEnd(bodyNode, childCount);
-
-  if (mFindBackwards) {
-    aStartPt->SetStart(bodyNode, childCount);
-    aStartPt->SetEnd(bodyNode, childCount);
-    aEndPt->SetStart(bodyNode, 0);
-    aEndPt->SetEnd(bodyNode, 0);
-  } else {
-    aStartPt->SetStart(bodyNode, 0);
-    aStartPt->SetEnd(bodyNode, 0);
-    aEndPt->SetStart(bodyNode, childCount);
-    aEndPt->SetEnd(bodyNode, childCount);
-  }
+  aStartPt->SetStart(*aDoc, 0, IgnoreErrors());
+  aStartPt->SetEnd(*aDoc, 0, IgnoreErrors());
+  aEndPt->SetStart(*aDoc, childCount, IgnoreErrors());
+  aEndPt->SetEnd(*aDoc, childCount, IgnoreErrors());
 
   return NS_OK;
 }
@@ -479,123 +392,117 @@ nsWebBrowserFind::SetRangeAroundDocument(nsIDOMRange* aSearchRange,
 // Set the range to go from the end of the current selection to the end of the
 // document (forward), or beginning to beginning (reverse). or around the whole
 // document if there's no selection.
-nsresult
-nsWebBrowserFind::GetSearchLimits(nsIDOMRange* aSearchRange,
-                                  nsIDOMRange* aStartPt, nsIDOMRange* aEndPt,
-                                  nsIDOMDocument* aDoc, nsISelection* aSel,
-                                  bool aWrap)
-{
+nsresult nsWebBrowserFind::GetSearchLimits(nsRange* aSearchRange,
+                                           nsRange* aStartPt, nsRange* aEndPt,
+                                           Document* aDoc, Selection* aSel,
+                                           bool aWrap) {
   NS_ENSURE_ARG_POINTER(aSel);
 
   // There is a selection.
-  int32_t count = -1;
-  nsresult rv = aSel->GetRangeCount(&count);
-  NS_ENSURE_SUCCESS(rv, rv);
-  if (count < 1) {
+  const uint32_t rangeCount = aSel->RangeCount();
+  if (rangeCount < 1) {
     return SetRangeAroundDocument(aSearchRange, aStartPt, aEndPt, aDoc);
   }
 
-  // Need bodyNode, for the start/end of the document
-  nsCOMPtr<nsIDOMNode> bodyNode;
-  rv = GetRootNode(aDoc, getter_AddRefs(bodyNode));
-  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_ARG_POINTER(aDoc);
 
-  nsCOMPtr<nsIContent> bodyContent(do_QueryInterface(bodyNode));
-  NS_ENSURE_ARG_POINTER(bodyContent);
-
-  uint32_t childCount = bodyContent->GetChildCount();
+  uint32_t childCount = aDoc->GetChildCount();
 
   // There are four possible range endpoints we might use:
   // DocumentStart, SelectionStart, SelectionEnd, DocumentEnd.
 
-  nsCOMPtr<nsIDOMRange> range;
-  nsCOMPtr<nsIDOMNode> node;
+  RefPtr<const nsRange> range;
+  nsCOMPtr<nsINode> node;
   uint32_t offset;
 
+  // Prevent the security checks in nsRange from getting into effect for the
+  // purposes of determining the search range. These ranges will never be
+  // exposed to content.
+  mozilla::dom::AutoNoJSAPI nojsapi;
+
   // Forward, not wrapping: SelEnd to DocEnd
   if (!mFindBackwards && !aWrap) {
     // This isn't quite right, since the selection's ranges aren't
     // necessarily in order; but they usually will be.
-    aSel->GetRangeAt(count - 1, getter_AddRefs(range));
+    range = aSel->GetRangeAt(rangeCount - 1);
     if (!range) {
       return NS_ERROR_UNEXPECTED;
     }
-    range->GetEndContainer(getter_AddRefs(node));
+    node = range->GetEndContainer();
     if (!node) {
       return NS_ERROR_UNEXPECTED;
     }
-    range->GetEndOffset(&offset);
+    offset = range->EndOffset();
 
-    aSearchRange->SetStart(node, offset);
-    aSearchRange->SetEnd(bodyNode, childCount);
-    aStartPt->SetStart(node, offset);
-    aStartPt->SetEnd(node, offset);
-    aEndPt->SetStart(bodyNode, childCount);
-    aEndPt->SetEnd(bodyNode, childCount);
+    aSearchRange->SetStart(*node, offset, IgnoreErrors());
+    aSearchRange->SetEnd(*aDoc, childCount, IgnoreErrors());
+    aStartPt->SetStart(*node, offset, IgnoreErrors());
+    aStartPt->SetEnd(*node, offset, IgnoreErrors());
+    aEndPt->SetStart(*aDoc, childCount, IgnoreErrors());
+    aEndPt->SetEnd(*aDoc, childCount, IgnoreErrors());
   }
   // Backward, not wrapping: DocStart to SelStart
   else if (mFindBackwards && !aWrap) {
-    aSel->GetRangeAt(0, getter_AddRefs(range));
+    range = aSel->GetRangeAt(0);
     if (!range) {
       return NS_ERROR_UNEXPECTED;
     }
-    range->GetStartContainer(getter_AddRefs(node));
+    node = range->GetStartContainer();
     if (!node) {
       return NS_ERROR_UNEXPECTED;
     }
-    range->GetStartOffset(&offset);
+    offset = range->StartOffset();
 
-    aSearchRange->SetStart(bodyNode, 0);
-    aSearchRange->SetEnd(bodyNode, childCount);
-    aStartPt->SetStart(node, offset);
-    aStartPt->SetEnd(node, offset);
-    aEndPt->SetStart(bodyNode, 0);
-    aEndPt->SetEnd(bodyNode, 0);
+    aSearchRange->SetStart(*aDoc, 0, IgnoreErrors());
+    aSearchRange->SetEnd(*aDoc, childCount, IgnoreErrors());
+    aStartPt->SetStart(*aDoc, 0, IgnoreErrors());
+    aStartPt->SetEnd(*aDoc, 0, IgnoreErrors());
+    aEndPt->SetStart(*node, offset, IgnoreErrors());
+    aEndPt->SetEnd(*node, offset, IgnoreErrors());
   }
   // Forward, wrapping: DocStart to SelEnd
   else if (!mFindBackwards && aWrap) {
-    aSel->GetRangeAt(count - 1, getter_AddRefs(range));
+    range = aSel->GetRangeAt(rangeCount - 1);
     if (!range) {
       return NS_ERROR_UNEXPECTED;
     }
-    range->GetEndContainer(getter_AddRefs(node));
+    node = range->GetEndContainer();
     if (!node) {
       return NS_ERROR_UNEXPECTED;
     }
-    range->GetEndOffset(&offset);
+    offset = range->EndOffset();
 
-    aSearchRange->SetStart(bodyNode, 0);
-    aSearchRange->SetEnd(bodyNode, childCount);
-    aStartPt->SetStart(bodyNode, 0);
-    aStartPt->SetEnd(bodyNode, 0);
-    aEndPt->SetStart(node, offset);
-    aEndPt->SetEnd(node, offset);
+    aSearchRange->SetStart(*aDoc, 0, IgnoreErrors());
+    aSearchRange->SetEnd(*aDoc, childCount, IgnoreErrors());
+    aStartPt->SetStart(*aDoc, 0, IgnoreErrors());
+    aStartPt->SetEnd(*aDoc, 0, IgnoreErrors());
+    aEndPt->SetStart(*node, offset, IgnoreErrors());
+    aEndPt->SetEnd(*node, offset, IgnoreErrors());
   }
   // Backward, wrapping: SelStart to DocEnd
   else if (mFindBackwards && aWrap) {
-    aSel->GetRangeAt(0, getter_AddRefs(range));
+    range = aSel->GetRangeAt(0);
     if (!range) {
       return NS_ERROR_UNEXPECTED;
     }
-    range->GetStartContainer(getter_AddRefs(node));
+    node = range->GetStartContainer();
     if (!node) {
       return NS_ERROR_UNEXPECTED;
     }
-    range->GetStartOffset(&offset);
+    offset = range->StartOffset();
 
-    aSearchRange->SetStart(bodyNode, 0);
-    aSearchRange->SetEnd(bodyNode, childCount);
-    aStartPt->SetStart(bodyNode, childCount);
-    aStartPt->SetEnd(bodyNode, childCount);
-    aEndPt->SetStart(node, offset);
-    aEndPt->SetEnd(node, offset);
+    aSearchRange->SetStart(*aDoc, 0, IgnoreErrors());
+    aSearchRange->SetEnd(*aDoc, childCount, IgnoreErrors());
+    aStartPt->SetStart(*node, offset, IgnoreErrors());
+    aStartPt->SetEnd(*node, offset, IgnoreErrors());
+    aEndPt->SetStart(*aDoc, childCount, IgnoreErrors());
+    aEndPt->SetEnd(*aDoc, childCount, IgnoreErrors());
   }
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::GetSearchFrames(bool* aSearchFrames)
-{
+nsWebBrowserFind::GetSearchFrames(bool* aSearchFrames) {
   NS_ENSURE_ARG_POINTER(aSearchFrames);
   // this only returns true if we are searching both sub and parent frames.
   // There is ambiguity if the caller has previously set one, but not both of
@@ -605,25 +512,25 @@ nsWebBrowserFind::GetSearchFrames(bool* aSearchFrames)
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::SetSearchFrames(bool aSearchFrames)
-{
+nsWebBrowserFind::SetSearchFrames(bool aSearchFrames) {
   mSearchSubFrames = aSearchFrames;
   mSearchParentFrames = aSearchFrames;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::GetCurrentSearchFrame(mozIDOMWindowProxy** aCurrentSearchFrame)
-{
+nsWebBrowserFind::GetCurrentSearchFrame(
+    mozIDOMWindowProxy** aCurrentSearchFrame) {
   NS_ENSURE_ARG_POINTER(aCurrentSearchFrame);
-  nsCOMPtr<mozIDOMWindowProxy> searchFrame = do_QueryReferent(mCurrentSearchFrame);
+  nsCOMPtr<mozIDOMWindowProxy> searchFrame =
+      do_QueryReferent(mCurrentSearchFrame);
   searchFrame.forget(aCurrentSearchFrame);
   return (*aCurrentSearchFrame) ? NS_OK : NS_ERROR_NOT_INITIALIZED;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::SetCurrentSearchFrame(mozIDOMWindowProxy* aCurrentSearchFrame)
-{
+nsWebBrowserFind::SetCurrentSearchFrame(
+    mozIDOMWindowProxy* aCurrentSearchFrame) {
   // is it ever valid to set this to null?
   NS_ENSURE_ARG(aCurrentSearchFrame);
   mCurrentSearchFrame = do_GetWeakReference(aCurrentSearchFrame);
@@ -631,8 +538,7 @@ nsWebBrowserFind::SetCurrentSearchFrame(mozIDOMWindowProxy* aCurrentSearchFrame)
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::GetRootSearchFrame(mozIDOMWindowProxy** aRootSearchFrame)
-{
+nsWebBrowserFind::GetRootSearchFrame(mozIDOMWindowProxy** aRootSearchFrame) {
   NS_ENSURE_ARG_POINTER(aRootSearchFrame);
   nsCOMPtr<mozIDOMWindowProxy> searchFrame = do_QueryReferent(mRootSearchFrame);
   searchFrame.forget(aRootSearchFrame);
@@ -640,8 +546,7 @@ nsWebBrowserFind::GetRootSearchFrame(mozIDOMWindowProxy** aRootSearchFrame)
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::SetRootSearchFrame(mozIDOMWindowProxy* aRootSearchFrame)
-{
+nsWebBrowserFind::SetRootSearchFrame(mozIDOMWindowProxy* aRootSearchFrame) {
   // is it ever valid to set this to null?
   NS_ENSURE_ARG(aRootSearchFrame);
   mRootSearchFrame = do_GetWeakReference(aRootSearchFrame);
@@ -649,31 +554,27 @@ nsWebBrowserFind::SetRootSearchFrame(mozIDOMWindowProxy* aRootSearchFrame)
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::GetSearchSubframes(bool* aSearchSubframes)
-{
+nsWebBrowserFind::GetSearchSubframes(bool* aSearchSubframes) {
   NS_ENSURE_ARG_POINTER(aSearchSubframes);
   *aSearchSubframes = mSearchSubFrames;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::SetSearchSubframes(bool aSearchSubframes)
-{
+nsWebBrowserFind::SetSearchSubframes(bool aSearchSubframes) {
   mSearchSubFrames = aSearchSubframes;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::GetSearchParentFrames(bool* aSearchParentFrames)
-{
+nsWebBrowserFind::GetSearchParentFrames(bool* aSearchParentFrames) {
   NS_ENSURE_ARG_POINTER(aSearchParentFrames);
   *aSearchParentFrames = mSearchParentFrames;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWebBrowserFind::SetSearchParentFrames(bool aSearchParentFrames)
-{
+nsWebBrowserFind::SetSearchParentFrames(bool aSearchParentFrames) {
   mSearchParentFrames = aSearchParentFrames;
   return NS_OK;
 }
@@ -682,10 +583,8 @@ nsWebBrowserFind::SetSearchParentFrames(bool aSearchParentFrames)
     This method handles finding in a single window (aka frame).
 
 */
-nsresult
-nsWebBrowserFind::SearchInFrame(nsPIDOMWindowOuter* aWindow, bool aWrapping,
-                                bool* aDidFind)
-{
+nsresult nsWebBrowserFind::SearchInFrame(nsPIDOMWindowOuter* aWindow,
+                                         bool aWrapping, bool* aDidFind) {
   NS_ENSURE_ARG(aWindow);
   NS_ENSURE_ARG_POINTER(aDidFind);
 
@@ -695,7 +594,7 @@ nsWebBrowserFind::SearchInFrame(nsPIDOMWindowOuter* aWindow, bool aWrapping,
   // accessible from the frame where the Find is being run.
 
   // get a uri for the window
-  nsCOMPtr<nsIDocument> theDoc = aWindow->GetDoc();
+  RefPtr<Document> theDoc = aWindow->GetDoc();
   if (!theDoc) {
     return NS_ERROR_FAILURE;
   }
@@ -709,6 +608,7 @@ nsWebBrowserFind::SearchInFrame(nsPIDOMWindowOuter* aWindow, bool aWrapping,
   NS_ENSURE_SUCCESS(rv, rv);
 
   (void)find->SetCaseSensitive(mMatchCase);
+  (void)find->SetMatchDiacritics(mMatchDiacritics);
   (void)find->SetFindBackwards(mFindBackwards);
 
   (void)find->SetEntireWord(mEntireWord);
@@ -717,38 +617,24 @@ nsWebBrowserFind::SearchInFrame(nsPIDOMWindowOuter* aWindow, bool aWrapping,
   // selection) models are up to date.
   theDoc->FlushPendingNotifications(FlushType::Frames);
 
-  nsCOMPtr<nsISelection> sel = GetFrameSelection(aWindow);
+  RefPtr<Selection> sel = GetFrameSelection(aWindow);
   NS_ENSURE_ARG_POINTER(sel);
 
-  nsCOMPtr<nsIDOMRange> searchRange = new nsRange(theDoc);
-  NS_ENSURE_ARG_POINTER(searchRange);
-  nsCOMPtr<nsIDOMRange> startPt = new nsRange(theDoc);
-  NS_ENSURE_ARG_POINTER(startPt);
-  nsCOMPtr<nsIDOMRange> endPt = new nsRange(theDoc);
-  NS_ENSURE_ARG_POINTER(endPt);
+  RefPtr<nsRange> searchRange = nsRange::Create(theDoc);
+  RefPtr<nsRange> startPt = nsRange::Create(theDoc);
+  RefPtr<nsRange> endPt = nsRange::Create(theDoc);
 
-  nsCOMPtr<nsIDOMRange> foundRange;
-
-  nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(theDoc);
-  MOZ_ASSERT(domDoc);
-
-  // If !aWrapping, search from selection to end
-  if (!aWrapping)
-    rv = GetSearchLimits(searchRange, startPt, endPt, domDoc, sel, false);
-
-  // If aWrapping, search the part of the starting frame
-  // up to the point where we left off.
-  else
-    rv = GetSearchLimits(searchRange, startPt, endPt, domDoc, sel, true);
+  RefPtr<nsRange> foundRange;
 
+  rv = GetSearchLimits(searchRange, startPt, endPt, theDoc, sel, aWrapping);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = find->Find(mSearchString.get(), searchRange, startPt, endPt,
+  rv = find->Find(mSearchString, searchRange, startPt, endPt,
                   getter_AddRefs(foundRange));
 
   if (NS_SUCCEEDED(rv) && foundRange) {
     *aDidFind = true;
-    sel->RemoveAllRanges();
+    sel->RemoveAllRanges(IgnoreErrors());
     // Beware! This may flush notifications via synchronous
     // ScrollSelectionIntoView.
     SetSelectionAndScroll(aWindow, foundRange);
@@ -760,29 +646,24 @@ nsWebBrowserFind::SearchInFrame(nsPIDOMWindowOuter* aWindow, bool aWrapping,
 // called when we start searching a frame that is not the initial focussed
 // frame. Prepare the frame to be searched. we clear the selection, so that the
 // search starts from the top of the frame.
-nsresult
-nsWebBrowserFind::OnStartSearchFrame(nsPIDOMWindowOuter* aWindow)
-{
+nsresult nsWebBrowserFind::OnStartSearchFrame(nsPIDOMWindowOuter* aWindow) {
   return ClearFrameSelection(aWindow);
 }
 
 // called when we are done searching a frame and didn't find anything, and about
 // about to start searching the next frame.
-nsresult
-nsWebBrowserFind::OnEndSearchFrame(nsPIDOMWindowOuter* aWindow)
-{
+nsresult nsWebBrowserFind::OnEndSearchFrame(nsPIDOMWindowOuter* aWindow) {
   return NS_OK;
 }
 
-already_AddRefed<nsISelection>
-nsWebBrowserFind::GetFrameSelection(nsPIDOMWindowOuter* aWindow)
-{
-  nsCOMPtr<nsIDocument> doc = aWindow->GetDoc();
+already_AddRefed<Selection> nsWebBrowserFind::GetFrameSelection(
+    nsPIDOMWindowOuter* aWindow) {
+  RefPtr<Document> doc = aWindow->GetDoc();
   if (!doc) {
     return nullptr;
   }
 
-  nsIPresShell* presShell = doc->GetShell();
+  PresShell* presShell = doc->GetPresShell();
   if (!presShell) {
     return nullptr;
   }
@@ -792,66 +673,52 @@ nsWebBrowserFind::GetFrameSelection(nsPIDOMWindowOuter* aWindow)
   nsPresContext* presContext = presShell->GetPresContext();
 
   nsCOMPtr<nsPIDOMWindowOuter> focusedWindow;
-  nsCOMPtr<nsIContent> focusedContent =
-    nsFocusManager::GetFocusedDescendant(aWindow,
-                                         nsFocusManager::eOnlyCurrentWindow,
-                                         getter_AddRefs(focusedWindow));
+  nsCOMPtr<nsIContent> focusedContent = nsFocusManager::GetFocusedDescendant(
+      aWindow, nsFocusManager::eOnlyCurrentWindow,
+      getter_AddRefs(focusedWindow));
 
   nsIFrame* frame =
-    focusedContent ? focusedContent->GetPrimaryFrame() : nullptr;
+      focusedContent ? focusedContent->GetPrimaryFrame() : nullptr;
 
   nsCOMPtr<nsISelectionController> selCon;
-  nsCOMPtr<nsISelection> sel;
+  RefPtr<Selection> sel;
   if (frame) {
     frame->GetSelectionController(presContext, getter_AddRefs(selCon));
-    selCon->GetSelection(nsISelectionController::SELECTION_NORMAL,
-                         getter_AddRefs(sel));
-    if (sel) {
-      int32_t count = -1;
-      sel->GetRangeCount(&count);
-      if (count > 0) {
-        return sel.forget();
-      }
+    sel = selCon->GetSelection(nsISelectionController::SELECTION_NORMAL);
+    if (sel && sel->RangeCount() > 0) {
+      return sel.forget();
     }
   }
 
-  selCon = do_QueryInterface(presShell);
-  selCon->GetSelection(nsISelectionController::SELECTION_NORMAL,
-                       getter_AddRefs(sel));
+  sel = presShell->GetSelection(nsISelectionController::SELECTION_NORMAL);
   return sel.forget();
 }
 
-nsresult
-nsWebBrowserFind::ClearFrameSelection(nsPIDOMWindowOuter* aWindow)
-{
+nsresult nsWebBrowserFind::ClearFrameSelection(nsPIDOMWindowOuter* aWindow) {
   NS_ENSURE_ARG(aWindow);
-  nsCOMPtr<nsISelection> selection = GetFrameSelection(aWindow);
+  RefPtr<Selection> selection = GetFrameSelection(aWindow);
   if (selection) {
-    selection->RemoveAllRanges();
+    selection->RemoveAllRanges(IgnoreErrors());
   }
 
   return NS_OK;
 }
 
-nsresult
-nsWebBrowserFind::OnFind(nsPIDOMWindowOuter* aFoundWindow)
-{
+nsresult nsWebBrowserFind::OnFind(nsPIDOMWindowOuter* aFoundWindow) {
   SetCurrentSearchFrame(aFoundWindow);
 
   // We don't want a selection to appear in two frames simultaneously
   nsCOMPtr<nsPIDOMWindowOuter> lastFocusedWindow =
-    do_QueryReferent(mLastFocusedWindow);
+      do_QueryReferent(mLastFocusedWindow);
   if (lastFocusedWindow && lastFocusedWindow != aFoundWindow) {
     ClearFrameSelection(lastFocusedWindow);
   }
 
-  nsCOMPtr<nsIFocusManager> fm = do_GetService(FOCUSMANAGER_CONTRACTID);
-  if (fm) {
+  if (RefPtr<nsFocusManager> fm = nsFocusManager::GetFocusManager()) {
     // get the containing frame and focus it. For top-level windows, the right
     // window should already be focused.
-    nsCOMPtr<nsIDOMElement> frameElement =
-      do_QueryInterface(aFoundWindow->GetFrameElementInternal());
-    if (frameElement) {
+    if (RefPtr<Element> frameElement =
+            aFoundWindow->GetFrameElementInternal()) {
       fm->SetFocus(frameElement, 0);
     }
 
