@@ -49,13 +49,29 @@ add_setup(function test_setup() {
 add_task(function test_gifft_counter() {
   Glean.testOnlyIpc.aCounter.add(20);
   Assert.equal(20, Glean.testOnlyIpc.aCounter.testGetValue());
-  Assert.equal(20, scalarValue("telemetry.test.mirror_for_counter"));
+  let telemetryValue = scalarValue("telemetry.test.mirror_for_counter");
+  Assert.equal(20, telemetryValue);
+  Assert.equal("number", typeof telemetryValue);
+
+  Glean.testOnlyIpc.aCounterForHgram.add(20);
+  Assert.equal(20, Glean.testOnlyIpc.aCounterForHgram.testGetValue());
+  const data = Telemetry.getHistogramById("TELEMETRY_TEST_COUNT").snapshot();
+  const expected = {
+    bucket_count: 3,
+    histogram_type: 4,
+    sum: 20,
+    range: [1, 2],
+    values: { 0: 1, 1: 0 },
+  };
+  Assert.deepEqual(data, expected, "histogram successfully mirrored-to.");
 });
 
 add_task(function test_gifft_boolean() {
   Glean.testOnlyIpc.aBool.set(false);
   Assert.equal(false, Glean.testOnlyIpc.aBool.testGetValue());
-  Assert.equal(false, scalarValue("telemetry.test.boolean_kind"));
+  let telemetryValue = scalarValue("telemetry.test.boolean_kind");
+  Assert.equal(false, telemetryValue);
+  Assert.equal("boolean", typeof telemetryValue);
 });
 
 add_task(function test_gifft_datetime() {
@@ -73,7 +89,9 @@ add_task(function test_gifft_string() {
   Glean.testOnlyIpc.aString.set(value);
 
   Assert.equal(value, Glean.testOnlyIpc.aString.testGetValue());
-  Assert.equal(value, scalarValue("telemetry.test.multiple_stores_string"));
+  let telemetryValue = scalarValue("telemetry.test.multiple_stores_string");
+  Assert.equal(value, telemetryValue);
+  Assert.equal("string", typeof telemetryValue);
 });
 
 add_task(function test_gifft_memory_dist() {
@@ -181,13 +199,35 @@ add_task(async function test_gifft_timing_dist() {
   );
 
   data = Telemetry.getHistogramById("TELEMETRY_TEST_EXPONENTIAL").snapshot();
-  // Suffers from same cast truncation issue of 9.... and 4.... values
-  Assert.greaterOrEqual(data.sum, 13, "Histogram's in milliseconds");
+  Assert.greaterOrEqual(
+    data.sum,
+    13 * NANOS_IN_MILLIS - EPSILON,
+    "Histogram's in nanos"
+  );
   Assert.equal(
     2,
     Object.entries(data.values).reduce((acc, [, count]) => acc + count, 0),
     "Only two samples"
   );
+
+  // How about those raw sample APIs?
+  Glean.testOnly.whatTimeIsIt.accumulateSingleSample(42);
+  Glean.testOnly.whatTimeIsIt.accumulateSamples([255, 2047]);
+  data = Glean.testOnly.whatTimeIsIt.testGetValue();
+  Assert.equal(
+    data.sum,
+    (42 + 255 + 2047) * 1000,
+    "test.only.what_time_is_it is in microseconds and testGetValue returns you nanos."
+  );
+
+  data = Telemetry.getHistogramById(
+    "TELEMETRY_TEST_MIRROR_FOR_TIMING"
+  ).snapshot();
+  Assert.equal(
+    data.sum,
+    42 + 255 + 2047,
+    "Telemetry gives back exactly what's put in."
+  );
 });
 
 add_task(function test_gifft_string_list_works() {
@@ -218,8 +258,6 @@ add_task(function test_gifft_string_list_works() {
 });
 
 add_task(function test_gifft_events() {
-  Telemetry.setEventRecordingEnabled("telemetry.test", true);
-
   Glean.testOnlyIpc.noExtraEvent.record();
   var events = Glean.testOnlyIpc.noExtraEvent.testGetValue();
   Assert.equal(1, events.length);
@@ -236,8 +274,10 @@ add_task(function test_gifft_events() {
   let { extra1, extra2 } = extra;
   let telExtra = { extra1, extra2 };
   Glean.testOnlyIpc.anEvent.record(extra);
+  // Also test a value of "".
+  Glean.testOnlyIpc.anEvent.record({ value: "" });
   events = Glean.testOnlyIpc.anEvent.testGetValue();
-  Assert.equal(1, events.length);
+  Assert.equal(2, events.length);
   Assert.equal("test_only.ipc", events[0].category);
   Assert.equal("an_event", events[0].name);
   Assert.deepEqual(extra, events[0].extra);
@@ -246,6 +286,7 @@ add_task(function test_gifft_events() {
     [
       ["telemetry.test", "not_expired_optout", "object1", undefined, undefined],
       ["telemetry.test", "mirror_with_extra", "object1", extra.value, telExtra],
+      ["telemetry.test", "mirror_with_extra", "object1", "", undefined],
     ],
     { category: "telemetry.test" }
   );
@@ -277,7 +318,7 @@ add_task(function test_gifft_labeled_counter() {
     undefined,
     Glean.testOnlyIpc.aLabeledCounter.__other__.testGetValue()
   );
-  Glean.testOnlyIpc.aLabeledCounter["1".repeat(72)].add(3);
+  Glean.testOnlyIpc.aLabeledCounter["1".repeat(112)].add(3);
   Assert.throws(
     () => Glean.testOnlyIpc.aLabeledCounter.__other__.testGetValue(),
     /DataError/,
@@ -291,10 +332,121 @@ add_task(function test_gifft_labeled_counter() {
     {
       a_label: 4,
       another_label: 2,
-      ["1".repeat(72)]: 3,
     },
     value
   );
+
+  // What about a labeled_counter that maps to a boolean hgram?
+  Assert.equal(
+    undefined,
+    Glean.testOnlyIpc.aLabeledCounterForHgram.true.testGetValue()
+  );
+  Assert.equal(
+    undefined,
+    Glean.testOnlyIpc.aLabeledCounterForHgram.false.testGetValue()
+  );
+
+  Glean.testOnlyIpc.aLabeledCounterForHgram.true.add(1);
+  Glean.testOnlyIpc.aLabeledCounterForHgram.true.add(1);
+  Glean.testOnlyIpc.aLabeledCounterForHgram.false.add(1);
+
+  Assert.equal(
+    2,
+    Glean.testOnlyIpc.aLabeledCounterForHgram.true.testGetValue()
+  );
+  Assert.equal(
+    1,
+    Glean.testOnlyIpc.aLabeledCounterForHgram.false.testGetValue()
+  );
+
+  value = Telemetry.getHistogramById("TELEMETRY_TEST_BOOLEAN");
+  Assert.deepEqual(
+    {
+      bucket_count: 3,
+      histogram_type: 2,
+      sum: 2,
+      range: [1, 2],
+      values: { 0: 1, 1: 2, 2: 0 },
+    },
+    value.snapshot()
+  );
+
+  // What about a labeled_counter that maps to a keyed count hgram?
+  Assert.equal(
+    undefined,
+    Glean.testOnlyIpc.aLabeledCounterForKeyedCountHgram.aLabel.testGetValue()
+  );
+  Assert.equal(
+    undefined,
+    Glean.testOnlyIpc.aLabeledCounterForKeyedCountHgram.anotherLabel.testGetValue()
+  );
+
+  Glean.testOnlyIpc.aLabeledCounterForKeyedCountHgram.aLabel.add(2);
+  Glean.testOnlyIpc.aLabeledCounterForKeyedCountHgram.anotherLabel.add(1);
+
+  Assert.equal(
+    2,
+    Glean.testOnlyIpc.aLabeledCounterForKeyedCountHgram.aLabel.testGetValue()
+  );
+  Assert.equal(
+    1,
+    Glean.testOnlyIpc.aLabeledCounterForKeyedCountHgram.anotherLabel.testGetValue()
+  );
+
+  value = Telemetry.getKeyedHistogramById("TELEMETRY_TEST_KEYED_COUNT");
+  Assert.deepEqual(
+    {
+      aLabel: {
+        bucket_count: 3,
+        histogram_type: 4,
+        sum: 2,
+        range: [1, 2],
+        values: { 0: 1, 1: 0 },
+      },
+      anotherLabel: {
+        bucket_count: 3,
+        histogram_type: 4,
+        sum: 1,
+        range: [1, 2],
+        values: { 0: 1, 1: 0 },
+      },
+    },
+    value.snapshot()
+  );
+
+  // What about a labeled_counter that maps to a categorical hgram?
+  Assert.equal(
+    undefined,
+    Glean.testOnlyIpc.aLabeledCounterForCategorical.CommonLabel.testGetValue()
+  );
+  Assert.equal(
+    undefined,
+    Glean.testOnlyIpc.aLabeledCounterForCategorical.Label4.testGetValue()
+  );
+
+  Glean.testOnlyIpc.aLabeledCounterForCategorical.CommonLabel.add(1);
+  Glean.testOnlyIpc.aLabeledCounterForCategorical.Label4.add(1);
+
+  Assert.equal(
+    1,
+    Glean.testOnlyIpc.aLabeledCounterForCategorical.CommonLabel.testGetValue()
+  );
+  Assert.equal(
+    1,
+    Glean.testOnlyIpc.aLabeledCounterForCategorical.Label4.testGetValue()
+  );
+
+  value = Telemetry.getHistogramById("TELEMETRY_TEST_CATEGORICAL_OPTOUT");
+  Assert.deepEqual(
+    {
+      bucket_count: 51,
+      histogram_type: 5,
+      sum: 1,
+      range: [1, 50],
+      values: { 0: 1, 1: 1, 2: 0 },
+    },
+    value.snapshot()
+  );
 });
 
 add_task(async function test_gifft_timespan() {
@@ -305,8 +457,8 @@ add_task(async function test_gifft_timespan() {
   Glean.testOnly.mirrorTime.stop();
 
   const NANOS_IN_MILLIS = 1e6;
-  // bug 1701949 - Sleep gets close, but sometimes doesn't wait long enough.
-  const EPSILON = 40000;
+  // bug 1931539 - Sleep gets close, but sometimes doesn't wait long enough.
+  const EPSILON = 50000;
   Assert.greater(
     Glean.testOnly.mirrorTime.testGetValue(),
     10 * NANOS_IN_MILLIS - EPSILON
@@ -344,7 +496,7 @@ add_task(async function test_gifft_labeled_boolean() {
     undefined,
     Glean.testOnly.mirrorsForLabeledBools.__other__.testGetValue()
   );
-  Glean.testOnly.mirrorsForLabeledBools["1".repeat(72)].set(true);
+  Glean.testOnly.mirrorsForLabeledBools["1".repeat(112)].set(true);
   Assert.throws(
     () => Glean.testOnly.mirrorsForLabeledBools.__other__.testGetValue(),
     /DataError/,
@@ -357,7 +509,6 @@ add_task(async function test_gifft_labeled_boolean() {
     {
       a_label: true,
       another_label: false,
-      ["1".repeat(72)]: true,
     },
     value
   );
@@ -523,3 +674,154 @@ add_task(function test_gifft_url_cropped() {
     scalarValue("telemetry.test.mirror_for_url")
   );
 });
+
+add_task(function test_gifft_labeled_custom_dist() {
+  Glean.testOnly.mabelsCustomLabelLengths.rubbermaid.accumulateSamples([
+    7, 268435458,
+  ]);
+
+  let data = Glean.testOnly.mabelsCustomLabelLengths.rubbermaid.testGetValue();
+  Assert.equal(7 + 268435458, data.sum, "Sum's correct");
+  for (let [bucket, count] of Object.entries(data.values)) {
+    Assert.ok(
+      count == 0 || (count == 1 && (bucket == 1 || bucket == 268435456)),
+      `Only two buckets have a sample ${bucket} ${count}`
+    );
+  }
+
+  data = Telemetry.getKeyedHistogramById(
+    "TELEMETRY_TEST_KEYED_LINEAR"
+  ).snapshot();
+  Telemetry.getKeyedHistogramById("TELEMETRY_TEST_KEYED_LINEAR").clear();
+  Assert.ok("rubbermaid" in data, "Mirror has key");
+  Assert.equal(
+    7 + 268435458,
+    data.rubbermaid.sum,
+    "Sum in histogram is correct"
+  );
+  Assert.equal(1, data.rubbermaid.values["1"], "One sample in the low bucket");
+  Assert.equal(
+    1,
+    data.rubbermaid.values["250000"],
+    "One sample in the high bucket"
+  );
+});
+
+add_task(function test_gifft_labeled_memory_dist() {
+  Glean.testOnly.whatDoYouRemember.childhood.accumulate(7);
+  Glean.testOnly.whatDoYouRemember.childhood.accumulate(17);
+
+  let data = Glean.testOnly.whatDoYouRemember.childhood.testGetValue();
+  // `data.sum` is in bytes, but the metric is in MB.
+  Assert.equal(24 * 1024 * 1024, data.sum, "Sum's correct");
+  for (let [bucket, count] of Object.entries(data.values)) {
+    Assert.ok(
+      count == 0 || (count == 1 && (bucket == 7053950 || bucket == 17520006)),
+      `Only two buckets have a sample ${bucket} ${count}`
+    );
+  }
+
+  data = Telemetry.getKeyedHistogramById(
+    "TELEMETRY_TEST_MIRROR_FOR_LABELED_MEMORY"
+  ).snapshot();
+  Telemetry.getKeyedHistogramById(
+    "TELEMETRY_TEST_MIRROR_FOR_LABELED_MEMORY"
+  ).clear();
+  Assert.ok("childhood" in data, "Key's present");
+  Assert.equal(24, data.childhood.sum, "Histogram's in `memory_unit` units");
+  Assert.equal(2, data.childhood.values["1"], "Both samples in a low bucket");
+});
+
+add_task(async function test_gifft_labeled_timing_dist() {
+  let t1 = Glean.testOnly.whereHasTheTimeGone["down the drain"].start();
+  let t2 = Glean.testOnly.whereHasTheTimeGone["down the drain"].start();
+
+  await sleep(5);
+
+  let t3 = Glean.testOnly.whereHasTheTimeGone["down the drain"].start();
+  Glean.testOnly.whereHasTheTimeGone["down the drain"].cancel(t1);
+
+  await sleep(5);
+
+  Glean.testOnly.whereHasTheTimeGone["down the drain"].stopAndAccumulate(t2); // 10ms
+  Glean.testOnly.whereHasTheTimeGone["down the drain"].stopAndAccumulate(t3); // 5ms
+
+  let data =
+    Glean.testOnly.whereHasTheTimeGone["down the drain"].testGetValue();
+  const NANOS_IN_MILLIS = 1e6;
+  // bug 1701949 - Sleep gets close, but sometimes doesn't wait long enough.
+  const EPSILON = 40000;
+
+  // Variance in timing makes getting the sum impossible to know.
+  // 10 and 5 input value can be trunacted to 4. + 9. >= 13. from cast
+  Assert.greater(data.sum, 13 * NANOS_IN_MILLIS - EPSILON);
+
+  // No guarantees from timers means no guarantees on buckets.
+  // But we can guarantee it's only two samples.
+  Assert.equal(
+    2,
+    Object.entries(data.values).reduce((acc, [, count]) => acc + count, 0),
+    "Only two buckets with samples"
+  );
+
+  data = Telemetry.getKeyedHistogramById(
+    "TELEMETRY_TEST_MIRROR_FOR_LABELED_TIMING"
+  ).snapshot();
+  Assert.ok("down the drain" in data, "Has the key");
+  data = data["down the drain"];
+  // Suffers from same cast truncation issue of 9.... and 4.... values
+  Assert.greaterOrEqual(data.sum, 13, "Histogram's in milliseconds");
+  Assert.equal(
+    2,
+    Object.entries(data.values).reduce((acc, [, count]) => acc + count, 0),
+    "Only two samples"
+  );
+
+  // Ensure that `labeled_timing_distribution` mirrors calls to
+  // * accumulateSamples
+  // * accumulateSingleSamples
+  Glean.testOnly.whereHasTheTimeGone["hourglass sands"].accumulateSamples([
+    1, 2, 3,
+  ]);
+  Glean.testOnly.whereHasTheTimeGone["hourglass sands"].accumulateSingleSample(
+    1
+  );
+
+  data = Telemetry.getKeyedHistogramById(
+    "TELEMETRY_TEST_MIRROR_FOR_LABELED_TIMING"
+  ).snapshot();
+  Assert.ok("hourglass sands" in data, "Has the key");
+  data = data["hourglass sands"];
+  Assert.equal(data.sum, 7);
+});
+
+add_task(async function test_gifft_labeled_quantity() {
+  Assert.equal(
+    undefined,
+    Glean.testOnly.buttonJars.pants.testGetValue(),
+    "New labels with no values should return undefined"
+  );
+  Glean.testOnly.buttonJars.pants.set(42);
+  Glean.testOnly.buttonJars.whoseGot.set(1);
+  Assert.equal(42, Glean.testOnly.buttonJars.pants.testGetValue());
+  Assert.equal(1, Glean.testOnly.buttonJars.whoseGot.testGetValue());
+  // What about invalid/__other__?
+  Assert.equal(undefined, Glean.testOnly.buttonJars.__other__.testGetValue());
+  Glean.testOnly.buttonJars["1".repeat(112)].set(9000);
+  Assert.throws(
+    () => Glean.testOnly.buttonJars.__other__.testGetValue(),
+    /DataError/,
+    "Should throw because of a recording error."
+  );
+
+  info(JSON.stringify(Telemetry.getSnapshotForKeyedScalars()));
+  // In Telemetry there is no invalid label
+  let value = keyedScalarValue("telemetry.test.mirror_for_labeled_quantity");
+  Assert.deepEqual(
+    {
+      pants: 42,
+      whoseGot: 1,
+    },
+    value
+  );
+});
