@@ -50,7 +50,7 @@ add_task(async function test() {
       [{ login, usernameRequested }],
       async ({ login: addedLogin, usernameRequested: aUsernameRequested }) => {
         const { LoginFormFactory } = ChromeUtils.importESModule(
-          "resource://gre/modules/LoginFormFactory.sys.mjs"
+          "resource://gre/modules/shared/LoginFormFactory.sys.mjs"
         );
         const { LoginManagerChild } = ChromeUtils.importESModule(
           "resource://gre/modules/LoginManagerChild.sys.mjs"
@@ -93,7 +93,7 @@ add_task(async function test() {
     });
     await processedPromise;
 
-    let logins = Services.logins.getAllLogins();
+    let logins = await Services.logins.getAllLogins();
 
     Assert.equal(logins.length, 1, "There should only be one login saved");
     Assert.equal(
@@ -101,7 +101,7 @@ add_task(async function test() {
       login.guid,
       "The saved login should match the one added and used above"
     );
-    checkOnlyLoginWasUsedTwice({ justChanged: false });
+    await checkOnlyLoginWasUsedTwice({ justChanged: false });
 
     BrowserTestUtils.removeTab(tab);
 
