@@ -6,8 +6,7 @@ add_task(async function test_fetch_existent() {
   await PlacesUtils.bookmarks.eraseEverything();
 
   // Populate places and historyvisits.
-  let uriString = `http://mozilla.com/test_browserhistory/test_fetch`;
-  let uri = NetUtil.newURI(uriString);
+  let uri = NetUtil.newURI(`http://mozilla.com/test_browserhistory/test_fetch`);
   let title = `Test Visit ${Math.random()}`;
   let dates = [];
   let visits = [];
@@ -36,12 +35,14 @@ add_task(async function test_fetch_existent() {
   Assert.equal(await PlacesTestUtils.visitsInDB(uri), visits.length);
 
   // Store guid for further use in testing.
-  guid = await PlacesTestUtils.fieldInDB(uri, "guid");
+  guid = await PlacesTestUtils.getDatabaseValue("moz_places", "guid", {
+    url: uri,
+  });
   Assert.ok(guid, guid);
 
   // Initialize the objects to compare against.
   let idealPageInfo = {
-    url: new URL(uriString),
+    url: URL.fromURI(uri),
     guid,
     title,
   };
@@ -101,6 +102,11 @@ add_task(async function test_fetch_page_meta_info() {
     pageInfo.previewImageURL,
     "fetch should return a null previewImageURL"
   );
+  Assert.strictEqual(
+    "",
+    pageInfo.siteName,
+    "fetch should return a null siteName"
+  );
   Assert.equal(
     "",
     pageInfo.description,
@@ -109,11 +115,13 @@ add_task(async function test_fetch_page_meta_info() {
 
   // Now set the pageMetaInfo for this page
   let description = "Test description";
+  let siteName = "Mozilla";
   let previewImageURL = "http://mozilla.com/test_preview_image.png";
   await PlacesUtils.history.update({
     url: TEST_URI,
     description,
     previewImageURL,
+    siteName,
   });
 
   includeMeta = true;
@@ -123,6 +131,7 @@ add_task(async function test_fetch_page_meta_info() {
     pageInfo.previewImageURL.href,
     "fetch should return a previewImageURL"
   );
+  Assert.equal(siteName, pageInfo.siteName, "fetch should return a siteName");
   Assert.equal(
     description,
     pageInfo.description,
@@ -135,6 +144,10 @@ add_task(async function test_fetch_page_meta_info() {
     !("description" in pageInfo),
     "fetch should not return a description if includeMeta is false"
   );
+  Assert.ok(
+    !("siteName" in pageInfo),
+    "fetch should not return a siteName if includeMeta is false"
+  );
   Assert.ok(
     !("previewImageURL" in pageInfo),
     "fetch should not return a previewImageURL if includeMeta is false"
@@ -218,7 +231,7 @@ add_task(async function test_fetch_nonexistent() {
 add_task(async function test_error_cases() {
   Assert.throws(
     () => PlacesUtils.history.fetch("3"),
-    /TypeError: 3 is not a valid /
+    /TypeError: URL constructor: 3 is not a valid /
   );
   Assert.throws(
     () => PlacesUtils.history.fetch({ not: "a valid string or guid" }),
