@@ -3,8 +3,8 @@
 
 // Tests that history initialization correctly handles a corrupt places schema.
 
-add_task(async function() {
-  let path = await setupPlacesDatabase("../migration/favicons_v41.sqlite");
+add_task(async function () {
+  let path = await setupPlacesDatabase(["migration", "favicons_v41.sqlite"]);
 
   // Ensure the database will go through a migration that depends on moz_places
   // and break the schema by dropping that table.
