@@ -22,7 +22,11 @@ regenerate() {
 cd $(dirname $0)
 cd ../../.. # Top level.
 
-regenerate gfx/layers/protobuf/ LayerScopePacket.proto
 regenerate devtools/shared/heapsnapshot/ CoreDump.proto
-regenerate toolkit/components/downloads/chromium/chrome/common/safe_browsing/ csd.proto
+regenerate toolkit/components/reputationservice/chromium/chrome/common/safe_browsing/ csd.proto
 regenerate toolkit/components/url-classifier/chromium/ safebrowsing.proto
+regenerate toolkit/components/cookiebanners/ cookieBanner.proto
+command cp third_party/content_analysis_sdk/proto/content_analysis/sdk/analysis.proto toolkit/components/contentanalysis/content_analysis/sdk/analysis.proto
+regenerate toolkit/components/contentanalysis/content_analysis/sdk/ analysis.proto
+command cp third_party/rust/viaduct/src/fetch_msg_types.proto toolkit/components/viaduct/fetch_msg_types.proto
+regenerate toolkit/components/viaduct/ fetch_msg_types.proto
