@@ -4,7 +4,9 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-const { NetUtil } = ChromeUtils.import("resource://gre/modules/NetUtil.jsm");
+const { NetUtil } = ChromeUtils.importESModule(
+  "resource://gre/modules/NetUtil.sys.mjs"
+);
 
 const gAppRep = Cc[
   "@mozilla.org/reputationservice/application-reputation-service;1"
@@ -113,7 +115,7 @@ add_task(async function test_setup() {
 
   gHttpServ = new HttpServer();
   gHttpServ.registerDirectory("/", do_get_cwd());
-  gHttpServ.registerPathHandler("/download", function (request, response) {
+  gHttpServ.registerPathHandler("/download", function (request) {
     if (gExpectedRemote) {
       let body = NetUtil.readInputStreamToString(
         request.bodyInputStream,
