@@ -1,9 +1,10 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
+const CONFIG = [{ identifier: "engine-1" }, { identifier: "engine-2" }];
+
 add_setup(async function () {
-  await AddonTestUtils.promiseStartupManager();
-  await SearchTestUtils.useTestEngines("simple-engines");
+  SearchTestUtils.setRemoteSettingsConfig(CONFIG);
   Services.fog.initializeFOG();
 });
 
@@ -19,11 +20,11 @@ add_task(async function test_async() {
   Assert.equal(engines.length, 2);
 
   // test jar engine is loaded ok.
-  let engine = Services.search.getEngineByName("basic");
+  let engine = Services.search.getEngineByName("engine-1");
   Assert.notEqual(engine, null);
   Assert.ok(engine.isAppProvided, "Should be shown as an app-provided engine");
 
-  engine = Services.search.getEngineByName("Simple Engine");
+  engine = Services.search.getEngineByName("engine-2");
   Assert.notEqual(engine, null);
   Assert.ok(engine.isAppProvided, "Should be shown as an app-provided engine");
 
