@@ -9,11 +9,12 @@ const { AppConstants } = ChromeUtils.importESModule(
 );
 
 ChromeUtils.defineESModuleGetters(this, {
+  AddonManager: "resource://gre/modules/AddonManager.sys.mjs",
   AddonTestUtils: "resource://testing-common/AddonTestUtils.sys.mjs",
   FileUtils: "resource://gre/modules/FileUtils.sys.mjs",
+  HttpServer: "resource://testing-common/httpd.sys.mjs",
   Log: "resource://gre/modules/Log.sys.mjs",
-  Preferences: "resource://gre/modules/Preferences.sys.mjs",
-  PromiseUtils: "resource://gre/modules/PromiseUtils.sys.mjs",
+  NetUtil: "resource://gre/modules/NetUtil.sys.mjs",
   TelemetryController: "resource://gre/modules/TelemetryController.sys.mjs",
   TelemetryScheduler: "resource://gre/modules/TelemetryScheduler.sys.mjs",
   TelemetrySend: "resource://gre/modules/TelemetrySend.sys.mjs",
@@ -21,11 +22,6 @@ ChromeUtils.defineESModuleGetters(this, {
   TelemetryUtils: "resource://gre/modules/TelemetryUtils.sys.mjs",
 });
 
-XPCOMUtils.defineLazyModuleGetters(this, {
-  HttpServer: "resource://testing-common/httpd.js",
-  NetUtil: "resource://gre/modules/NetUtil.jsm",
-});
-
 const gIsWindows = AppConstants.platform == "win";
 const gIsMac = AppConstants.platform == "macosx";
 const gIsAndroid = AppConstants.platform == "android";
@@ -48,7 +44,7 @@ var gGlobalScope = this;
 const PingServer = {
   _httpServer: null,
   _started: false,
-  _defers: [PromiseUtils.defer()],
+  _defers: [Promise.withResolvers()],
   _currentDeferred: 0,
   _logger: null,
 
@@ -81,13 +77,13 @@ const PingServer = {
   },
 
   resetPingHandler() {
-    this.registerPingHandler((request, response) => {
+    this.registerPingHandler(request => {
       let r = request;
       this._log.trace(
         `defaultPingHandler() - ${r.method} ${r.scheme}://${r.host}:${r.port}${r.path}`
       );
       let deferred = this._defers[this._defers.length - 1];
-      this._defers.push(PromiseUtils.defer());
+      this._defers.push(Promise.withResolvers());
       deferred.resolve(request);
     });
   },
@@ -108,7 +104,7 @@ const PingServer = {
   },
 
   clearRequests() {
-    this._defers = [PromiseUtils.defer()];
+    this._defers = [Promise.withResolvers()];
     this._currentDeferred = 0;
   },
 
@@ -206,6 +202,11 @@ function decodeRequestPayload(request) {
       payload.clientId,
       `Known clientId shouldn't appear in a "${payload.type}" ping on the server.`
     );
+
+    Assert.ok(
+      "profileGroupId" in payload,
+      "Pings with a clientId must also contain a profileGroupId"
+    );
   }
 
   return payload;
@@ -290,14 +291,44 @@ async function loadAddonManager(...args) {
   AddonTestUtils.overrideCertDB();
   createAppInfo(...args);
 
-  // As we're not running in application, we need to setup the features directory
-  // used by system add-ons.
-  const distroDir = FileUtils.getDir("ProfD", ["sysfeatures", "app0"], true);
-  AddonTestUtils.registerDirectory("XREAppFeat", distroDir);
-  await AddonTestUtils.overrideBuiltIns({
-    system: ["tel-system-xpi@tests.mozilla.org"],
-  });
-  return AddonTestUtils.promiseStartupManager();
+  // As we're not running in application, we need to setup the built-in
+  // add-ons to reseamble a setup similar to a Firefox Desktop instance.
+
+  // Enable SCOPE_APPLICATION for builtin testing.  Default in tests is only SCOPE_PROFILE.
+  let scopes = AddonManager.SCOPE_PROFILE | AddonManager.SCOPE_APPLICATION;
+  Services.prefs.setIntPref("extensions.enabledScopes", scopes);
+
+  // Disable XPIProvider auto-installed default theme logic
+  // for the unit tests using this helper.
+  Services.prefs.setBoolPref(
+    "extensions.skipInstallDefaultThemeForTests",
+    true
+  );
+
+  // NOTE: keep the addon id and version in sync with the content of
+  // toolkit/components/telemetry/tests/addons/system/manifest.json
+  const addon_id = "tel-system-xpi@tests.mozilla.org";
+  const addon_version = "1.0";
+  const addon_res_url_path = "telemetry-test-builtin-addon";
+  // The built-in location requires a resource: URL that maps to a
+  // jar: or file: URL.  This would typically be something bundled
+  // into omni.ja but for testing we just use a temp file.
+  const xpi = do_get_file("system.xpi");
+  let base = Services.io.newURI(`jar:file:${xpi.path}!/`);
+  let resProto = Services.io
+    .getProtocolHandler("resource")
+    .QueryInterface(Ci.nsIResProtocolHandler);
+  resProto.setSubstitution(addon_res_url_path, base);
+  let builtins = [
+    {
+      addon_id,
+      addon_version,
+      res_url: `resource://${addon_res_url_path}/`,
+    },
+  ];
+  await AddonTestUtils.overrideBuiltIns({ builtins });
+  await AddonTestUtils.promiseStartupManager();
+  return { builtins };
 }
 
 function finishAddonManagerStartup() {
@@ -415,7 +446,7 @@ function fakeGzipCompressStringForNextPing(length) {
     "resource://gre/modules/TelemetrySend.sys.mjs"
   );
   let largePayload = generateString(length);
-  Policy.gzipCompressString = data => {
+  Policy.gzipCompressString = () => {
     Policy.gzipCompressString = gzipCompressString;
     return largePayload;
   };
@@ -548,7 +579,7 @@ if (runningInParent) {
   }
 
   fakePingSendTimer(
-    (callback, timeout) => {
+    callback => {
       Services.tm.dispatchToMainThread(() => callback());
     },
     () => {}
@@ -582,6 +613,3 @@ const DISTRIBUTION_CUSTOMIZATION_COMPLETE_TOPIC =
 const PLUGIN2_NAME = "Quicktime";
 const PLUGIN2_DESC = "A mock Quicktime plugin";
 const PLUGIN2_VERSION = "2.3";
-//
-// system add-ons are enabled at startup, so record date when the test starts
-const SYSTEM_ADDON_INSTALL_DATE = Date.now();
