@@ -2,7 +2,7 @@
 
 let tempFile;
 add_setup(async function () {
-  await SpecialPowers.pushPrefEnv({ set: [["ui.tooltipDelay", 0]] });
+  await SpecialPowers.pushPrefEnv({ set: [["ui.tooltip.delay_ms", 0]] });
   tempFile = createTempFile();
   registerCleanupFunction(function () {
     tempFile.remove(true);
@@ -63,9 +63,9 @@ async function do_test(test) {
   if (test.value) {
     info("Creating mock filepicker to select files");
     let MockFilePicker = SpecialPowers.MockFilePicker;
-    MockFilePicker.init(window);
+    MockFilePicker.init(window.browsingContext);
     MockFilePicker.returnValue = MockFilePicker.returnOK;
-    MockFilePicker.displayDirectory = FileUtils.getDir("TmpD", [], false);
+    MockFilePicker.displayDirectory = FileUtils.getDir("TmpD", []);
     MockFilePicker.setFiles([tempFile]);
     MockFilePicker.afterOpenCallback = MockFilePicker.cleanup;
 
@@ -124,7 +124,7 @@ async function do_test(test) {
 }
 
 function createTempFile() {
-  let file = FileUtils.getDir("TmpD", [], false);
+  let file = FileUtils.getDir("TmpD", []);
   file.append("testfile_bug1251809");
   file.create(Ci.nsIFile.NORMAL_FILE_TYPE, 0o644);
   return file;
