@@ -13,11 +13,9 @@
 #include "nsIObserver.h"
 #include "nsUrlClassifierPrefixSet.h"
 #include "nsIUrlClassifierHashCompleter.h"
-#include "nsIUrlListManager.h"
 #include "nsIUrlClassifierDBService.h"
 #include "nsIUrlClassifierInfo.h"
 #include "nsIURIClassifier.h"
-#include "nsICryptoHMAC.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/Mutex.h"
 #include "mozilla/TimeStamp.h"
@@ -26,13 +24,6 @@
 #include "LookupCache.h"
 #include "HashStore.h"
 
-// GCC < 6.1 workaround, see bug 1329593
-#if defined(XP_WIN) && defined(__MINGW32__)
-#  define GCC_MANGLING_WORKAROUND __stdcall
-#else
-#  define GCC_MANGLING_WORKAROUND
-#endif
-
 // The hash length for a domain key.
 #define DOMAIN_LENGTH 4
 
@@ -87,7 +78,7 @@ class nsUrlClassifierDBService final : public nsIUrlClassifierDBService,
   static already_AddRefed<nsUrlClassifierDBService> GetInstance(
       nsresult* result);
 
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_URLCLASSIFIERDBSERVICE_CID)
+  NS_INLINE_DECL_STATIC_IID(NS_URLCLASSIFIERDBSERVICE_CID)
 
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIURLCLASSIFIERDBSERVICE
@@ -180,22 +171,22 @@ class nsUrlClassifierDBServiceWorker final : public nsIUrlClassifierDBService {
       LookupResultArray& aResults);
 
   // Open the DB connection
-  nsresult GCC_MANGLING_WORKAROUND OpenDb();
+  nsresult OpenDb();
 
   // Provide a way to forcibly close the db connection.
-  nsresult GCC_MANGLING_WORKAROUND CloseDb();
+  nsresult CloseDb();
 
-  nsresult GCC_MANGLING_WORKAROUND PreShutdown();
+  nsresult PreShutdown();
 
   nsresult CacheCompletions(const ConstCacheResultArray& aEntries);
 
   // Used to probe the state of the worker thread. When the update begins,
   // mUpdateObserver will be set. When the update finished, mUpdateObserver
   // will be nulled out in NotifyUpdateObserver.
-  bool IsBusyUpdating() const { return !!mUpdateObserver; }
-
-  // Check the DB ready state of the worker thread
-  bool IsDBOpened() const { return !!mClassifier; }
+  bool IsBusyUpdating() {
+    mozilla::MutexAutoLock lock(mUpdateObserverLock);
+    return !!mUpdateObserver;
+  }
 
   // Delegate Classifier to disable async update. If there is an
   // ongoing update on the update thread, we will be blocked until
@@ -255,7 +246,12 @@ class nsUrlClassifierDBServiceWorker final : public nsIUrlClassifierDBService {
   nsresult mUpdateStatus;
   nsTArray<nsCString> mUpdateTables;
 
-  nsCOMPtr<nsIUrlClassifierUpdateObserver> mUpdateObserver;
+  // The mUpdateObserver will be accessed by both the main thread and the worker
+  // thread. The lock is used to protect the mUpdateObserver.
+  mozilla::Mutex mUpdateObserverLock;
+
+  nsCOMPtr<nsIUrlClassifierUpdateObserver> mUpdateObserver
+      MOZ_GUARDED_BY(mUpdateObserverLock);
   bool mInStream;
 
   // The number of noise entries to add to the set of lookup results.
@@ -263,7 +259,7 @@ class nsUrlClassifierDBServiceWorker final : public nsIUrlClassifierDBService {
 
   // Pending lookups are stored in a queue for processing.  The queue
   // is protected by mPendingLookupLock.
-  mozilla::Mutex mPendingLookupLock;
+  mozilla::Mutex mPendingLookupLock MOZ_UNANNOTATED;
 
   class PendingLookup {
    public:
@@ -275,14 +271,6 @@ class nsUrlClassifierDBServiceWorker final : public nsIUrlClassifierDBService {
 
   // list of pending lookups
   nsTArray<PendingLookup> mPendingLookups;
-
-#ifdef MOZ_SAFEBROWSING_DUMP_FAILED_UPDATES
-  // The raw update response for debugging.
-  nsCString mRawTableUpdates;
-#endif
 };
 
-NS_DEFINE_STATIC_IID_ACCESSOR(nsUrlClassifierDBService,
-                              NS_URLCLASSIFIERDBSERVICE_CID)
-
 #endif  // nsUrlClassifierDBService_h_
