@@ -14,29 +14,28 @@ add_task(async function test_chrome_opens_window() {
   });
 
   let newWinPromise = BrowserTestUtils.waitForNewWindow({
-    url: "http://example.com/",
+    url: "https://example.com/",
   });
 
-  await ContentTask.spawn(gBrowser.selectedBrowser, null, async function() {
-    content.open("http://example.com/", "_blank");
+  await SpecialPowers.spawn(gBrowser.selectedBrowser, [], async function () {
+    content.open("https://example.com/", "_blank");
   });
 
   let win = await newWinPromise;
   let browser = win.gBrowser.selectedBrowser;
-  Assert.equal(
-    browser.remoteType,
-    "web",
+  Assert.ok(
+    E10SUtils.isWebRemoteType(browser.remoteType),
     "Should have the default content remote type."
   );
 
-  await ContentTask.spawn(browser, null, async function() {
+  await SpecialPowers.spawn(browser, [], async function () {
     Assert.ok(
       !content.document.nodePrincipal.isSystemPrincipal,
       "We should not have a system principal."
     );
     Assert.equal(
       content.document.nodePrincipal.origin,
-      "http://example.com",
+      "https://example.com",
       "Should have the example.com principal"
     );
   });
