@@ -5,6 +5,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsNativeAppSupportBase.h"
+#include "nsComponentManagerUtils.h"
 #include "nsCOMPtr.h"
 #include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
@@ -17,15 +18,11 @@
 
 #include "nsIFile.h"
 #include "nsDirectoryServiceDefs.h"
-#include "nsICommandLineRunner.h"
-#include "nsIWindowMediator.h"
 #include "nsPIDOMWindow.h"
-#include "nsIDocShell.h"
-#include "nsIBaseWindow.h"
 #include "nsIWidget.h"
-#include "nsIWritablePropertyBag2.h"
-#include "nsIPrefService.h"
+#include "mozilla/Logging.h"
 #include "mozilla/Services.h"
+#include "mozilla/XREAppData.h"
 
 #include <stdlib.h>
 #include <glib.h>
@@ -33,52 +30,55 @@
 #include <gtk/gtk.h>
 
 #ifdef MOZ_X11
-#include <gdk/gdkx.h>
-#include <X11/ICE/ICElib.h>
-#include <X11/SM/SMlib.h>
-#include <fcntl.h>
-#include "nsThreadUtils.h"
+#  include <gdk/gdkx.h>
+#  include <X11/ICE/ICElib.h>
+#  include <X11/SM/SMlib.h>
+#  include <fcntl.h>
+#  include "nsThreadUtils.h"
 
-#include <pwd.h>
+#  include <pwd.h>
 #endif
 
 #ifdef MOZ_ENABLE_DBUS
-#include <dbus/dbus.h>
+#  include <dbus/dbus.h>
 #endif
 
 #define MIN_GTK_MAJOR_VERSION 2
 #define MIN_GTK_MINOR_VERSION 10
-#define UNSUPPORTED_GTK_MSG "We're sorry, this application requires a version of the GTK+ library that is not installed on your computer.\n\n\
+#define UNSUPPORTED_GTK_MSG \
+  "We're sorry, this application requires a version of the GTK+ library that is not installed on your computer.\n\n\
 You have GTK+ %d.%d.\nThis application requires GTK+ %d.%d or newer.\n\n\
 Please upgrade your GTK+ library if you wish to use this application."
 
 #if MOZ_X11
-#undef IceSetIOErrorHandler
-#undef IceAddConnectionWatch
-#undef IceConnectionNumber
-#undef IceProcessMessages
-#undef IceGetConnectionContext
-#undef SmcInteractDone
-#undef SmcSaveYourselfDone
-#undef SmcInteractRequest
-#undef SmcCloseConnection
-#undef SmcOpenConnection
-#undef SmcSetProperties
+#  undef IceSetIOErrorHandler
+#  undef IceAddConnectionWatch
+#  undef IceConnectionNumber
+#  undef IceProcessMessages
+#  undef IceGetConnectionContext
+#  undef SmcInteractDone
+#  undef SmcSaveYourselfDone
+#  undef SmcInteractRequest
+#  undef SmcCloseConnection
+#  undef SmcOpenConnection
+#  undef SmcSetProperties
 
-typedef IceIOErrorHandler (*IceSetIOErrorHandlerFn) (IceIOErrorHandler);
-typedef int (*IceAddConnectionWatchFn) (IceWatchProc, IcePointer);
-typedef int (*IceConnectionNumberFn) (IceConn);
-typedef IceProcessMessagesStatus (*IceProcessMessagesFn) (IceConn, IceReplyWaitInfo*, Bool*);
-typedef IcePointer (*IceGetConnectionContextFn) (IceConn);
+typedef IceIOErrorHandler (*IceSetIOErrorHandlerFn)(IceIOErrorHandler);
+typedef int (*IceAddConnectionWatchFn)(IceWatchProc, IcePointer);
+typedef int (*IceConnectionNumberFn)(IceConn);
+typedef IceProcessMessagesStatus (*IceProcessMessagesFn)(IceConn,
+                                                         IceReplyWaitInfo*,
+                                                         Bool*);
+typedef IcePointer (*IceGetConnectionContextFn)(IceConn);
 
-typedef void (*SmcInteractDoneFn) (SmcConn, Bool);
-typedef void (*SmcSaveYourselfDoneFn) (SmcConn, Bool);
-typedef int (*SmcInteractRequestFn) (SmcConn, int, SmcInteractProc, SmPointer);
-typedef SmcCloseStatus (*SmcCloseConnectionFn) (SmcConn, int, char**);
-typedef SmcConn (*SmcOpenConnectionFn) (char*, SmPointer, int, int,
-                                        unsigned long, SmcCallbacks*,
-                                        const char*, char**, int, char*);
-typedef void (*SmcSetPropertiesFn) (SmcConn, int, SmProp**);
+typedef void (*SmcInteractDoneFn)(SmcConn, Bool);
+typedef void (*SmcSaveYourselfDoneFn)(SmcConn, Bool);
+typedef int (*SmcInteractRequestFn)(SmcConn, int, SmcInteractProc, SmPointer);
+typedef SmcCloseStatus (*SmcCloseConnectionFn)(SmcConn, int, char**);
+typedef SmcConn (*SmcOpenConnectionFn)(char*, SmPointer, int, int,
+                                       unsigned long, SmcCallbacks*,
+                                       const char*, char**, int, char*);
+typedef void (*SmcSetPropertiesFn)(SmcConn, int, SmProp**);
 
 static IceSetIOErrorHandlerFn IceSetIOErrorHandlerPtr;
 static IceAddConnectionWatchFn IceAddConnectionWatchPtr;
@@ -92,17 +92,17 @@ static SmcCloseConnectionFn SmcCloseConnectionPtr;
 static SmcOpenConnectionFn SmcOpenConnectionPtr;
 static SmcSetPropertiesFn SmcSetPropertiesPtr;
 
-#define IceSetIOErrorHandler IceSetIOErrorHandlerPtr
-#define IceAddConnectionWatch IceAddConnectionWatchPtr
-#define IceConnectionNumber IceConnectionNumberPtr
-#define IceProcessMessages IceProcessMessagesPtr
-#define IceGetConnectionContext IceGetConnectionContextPtr
-#define SmcInteractDone SmcInteractDonePtr
-#define SmcSaveYourselfDone SmcSaveYourselfDonePtr
-#define SmcInteractRequest SmcInteractRequestPtr
-#define SmcCloseConnection SmcCloseConnectionPtr
-#define SmcOpenConnection SmcOpenConnectionPtr
-#define SmcSetProperties SmcSetPropertiesPtr
+#  define IceSetIOErrorHandler IceSetIOErrorHandlerPtr
+#  define IceAddConnectionWatch IceAddConnectionWatchPtr
+#  define IceConnectionNumber IceConnectionNumberPtr
+#  define IceProcessMessages IceProcessMessagesPtr
+#  define IceGetConnectionContext IceGetConnectionContextPtr
+#  define SmcInteractDone SmcInteractDonePtr
+#  define SmcSaveYourselfDone SmcSaveYourselfDonePtr
+#  define SmcInteractRequest SmcInteractRequestPtr
+#  define SmcCloseConnection SmcCloseConnectionPtr
+#  define SmcOpenConnection SmcOpenConnectionPtr
+#  define SmcSetProperties SmcSetPropertiesPtr
 
 enum ClientState {
   STATE_DISCONNECTED,
@@ -112,25 +112,20 @@ enum ClientState {
   STATE_SHUTDOWN_CANCELLED
 };
 
-static const char *gClientStateTable[] = {
-  "DISCONNECTED",
-  "REGISTERING",
-  "IDLE",
-  "INTERACTING",
-  "SHUTDOWN_CANCELLED"
-};
+using namespace mozilla;
+
+static const char* gClientStateTable[] = {"DISCONNECTED", "REGISTERING", "IDLE",
+                                          "INTERACTING", "SHUTDOWN_CANCELLED"};
 
 static LazyLogModule sMozSMLog("MozSM");
 #endif /* MOZ_X11 */
 
-class nsNativeAppSupportUnix : public nsNativeAppSupportBase
-{
-public:
+class nsNativeAppSupportUnix : public nsNativeAppSupportBase {
+ public:
 #if MOZ_X11
-  nsNativeAppSupportUnix(): mSessionConnection(nullptr),
-                            mClientState(STATE_DISCONNECTED) {};
-  ~nsNativeAppSupportUnix()
-  {
+  nsNativeAppSupportUnix()
+      : mSessionConnection(nullptr), mClientState(STATE_DISCONNECTED) {};
+  ~nsNativeAppSupportUnix() {
     // this goes out of scope after "web-workers-shutdown" async shutdown phase
     // so it's safe to disconnect here (i.e. the application won't lose data)
     DisconnectFromSM();
@@ -139,10 +134,9 @@ public:
   void DisconnectFromSM();
 #endif
   NS_IMETHOD Start(bool* aRetVal) override;
-  NS_IMETHOD Stop(bool *aResult) override;
   NS_IMETHOD Enable() override;
 
-private:
+ private:
 #if MOZ_X11
   static void SaveYourselfCB(SmcConn smc_conn, SmPointer client_data,
                              int save_style, Bool shutdown, int interact_style,
@@ -152,10 +146,10 @@ private:
   static void SaveCompleteCB(SmcConn smc_conn, SmPointer client_data) {};
   static void ShutdownCancelledCB(SmcConn smc_conn, SmPointer client_data);
   void DoInteract();
-  void SetClientState(ClientState aState)
-  {
+  void SetClientState(ClientState aState) {
     mClientState = aState;
-    MOZ_LOG(sMozSMLog, LogLevel::Debug, ("New state = %s\n", gClientStateTable[aState]));
+    MOZ_LOG(sMozSMLog, LogLevel::Debug,
+            ("New state = %s\n", gClientStateTable[aState]));
   }
 
   SmcConn mSessionConnection;
@@ -164,54 +158,48 @@ private:
 };
 
 #if MOZ_X11
-static gboolean
-process_ice_messages(IceConn connection)
-{
+static gboolean process_ice_messages(IceConn connection) {
   IceProcessMessagesStatus status;
 
   status = IceProcessMessages(connection, nullptr, nullptr);
 
   switch (status) {
-  case IceProcessMessagesSuccess:
-    return TRUE;
+    case IceProcessMessagesSuccess:
+      return TRUE;
 
-  case IceProcessMessagesIOError: {
-      nsNativeAppSupportUnix *native =
-        static_cast<nsNativeAppSupportUnix *>(IceGetConnectionContext(connection));
+    case IceProcessMessagesIOError: {
+      nsNativeAppSupportUnix* native = static_cast<nsNativeAppSupportUnix*>(
+          IceGetConnectionContext(connection));
       native->DisconnectFromSM();
     }
-    return FALSE;
+      return FALSE;
 
-  case IceProcessMessagesConnectionClosed:
-    return FALSE;
+    case IceProcessMessagesConnectionClosed:
+      return FALSE;
 
-  default:
-    g_assert_not_reached ();
+    default:
+      g_assert_not_reached();
   }
 }
 
-static gboolean
-ice_iochannel_watch(GIOChannel *channel, GIOCondition condition,
-                    gpointer client_data)
-{
+static gboolean ice_iochannel_watch(GIOChannel* channel, GIOCondition condition,
+                                    gpointer client_data) {
   return process_ice_messages(static_cast<IceConn>(client_data));
 }
 
-static void
-ice_connection_watch(IceConn connection, IcePointer  client_data,
-                     Bool opening, IcePointer *watch_data)
-{
+static void ice_connection_watch(IceConn connection, IcePointer client_data,
+                                 Bool opening, IcePointer* watch_data) {
   guint watch_id;
 
   if (opening) {
-    GIOChannel *channel;
+    GIOChannel* channel;
     int fd = IceConnectionNumber(connection);
 
     fcntl(fd, F_SETFD, fcntl(fd, F_GETFD, 0) | FD_CLOEXEC);
     channel = g_io_channel_unix_new(fd);
-    watch_id = g_io_add_watch(channel,
-                              static_cast<GIOCondition>(G_IO_IN | G_IO_ERR),
-                              ice_iochannel_watch, connection);
+    watch_id =
+        g_io_add_watch(channel, static_cast<GIOCondition>(G_IO_IN | G_IO_ERR),
+                       ice_iochannel_watch, connection);
     g_io_channel_unref(channel);
 
     *watch_data = GUINT_TO_POINTER(watch_id);
@@ -221,16 +209,12 @@ ice_connection_watch(IceConn connection, IcePointer  client_data,
   }
 }
 
-static void
-ice_io_error_handler(IceConn connection)
-{
+static void ice_io_error_handler(IceConn connection) {
   // override the default handler which would exit the application;
   // do nothing and let ICELib handle the failure of the connection gracefully.
 }
 
-static void
-ice_init(void)
-{
+static void ice_init(void) {
   static bool initted = false;
 
   if (!initted) {
@@ -240,11 +224,10 @@ ice_init(void)
   }
 }
 
-void
-nsNativeAppSupportUnix::InteractCB(SmcConn smc_conn, SmPointer client_data)
-{
-  nsNativeAppSupportUnix *self =
-    static_cast<nsNativeAppSupportUnix *>(client_data);
+void nsNativeAppSupportUnix::InteractCB(SmcConn smc_conn,
+                                        SmPointer client_data) {
+  nsNativeAppSupportUnix* self =
+      static_cast<nsNativeAppSupportUnix*>(client_data);
 
   self->SetClientState(STATE_INTERACTING);
 
@@ -255,16 +238,13 @@ nsNativeAppSupportUnix::InteractCB(SmcConn smc_conn, SmPointer client_data)
   // with a pending ShutdownCancelled, and we would certainly like to handle Die
   // whilst a dialog is displayed
   NS_DispatchToCurrentThread(
-    NewRunnableMethod("nsNativeAppSupportUnix::DoInteract",
-                      self,
-                      &nsNativeAppSupportUnix::DoInteract));
+      NewRunnableMethod("nsNativeAppSupportUnix::DoInteract", self,
+                        &nsNativeAppSupportUnix::DoInteract));
 }
 
-void
-nsNativeAppSupportUnix::DoInteract()
-{
+void nsNativeAppSupportUnix::DoInteract() {
   nsCOMPtr<nsIObserverService> obsServ =
-    mozilla::services::GetObserverService();
+      mozilla::services::GetObserverService();
   if (!obsServ) {
     SmcInteractDone(mSessionConnection, False);
     SmcSaveYourselfDone(mSessionConnection, True);
@@ -273,7 +253,7 @@ nsNativeAppSupportUnix::DoInteract()
   }
 
   nsCOMPtr<nsISupportsPRBool> cancelQuit =
-    do_CreateInstance(NS_SUPPORTS_PRBOOL_CONTRACTID);
+      do_CreateInstance(NS_SUPPORTS_PRBOOL_CONTRACTID);
 
   bool abortQuit = false;
   if (cancelQuit) {
@@ -287,10 +267,11 @@ nsNativeAppSupportUnix::DoInteract()
     // The session manager disappeared, whilst we were interacting, so
     // quit now
     nsCOMPtr<nsIAppStartup> appService =
-      do_GetService("@mozilla.org/toolkit/app-startup;1");
+        do_GetService("@mozilla.org/toolkit/app-startup;1");
 
     if (appService) {
-      appService->Quit(nsIAppStartup::eForceQuit);
+      bool userAllowedQuit = true;
+      appService->Quit(nsIAppStartup::eForceQuit, 0, &userAllowedQuit);
     }
   } else {
     if (mClientState != STATE_SHUTDOWN_CANCELLED) {
@@ -303,13 +284,12 @@ nsNativeAppSupportUnix::DoInteract()
   }
 }
 
-void
-nsNativeAppSupportUnix::SaveYourselfCB(SmcConn smc_conn, SmPointer client_data,
-                                       int save_style, Bool shutdown,
-                                       int interact_style, Bool fast)
-{
-  nsNativeAppSupportUnix *self =
-    static_cast<nsNativeAppSupportUnix *>(client_data);
+void nsNativeAppSupportUnix::SaveYourselfCB(SmcConn smc_conn,
+                                            SmPointer client_data,
+                                            int save_style, Bool shutdown,
+                                            int interact_style, Bool fast) {
+  nsNativeAppSupportUnix* self =
+      static_cast<nsNativeAppSupportUnix*>(client_data);
 
   // Expect a SaveYourselfCB if we're registering a new client.
   // All properties are already set in Start() so just reply with
@@ -339,29 +319,27 @@ nsNativeAppSupportUnix::SaveYourselfCB(SmcConn smc_conn, SmPointer client_data,
   }
 
   nsCOMPtr<nsIObserverService> obsServ =
-    mozilla::services::GetObserverService();
+      mozilla::services::GetObserverService();
   if (!obsServ) {
     SmcSaveYourselfDone(smc_conn, True);
     return;
   }
 
   bool status = false;
-  if (save_style != SmSaveGlobal) {
-    nsCOMPtr<nsISupportsPRBool> didSaveSession =
+  nsCOMPtr<nsISupportsPRBool> didSaveSession =
       do_CreateInstance(NS_SUPPORTS_PRBOOL_CONTRACTID);
 
-    if (!didSaveSession) {
-      SmcSaveYourselfDone(smc_conn, True);
-      return;
-    }
-
-    // Notify observers to save the session state
-    didSaveSession->SetData(false);
-    obsServ->NotifyObservers(didSaveSession, "session-save", nullptr);
-
-    didSaveSession->GetData(&status);
+  if (!didSaveSession) {
+    SmcSaveYourselfDone(smc_conn, True);
+    return;
   }
 
+  // Notify observers to save the session state
+  didSaveSession->SetData(false);
+  obsServ->NotifyObservers(didSaveSession, "session-save", nullptr);
+
+  didSaveSession->GetData(&status);
+
   // If the interact style permits us to, we are shutting down and we didn't
   // manage to (or weren't asked to) save the local state, then notify the user
   // in advance that we are doing to quit (assuming that we aren't already
@@ -376,25 +354,22 @@ nsNativeAppSupportUnix::SaveYourselfCB(SmcConn smc_conn, SmPointer client_data,
   }
 }
 
-void
-nsNativeAppSupportUnix::DieCB(SmcConn smc_conn, SmPointer client_data)
-{
+void nsNativeAppSupportUnix::DieCB(SmcConn smc_conn, SmPointer client_data) {
   nsCOMPtr<nsIAppStartup> appService =
-    do_GetService("@mozilla.org/toolkit/app-startup;1");
+      do_GetService("@mozilla.org/toolkit/app-startup;1");
 
   if (appService) {
-    appService->Quit(nsIAppStartup::eForceQuit);
+    bool userAllowedQuit = false;
+    appService->Quit(nsIAppStartup::eForceQuit, 0, &userAllowedQuit);
   }
   // Quit causes the shutdown to begin but the shutdown process is asynchronous
   // so we can't DisconnectFromSM() yet
 }
 
-void
-nsNativeAppSupportUnix::ShutdownCancelledCB(SmcConn smc_conn,
-                                            SmPointer client_data)
-{
-  nsNativeAppSupportUnix *self =
-    static_cast<nsNativeAppSupportUnix *>(client_data);
+void nsNativeAppSupportUnix::ShutdownCancelledCB(SmcConn smc_conn,
+                                                 SmPointer client_data) {
+  nsNativeAppSupportUnix* self =
+      static_cast<nsNativeAppSupportUnix*>(client_data);
 
   // Interacting is the only time when we wouldn't already have called
   // SmcSaveYourselfDone. Do that now, then set the state to make sure we
@@ -405,9 +380,7 @@ nsNativeAppSupportUnix::ShutdownCancelledCB(SmcConn smc_conn,
   }
 }
 
-void
-nsNativeAppSupportUnix::DisconnectFromSM()
-{
+void nsNativeAppSupportUnix::DisconnectFromSM() {
   // the SM is free to exit any time after we disconnect, so callers must be
   // sure to have reached a sufficiently advanced phase of shutdown that there
   // is no risk of data loss:
@@ -420,17 +393,13 @@ nsNativeAppSupportUnix::DisconnectFromSM()
   }
 }
 
-static void
-SetSMValue(SmPropValue& val, const nsCString& data)
-{
+static void SetSMValue(SmPropValue& val, const nsCString& data) {
   val.value = static_cast<SmPointer>(const_cast<char*>(data.get()));
   val.length = data.Length();
 }
 
-static void
-SetSMProperty(SmProp& prop, const char* name, const char* type, int numVals,
-              SmPropValue vals[])
-{
+static void SetSMProperty(SmProp& prop, const char* name, const char* type,
+                          int numVals, SmPropValue vals[]) {
   prop.name = const_cast<char*>(name);
   prop.type = const_cast<char*>(type);
   prop.num_vals = numVals;
@@ -438,8 +407,7 @@ SetSMProperty(SmProp& prop, const char* name, const char* type, int numVals,
 }
 #endif /* MOZ_X11 */
 
-static void RemoveArg(char **argv)
-{
+static void RemoveArg(char** argv) {
   do {
     *argv = *(argv + 1);
     ++argv;
@@ -449,8 +417,7 @@ static void RemoveArg(char **argv)
 }
 
 NS_IMETHODIMP
-nsNativeAppSupportUnix::Start(bool *aRetVal)
-{
+nsNativeAppSupportUnix::Start(bool* aRetVal) {
   NS_ASSERTION(gAppData, "gAppData must not be null.");
 
 // The dbus library is used by both nsWifiScannerDBus and BluetoothDBusService,
@@ -470,9 +437,9 @@ nsNativeAppSupportUnix::Start(bool *aRetVal)
 
   nsAutoCString prev_client_id;
 
-  char **curarg = gArgv + 1;
+  char** curarg = gArgv + 1;
   while (*curarg) {
-    char *arg = *curarg;
+    char* arg = *curarg;
     if (arg[0] == '-' && arg[1] == '-') {
       arg += 2;
       if (!strcmp(arg, "sm-disable")) {
@@ -499,37 +466,49 @@ nsNativeAppSupportUnix::Start(bool *aRetVal)
   // We don't want child processes to use the same ID
   unsetenv("DESKTOP_AUTOSTART_ID");
 
-  char *client_id = nullptr;
+  char* client_id = nullptr;
   if (!sm_disable) {
-    PRLibrary *iceLib = PR_LoadLibrary("libICE.so.6");
+    PRLibrary* iceLib = PR_LoadLibrary("libICE.so.6");
     if (!iceLib) {
       return NS_OK;
     }
 
-    PRLibrary *smLib = PR_LoadLibrary("libSM.so.6");
+    PRLibrary* smLib = PR_LoadLibrary("libSM.so.6");
     if (!smLib) {
       PR_UnloadLibrary(iceLib);
       return NS_OK;
     }
 
-    IceSetIOErrorHandler = (IceSetIOErrorHandlerFn)PR_FindFunctionSymbol(iceLib, "IceSetIOErrorHandler");
-    IceAddConnectionWatch = (IceAddConnectionWatchFn)PR_FindFunctionSymbol(iceLib, "IceAddConnectionWatch");
-    IceConnectionNumber = (IceConnectionNumberFn)PR_FindFunctionSymbol(iceLib, "IceConnectionNumber");
-    IceProcessMessages = (IceProcessMessagesFn)PR_FindFunctionSymbol(iceLib, "IceProcessMessages");
-    IceGetConnectionContext = (IceGetConnectionContextFn)PR_FindFunctionSymbol(iceLib, "IceGetConnectionContext");
+    IceSetIOErrorHandler = (IceSetIOErrorHandlerFn)PR_FindFunctionSymbol(
+        iceLib, "IceSetIOErrorHandler");
+    IceAddConnectionWatch = (IceAddConnectionWatchFn)PR_FindFunctionSymbol(
+        iceLib, "IceAddConnectionWatch");
+    IceConnectionNumber = (IceConnectionNumberFn)PR_FindFunctionSymbol(
+        iceLib, "IceConnectionNumber");
+    IceProcessMessages = (IceProcessMessagesFn)PR_FindFunctionSymbol(
+        iceLib, "IceProcessMessages");
+    IceGetConnectionContext = (IceGetConnectionContextFn)PR_FindFunctionSymbol(
+        iceLib, "IceGetConnectionContext");
     if (!IceSetIOErrorHandler || !IceAddConnectionWatch ||
-	!IceConnectionNumber  || !IceProcessMessages || !IceGetConnectionContext) {
+        !IceConnectionNumber || !IceProcessMessages ||
+        !IceGetConnectionContext) {
       PR_UnloadLibrary(iceLib);
       PR_UnloadLibrary(smLib);
       return NS_OK;
     }
 
-    SmcInteractDone = (SmcInteractDoneFn)PR_FindFunctionSymbol(smLib, "SmcInteractDone");
-    SmcSaveYourselfDone = (SmcSaveYourselfDoneFn)PR_FindFunctionSymbol(smLib, "SmcSaveYourselfDone");
-    SmcInteractRequest = (SmcInteractRequestFn)PR_FindFunctionSymbol(smLib, "SmcInteractRequest");
-    SmcCloseConnection = (SmcCloseConnectionFn)PR_FindFunctionSymbol(smLib, "SmcCloseConnection");
-    SmcOpenConnection = (SmcOpenConnectionFn)PR_FindFunctionSymbol(smLib, "SmcOpenConnection");
-    SmcSetProperties = (SmcSetPropertiesFn)PR_FindFunctionSymbol(smLib, "SmcSetProperties");
+    SmcInteractDone =
+        (SmcInteractDoneFn)PR_FindFunctionSymbol(smLib, "SmcInteractDone");
+    SmcSaveYourselfDone = (SmcSaveYourselfDoneFn)PR_FindFunctionSymbol(
+        smLib, "SmcSaveYourselfDone");
+    SmcInteractRequest = (SmcInteractRequestFn)PR_FindFunctionSymbol(
+        smLib, "SmcInteractRequest");
+    SmcCloseConnection = (SmcCloseConnectionFn)PR_FindFunctionSymbol(
+        smLib, "SmcCloseConnection");
+    SmcOpenConnection =
+        (SmcOpenConnectionFn)PR_FindFunctionSymbol(smLib, "SmcOpenConnection");
+    SmcSetProperties =
+        (SmcSetPropertiesFn)PR_FindFunctionSymbol(smLib, "SmcSetProperties");
     if (!SmcInteractDone || !SmcSaveYourselfDone || !SmcInteractRequest ||
         !SmcCloseConnection || !SmcOpenConnection || !SmcSetProperties) {
       PR_UnloadLibrary(iceLib);
@@ -539,7 +518,8 @@ nsNativeAppSupportUnix::Start(bool *aRetVal)
 
     ice_init();
 
-    // all callbacks are mandatory in libSM 1.0, so listen even if we don't care.
+    // all callbacks are mandatory in libSM 1.0, so listen even if we don't
+    // care.
     unsigned long mask = SmcSaveYourselfProcMask | SmcDieProcMask |
                          SmcSaveCompleteProcMask | SmcShutdownCancelledProcMask;
 
@@ -554,21 +534,21 @@ nsNativeAppSupportUnix::Start(bool *aRetVal)
     callbacks.save_complete.client_data = nullptr;
 
     callbacks.shutdown_cancelled.callback =
-      nsNativeAppSupportUnix::ShutdownCancelledCB;
+        nsNativeAppSupportUnix::ShutdownCancelledCB;
     callbacks.shutdown_cancelled.client_data = static_cast<SmPointer>(this);
 
     char errbuf[256];
-    mSessionConnection = SmcOpenConnection(nullptr, this, SmProtoMajor,
-                                           SmProtoMinor, mask, &callbacks,
-                                           prev_client_id.get(), &client_id,
-                                           sizeof(errbuf), errbuf);
+    mSessionConnection = SmcOpenConnection(
+        nullptr, this, SmProtoMajor, SmProtoMinor, mask, &callbacks,
+        prev_client_id.get(), &client_id, sizeof(errbuf), errbuf);
   }
 
   if (!mSessionConnection) {
     return NS_OK;
   }
 
-  LogModule::Init();  // need to make sure initialized before SetClientState
+  LogModule::Init(
+      gArgc, gArgv);  // need to make sure initialized before SetClientState
   if (prev_client_id.IsEmpty() ||
       (client_id && !prev_client_id.Equals(client_id))) {
     SetClientState(STATE_REGISTERING);
@@ -588,18 +568,21 @@ nsNativeAppSupportUnix::Start(bool *aRetVal)
   nsAutoCString path(getenv("MOZ_APP_LAUNCHER"));
 
   if (path.IsEmpty()) {
-    NS_ASSERTION(gDirServiceProvider, "gDirServiceProvider is NULL! This shouldn't happen!");
+    NS_ASSERTION(gDirServiceProvider,
+                 "gDirServiceProvider is NULL! This shouldn't happen!");
     nsCOMPtr<nsIFile> executablePath;
     nsresult rv;
 
     bool dummy;
-    rv = gDirServiceProvider->GetFile(XRE_EXECUTABLE_FILE, &dummy, getter_AddRefs(executablePath));
+    rv = gDirServiceProvider->GetFile(XRE_EXECUTABLE_FILE, &dummy,
+                                      getter_AddRefs(executablePath));
 
     if (NS_SUCCEEDED(rv)) {
-      // Strip off the -bin suffix to get the shell script we should run; this is what Breakpad does
+      // Strip off the -bin suffix to get the shell script we should run; this
+      // is what Breakpad does
       nsAutoCString leafName;
       rv = executablePath->GetNativeLeafName(leafName);
-      if (NS_SUCCEEDED(rv) && StringEndsWith(leafName, NS_LITERAL_CSTRING("-bin"))) {
+      if (NS_SUCCEEDED(rv) && StringEndsWith(leafName, "-bin"_ns)) {
         leafName.SetLength(leafName.Length() - strlen("-bin"));
         executablePath->SetNativeLeafName(leafName);
       }
@@ -615,14 +598,15 @@ nsNativeAppSupportUnix::Start(bool *aRetVal)
     path = gAppData->name;  // will always be set
     ToLowerCase(path);
     MOZ_LOG(sMozSMLog, LogLevel::Warning,
-        ("Could not determine executable path. Falling back to %s.", path.get()));
+            ("Could not determine executable path. Falling back to %s.",
+             path.get()));
   }
 
   SmProp propRestart, propClone, propProgram, propUser, *props[4];
   SmPropValue valsRestart[3], valsClone[1], valsProgram[1], valsUser[1];
   int n = 0;
 
-  NS_NAMED_LITERAL_CSTRING(kClientIDParam, "--sm-client-id");
+  constexpr auto kClientIDParam = "--sm-client-id"_ns;
 
   SetSMValue(valsRestart[0], path);
   SetSMValue(valsRestart[1], kClientIDParam);
@@ -646,8 +630,9 @@ nsNativeAppSupportUnix::Start(bool *aRetVal)
   if (pw && pw->pw_name) {
     userName = pw->pw_name;
   } else {
-    userName = NS_LITERAL_CSTRING("nobody");
-    MOZ_LOG(sMozSMLog, LogLevel::Warning,
+    userName = "nobody"_ns;
+    MOZ_LOG(
+        sMozSMLog, LogLevel::Warning,
         ("Could not determine user-name. Falling back to %s.", userName.get()));
   }
 
@@ -664,25 +649,11 @@ nsNativeAppSupportUnix::Start(bool *aRetVal)
 }
 
 NS_IMETHODIMP
-nsNativeAppSupportUnix::Stop(bool *aResult)
-{
-  NS_ENSURE_ARG(aResult);
-  *aResult = true;
-  return NS_OK;
-}
+nsNativeAppSupportUnix::Enable() { return NS_OK; }
 
-NS_IMETHODIMP
-nsNativeAppSupportUnix::Enable()
-{
-  return NS_OK;
-}
-
-nsresult
-NS_CreateNativeAppSupport(nsINativeAppSupport **aResult)
-{
+nsresult NS_CreateNativeAppSupport(nsINativeAppSupport** aResult) {
   nsNativeAppSupportBase* native = new nsNativeAppSupportUnix();
-  if (!native)
-    return NS_ERROR_OUT_OF_MEMORY;
+  if (!native) return NS_ERROR_OUT_OF_MEMORY;
 
   *aResult = native;
   NS_ADDREF(*aResult);
