@@ -7,10 +7,13 @@
 #define mozilla_MouseEvents_h__
 
 #include <stdint.h>
+#include <math.h>
 
 #include "mozilla/BasicEvents.h"
+#include "mozilla/EventForwards.h"
 #include "mozilla/MathAlgorithms.h"
 #include "mozilla/dom/DataTransfer.h"
+#include "mozilla/ipc/IPCForwards.h"
 #include "nsCOMPtr.h"
 
 namespace mozilla {
@@ -40,22 +43,22 @@ class WidgetPointerEventHolder final {
 
 class WidgetPointerHelper {
  public:
-  uint32_t pointerId;
-  uint32_t tiltX;
-  uint32_t tiltY;
-  uint32_t twist;
-  float tangentialPressure;
-  bool convertToPointer;
+  uint32_t pointerId = 0;
+  int32_t tiltX = 0;
+  int32_t tiltY = 0;
+  int32_t twist = 0;
+  float tangentialPressure = 0.0f;
+  bool convertToPointer = true;
+  // When convertToPointerRawUpdate is set to true, the event or the touch may
+  // cause ePointerRawUpdate event in PresShell::HandleEvent() if it's requested
+  // by the web app.  This is set to false if the source mouse event or the
+  // source touch move event is not dispatched immediately by BrowserChild
+  // because BrowserChild dispatches only eMouseRawUpdate or eTouchRawUpdate to
+  // dispatch ePointerRawUpdate immediately.
+  bool convertToPointerRawUpdate = true;
   RefPtr<WidgetPointerEventHolder> mCoalescedWidgetEvents;
 
-  WidgetPointerHelper()
-      : pointerId(0),
-        tiltX(0),
-        tiltY(0),
-        twist(0),
-        tangentialPressure(0),
-        convertToPointer(true) {}
-
+  WidgetPointerHelper() = default;
   WidgetPointerHelper(uint32_t aPointerId, uint32_t aTiltX, uint32_t aTiltY,
                       uint32_t aTwist = 0, float aTangentialPressure = 0)
       : pointerId(aPointerId),
@@ -67,6 +70,36 @@ class WidgetPointerHelper {
 
   explicit WidgetPointerHelper(const WidgetPointerHelper& aHelper) = default;
 
+  constexpr static double kPi =
+#ifdef M_PI
+      M_PI;
+#else
+      3.14159265358979323846;
+#endif
+  constexpr static double kHalfPi =
+#ifdef M_PI_2
+      M_PI_2;
+#else
+      1.57079632679489661923;
+#endif
+  constexpr static double kDoublePi = kPi * 2;
+
+  constexpr static double GetDefaultAltitudeAngle() { return kHalfPi; }
+  constexpr static double GetDefaultAzimuthAngle() { return 0.0; }
+
+  double ComputeAltitudeAngle() const {
+    return ComputeAltitudeAngle(tiltX, tiltY);
+  }
+  double ComputeAzimuthAngle() const {
+    return ComputeAzimuthAngle(tiltX, tiltY);
+  }
+
+  static double ComputeAltitudeAngle(int32_t aTiltX, int32_t aTiltY);
+  static double ComputeAzimuthAngle(int32_t aTiltX, int32_t aTiltY);
+
+  static double ComputeTiltX(double aAltitudeAngle, double aAzimuthAngle);
+  static double ComputeTiltY(double aAltitudeAngle, double aAzimuthAngle);
+
   void AssignPointerHelperData(const WidgetPointerHelper& aEvent,
                                bool aCopyCoalescedEvents = false) {
     pointerId = aEvent.pointerId;
@@ -75,10 +108,16 @@ class WidgetPointerHelper {
     twist = aEvent.twist;
     tangentialPressure = aEvent.tangentialPressure;
     convertToPointer = aEvent.convertToPointer;
+    convertToPointerRawUpdate = aEvent.convertToPointerRawUpdate;
     if (aCopyCoalescedEvents) {
       mCoalescedWidgetEvents = aEvent.mCoalescedWidgetEvents;
     }
   }
+
+ private:
+  static int32_t GetValidTiltValue(int32_t aTilt);
+  static double GetValidAltitudeAngle(double aAltitudeAngle);
+  static double GetValidAzimuthAngle(double aAzimuthAngle);
 };
 
 /******************************************************************************
@@ -90,6 +129,7 @@ class WidgetMouseEventBase : public WidgetInputEvent {
   friend class dom::PBrowserParent;
   friend class dom::PBrowserChild;
   friend class dom::PBrowserBridgeParent;
+  ALLOW_DEPRECATED_READPARAM
 
  protected:
   WidgetMouseEventBase()
@@ -101,8 +141,9 @@ class WidgetMouseEventBase : public WidgetInputEvent {
   // we have to hardcode MouseEvent_Binding::MOZ_SOURCE_MOUSE.
 
   WidgetMouseEventBase(bool aIsTrusted, EventMessage aMessage,
-                       nsIWidget* aWidget, EventClassID aEventClassID)
-      : WidgetInputEvent(aIsTrusted, aMessage, aWidget, aEventClassID),
+                       nsIWidget* aWidget, EventClassID aEventClassID,
+                       const WidgetEventTime* aTime = nullptr)
+      : WidgetInputEvent(aIsTrusted, aMessage, aWidget, aEventClassID, aTime),
         mPressure(0),
         mButton(0),
         mButtons(0),
@@ -117,12 +158,15 @@ class WidgetMouseEventBase : public WidgetInputEvent {
     MOZ_CRASH("WidgetMouseEventBase must not be most-subclass");
   }
 
-  // ID of the canvas HitRegion
-  nsString mRegion;
-
   // Finger or touch pressure of event. It ranges between 0.0 and 1.0.
   float mPressure;
 
+  /**
+   * Returns computed pressure for mouse button from mPressure, mButtons and
+   * mInputSource.
+   */
+  [[nodiscard]] float ComputeMouseButtonPressure() const;
+
   // Pressed button ID of mousedown or mouseup event.
   // This is set only when pressing a button causes the event.
   int16_t mButton;
@@ -164,8 +208,66 @@ class WidgetMouseEventBase : public WidgetInputEvent {
    * Returns true if left click event.
    */
   bool IsLeftClickEvent() const {
-    return mMessage == eMouseClick && mButton == MouseButton::ePrimary;
+    return mMessage == ePointerClick && mButton == MouseButton::ePrimary;
   }
+
+  /**
+   * Returns true if this event changes a button state to "pressed".
+   */
+  [[nodiscard]] bool IsPressingButton() const {
+    MOZ_ASSERT(IsTrusted());
+    if (mClass == eMouseEventClass) {
+      return mMessage == eMouseDown;
+    }
+    if (mButton == MouseButton::eNotPressed) {
+      return false;
+    }
+    // If this is an ePointerDown event whose mButton is not "not pressed", this
+    // is a button pressing event.
+    if (mMessage == ePointerDown) {
+      return true;
+    }
+    // If 2 or more buttons are pressed at same time, they are sent with
+    // pointermove rather than pointerdown.  Therefore, let's check whether
+    // mButtons contains the proper flag for the pressing button.
+    const bool buttonsContainButton = !!(
+        mButtons & MouseButtonsFlagToChange(static_cast<MouseButton>(mButton)));
+    return mMessage == ePointerMove && buttonsContainButton;
+  }
+
+  /**
+   * Returns true if this event changes a button state to "released".
+   */
+  [[nodiscard]] bool IsReleasingButton() const {
+    MOZ_ASSERT(IsTrusted());
+    if (mClass == eMouseEventClass) {
+      return mMessage == eMouseUp;
+    }
+    if (mButton == MouseButton::eNotPressed) {
+      return false;
+    }
+    // If this is an ePointerUp event whose mButton is not "not pressed", this
+    // is a button release event.
+    if (mMessage == ePointerUp) {
+      return true;
+    }
+    // If the releasing button is not the last button of pressing buttons, web
+    // apps notified by pointermove rather than pointerup.  Therefore, let's
+    // check whether mButtons loses the proper flag for the releasing button.
+    const bool buttonsLoseTheButton = !(
+        mButtons & MouseButtonsFlagToChange(static_cast<MouseButton>(mButton)));
+    return mMessage == ePointerMove && buttonsLoseTheButton;
+  }
+
+  /**
+   * Returns true if the input source supports hover state like a mouse.
+   */
+  [[nodiscard]] bool InputSourceSupportsHover() const;
+
+  /**
+   * Returns true if corresponding DOM event should use fractional coordinates.
+   */
+  [[nodiscard]] bool DOMEventShouldUseFractionalCoords() const;
 };
 
 /******************************************************************************
@@ -178,6 +280,7 @@ class WidgetMouseEvent : public WidgetMouseEventBase,
   friend class dom::PBrowserParent;
   friend class dom::PBrowserChild;
   friend class dom::PBrowserBridgeParent;
+  ALLOW_DEPRECATED_READPARAM
 
  public:
   typedef bool ReasonType;
@@ -199,23 +302,16 @@ class WidgetMouseEvent : public WidgetMouseEventBase,
   };
 
  protected:
-  WidgetMouseEvent()
-      : mReason(eReal),
-        mContextMenuTrigger(eNormal),
-        mClickCount(0),
-        mIgnoreRootScrollFrame(false),
-        mUseLegacyNonPrimaryDispatch(false),
-        mClickEventPrevented(false) {}
+  WidgetMouseEvent() = default;
 
   WidgetMouseEvent(bool aIsTrusted, EventMessage aMessage, nsIWidget* aWidget,
-                   EventClassID aEventClassID, Reason aReason)
-      : WidgetMouseEventBase(aIsTrusted, aMessage, aWidget, aEventClassID),
+                   EventClassID aEventClassID, Reason aReason,
+                   ContextMenuTrigger aContextMenuTrigger,
+                   const WidgetEventTime* aTime = nullptr)
+      : WidgetMouseEventBase(aIsTrusted, aMessage, aWidget, aEventClassID,
+                             aTime),
         mReason(aReason),
-        mContextMenuTrigger(eNormal),
-        mClickCount(0),
-        mIgnoreRootScrollFrame(false),
-        mUseLegacyNonPrimaryDispatch(false),
-        mClickEventPrevented(false) {}
+        mContextMenuTrigger(aContextMenuTrigger) {}
 
 #ifdef DEBUG
   void AssertContextMenuEventButtonConsistency() const;
@@ -225,15 +321,14 @@ class WidgetMouseEvent : public WidgetMouseEventBase,
   virtual WidgetMouseEvent* AsMouseEvent() override { return this; }
 
   WidgetMouseEvent(bool aIsTrusted, EventMessage aMessage, nsIWidget* aWidget,
-                   Reason aReason,
-                   ContextMenuTrigger aContextMenuTrigger = eNormal)
-      : WidgetMouseEventBase(aIsTrusted, aMessage, aWidget, eMouseEventClass),
+                   Reason aReason = eReal,
+                   ContextMenuTrigger aContextMenuTrigger = eNormal,
+                   const WidgetEventTime* aTime = nullptr)
+      : WidgetMouseEventBase(aIsTrusted, aMessage, aWidget, eMouseEventClass,
+                             aTime),
         mReason(aReason),
-        mContextMenuTrigger(aContextMenuTrigger),
-        mClickCount(0),
-        mIgnoreRootScrollFrame(false),
-        mUseLegacyNonPrimaryDispatch(false),
-        mClickEventPrevented(false) {
+        mContextMenuTrigger(aContextMenuTrigger) {
+    MOZ_ASSERT_IF(aIsTrusted, !IsPointerEventMessage(mMessage));
     if (aMessage == eContextMenu) {
       mButton = (mContextMenuTrigger == eNormal) ? MouseButton::eSecondary
                                                  : MouseButton::ePrimary;
@@ -249,7 +344,7 @@ class WidgetMouseEvent : public WidgetMouseEventBase,
                "Duplicate() must be overridden by sub class");
     // Not copying widget, it is a weak reference.
     WidgetMouseEvent* result = new WidgetMouseEvent(
-        false, mMessage, nullptr, mReason, mContextMenuTrigger);
+        false, mMessage, nullptr, mReason, mContextMenuTrigger, this);
     result->AssignMouseEventData(*this, true);
     result->mFlags = mFlags;
     return result;
@@ -263,12 +358,12 @@ class WidgetMouseEvent : public WidgetMouseEventBase,
   // - Representing mouse operation.
   // - Synthesized for emulating mousemove event when the content under the
   //   mouse cursor is scrolled.
-  Reason mReason;
+  Reason mReason = eReal;
 
   // mContextMenuTrigger is valid only when mMessage is eContextMenu.
   // This indicates if the context menu event is caused by context menu key or
   // other reasons (typically, a click of right mouse button).
-  ContextMenuTrigger mContextMenuTrigger;
+  ContextMenuTrigger mContextMenuTrigger = eNormal;
 
   // mExitFrom contains a value only when mMessage is eMouseExitFromWidget.
   // This indicates if the mouse cursor exits from a top level platform widget,
@@ -276,28 +371,35 @@ class WidgetMouseEvent : public WidgetMouseEventBase,
   Maybe<ExitFrom> mExitFrom;
 
   // mClickCount may be non-zero value when mMessage is eMouseDown, eMouseUp,
-  // eMouseClick or eMouseDoubleClick. The number is count of mouse clicks.
+  // ePointerClick or eMouseDoubleClick. The number is count of mouse clicks.
   // Otherwise, this must be 0.
-  uint32_t mClickCount;
+  uint32_t mClickCount = 0;
 
   // Whether the event should ignore scroll frame bounds during dispatch.
-  bool mIgnoreRootScrollFrame;
+  bool mIgnoreRootScrollFrame = false;
 
-  // Indicates whether the event should dispatch click events for non-primary
-  // mouse buttons on window and document.
-  bool mUseLegacyNonPrimaryDispatch;
+  // Whether the event should be dispatched on a target limited in capturing
+  // content.
+  bool mIgnoreCapturingContent = false;
 
   // Whether the event shouldn't cause click event.
-  bool mClickEventPrevented;
+  bool mClickEventPrevented = false;
+
+  // If this is set to true while the event is being dispatched,
+  // PresShell::EventHandler::FinalizeHandlingEvent will dispatch a synthesized
+  // eMouseMove or ePointerMove.
+  bool mSynthesizeMoveAfterDispatch = false;
 
   void AssignMouseEventData(const WidgetMouseEvent& aEvent, bool aCopyTargets) {
     AssignMouseEventBaseData(aEvent, aCopyTargets);
     AssignPointerHelperData(aEvent, /* aCopyCoalescedEvents */ true);
 
+    mReason = aEvent.mReason;
+    mContextMenuTrigger = aEvent.mContextMenuTrigger;
     mExitFrom = aEvent.mExitFrom;
     mClickCount = aEvent.mClickCount;
     mIgnoreRootScrollFrame = aEvent.mIgnoreRootScrollFrame;
-    mUseLegacyNonPrimaryDispatch = aEvent.mUseLegacyNonPrimaryDispatch;
+    mIgnoreCapturingContent = aEvent.mIgnoreCapturingContent;
     mClickEventPrevented = aEvent.mClickEventPrevented;
   }
 
@@ -312,7 +414,13 @@ class WidgetMouseEvent : public WidgetMouseEventBase,
    * Returns true if the event is a real mouse event.  Otherwise, i.e., it's
    * a synthesized event by scroll or something, returns false.
    */
-  bool IsReal() const { return mReason == eReal; }
+  [[nodiscard]] bool IsReal() const { return mReason == eReal; }
+
+  /**
+   * Returns true if the event is synthesized for scroll or layout change.
+   * Do not confuse this with a synthesized event for tests.
+   */
+  [[nodiscard]] bool IsSynthesized() const { return mReason == eSynthesized; }
 
   /**
    * Returns true if middle click paste is enabled.
@@ -328,24 +436,31 @@ class WidgetDragEvent : public WidgetMouseEvent {
  private:
   friend class mozilla::dom::PBrowserParent;
   friend class mozilla::dom::PBrowserChild;
+  ALLOW_DEPRECATED_READPARAM
 
  protected:
   WidgetDragEvent()
-      : mUserCancelled(false), mDefaultPreventedOnContent(false) {}
+      : mUserCancelled(false),
+        mDefaultPreventedOnContent(false),
+        mInHTMLEditorEventListener(false) {}
 
  public:
   virtual WidgetDragEvent* AsDragEvent() override { return this; }
 
-  WidgetDragEvent(bool aIsTrusted, EventMessage aMessage, nsIWidget* aWidget)
-      : WidgetMouseEvent(aIsTrusted, aMessage, aWidget, eDragEventClass, eReal),
+  WidgetDragEvent(bool aIsTrusted, EventMessage aMessage, nsIWidget* aWidget,
+                  const WidgetEventTime* aTime = nullptr)
+      : WidgetMouseEvent(aIsTrusted, aMessage, aWidget, eDragEventClass, eReal,
+                         eNormal, aTime),
         mUserCancelled(false),
-        mDefaultPreventedOnContent(false) {}
+        mDefaultPreventedOnContent(false),
+        mInHTMLEditorEventListener(false) {}
 
   virtual WidgetEvent* Duplicate() const override {
     MOZ_ASSERT(mClass == eDragEventClass,
                "Duplicate() must be overridden by sub class");
     // Not copying widget, it is a weak reference.
-    WidgetDragEvent* result = new WidgetDragEvent(false, mMessage, nullptr);
+    WidgetDragEvent* result =
+        new WidgetDragEvent(false, mMessage, nullptr, this);
     result->AssignDragEventData(*this, true);
     result->mFlags = mFlags;
     return result;
@@ -358,6 +473,8 @@ class WidgetDragEvent : public WidgetMouseEvent {
   bool mUserCancelled;
   // If this is true, the drag event's preventDefault() is called on content.
   bool mDefaultPreventedOnContent;
+  // If this event is currently being handled by HTMLEditorEventListener.
+  bool mInHTMLEditorEventListener;
 
   // XXX Not tested by test_assign_event_data.html
   void AssignDragEventData(const WidgetDragEvent& aEvent, bool aCopyTargets) {
@@ -367,6 +484,14 @@ class WidgetDragEvent : public WidgetMouseEvent {
     // XXX mUserCancelled isn't copied, is this intentionally?
     mUserCancelled = false;
     mDefaultPreventedOnContent = aEvent.mDefaultPreventedOnContent;
+    // XXX mInHTMLEditorEventListener isn't copied, is this intentionally?
+    mInHTMLEditorEventListener = false;
+  }
+
+  bool CanConvertToInputData() const {
+    return mMessage == eDragStart || mMessage == eDragEnd ||
+           mMessage == eDragEnter || mMessage == eDragOver ||
+           mMessage == eDragExit || mMessage == eDrop;
   }
 
   /**
@@ -394,9 +519,10 @@ class WidgetMouseScrollEvent : public WidgetMouseEventBase {
   virtual WidgetMouseScrollEvent* AsMouseScrollEvent() override { return this; }
 
   WidgetMouseScrollEvent(bool aIsTrusted, EventMessage aMessage,
-                         nsIWidget* aWidget)
+                         nsIWidget* aWidget,
+                         const WidgetEventTime* aTime = nullptr)
       : WidgetMouseEventBase(aIsTrusted, aMessage, aWidget,
-                             eMouseScrollEventClass),
+                             eMouseScrollEventClass, aTime),
         mDelta(0),
         mIsHorizontal(false) {}
 
@@ -405,7 +531,7 @@ class WidgetMouseScrollEvent : public WidgetMouseEventBase {
                "Duplicate() must be overridden by sub class");
     // Not copying widget, it is a weak reference.
     WidgetMouseScrollEvent* result =
-        new WidgetMouseScrollEvent(false, mMessage, nullptr);
+        new WidgetMouseScrollEvent(false, mMessage, nullptr, this);
     result->AssignMouseScrollEventData(*this, true);
     result->mFlags = mFlags;
     return result;
@@ -440,6 +566,7 @@ class WidgetWheelEvent : public WidgetMouseEventBase {
  private:
   friend class mozilla::dom::PBrowserParent;
   friend class mozilla::dom::PBrowserChild;
+  ALLOW_DEPRECATED_READPARAM
 
   WidgetWheelEvent()
       : mDeltaX(0.0),
@@ -466,8 +593,10 @@ class WidgetWheelEvent : public WidgetMouseEventBase {
  public:
   virtual WidgetWheelEvent* AsWheelEvent() override { return this; }
 
-  WidgetWheelEvent(bool aIsTrusted, EventMessage aMessage, nsIWidget* aWidget)
-      : WidgetMouseEventBase(aIsTrusted, aMessage, aWidget, eWheelEventClass),
+  WidgetWheelEvent(bool aIsTrusted, EventMessage aMessage, nsIWidget* aWidget,
+                   const WidgetEventTime* aTime = nullptr)
+      : WidgetMouseEventBase(aIsTrusted, aMessage, aWidget, eWheelEventClass,
+                             aTime),
         mDeltaX(0.0),
         mDeltaY(0.0),
         mDeltaZ(0.0),
@@ -493,16 +622,17 @@ class WidgetWheelEvent : public WidgetMouseEventBase {
     MOZ_ASSERT(mClass == eWheelEventClass,
                "Duplicate() must be overridden by sub class");
     // Not copying widget, it is a weak reference.
-    WidgetWheelEvent* result = new WidgetWheelEvent(false, mMessage, nullptr);
+    WidgetWheelEvent* result =
+        new WidgetWheelEvent(false, mMessage, nullptr, this);
     result->AssignWheelEventData(*this, true);
     result->mFlags = mFlags;
     return result;
   }
 
-  // On OS X, scroll gestures that start at the edge of the scrollable range
-  // can result in a swipe gesture. For the first wheel event of such a
-  // gesture, call TriggersSwipe() after the event has been processed
-  // in order to find out whether a swipe should be started.
+  // Scroll gestures that start at the edge of the scrollable range can result
+  // in a swipe gesture. For the first wheel event of such a gesture, call
+  // TriggersSwipe() after the event has been processed in order to find out
+  // whether a swipe should be started.
   bool TriggersSwipe() const {
     return mCanTriggerSwipe && mViewPortIsOverscrolled &&
            this->mOverflowDeltaX != 0.0;
@@ -591,7 +721,7 @@ class WidgetWheelEvent : public WidgetMouseEventBase {
   enum ScrollType : uint8_t {
     SCROLL_DEFAULT,
     SCROLL_SYNCHRONOUSLY,
-    SCROLL_ASYNCHRONOUSELY,
+    SCROLL_ASYNCHRONOUSLY,
     SCROLL_SMOOTHLY
   };
   ScrollType mScrollType;
@@ -686,26 +816,36 @@ class WidgetWheelEvent : public WidgetMouseEventBase {
 class WidgetPointerEvent : public WidgetMouseEvent {
   friend class mozilla::dom::PBrowserParent;
   friend class mozilla::dom::PBrowserChild;
+  ALLOW_DEPRECATED_READPARAM
 
-  WidgetPointerEvent()
-      : mWidth(1), mHeight(1), mIsPrimary(true), mFromTouchEvent(false) {}
+  WidgetPointerEvent() = default;
 
  public:
   virtual WidgetPointerEvent* AsPointerEvent() override { return this; }
 
-  WidgetPointerEvent(bool aIsTrusted, EventMessage aMsg, nsIWidget* w)
-      : WidgetMouseEvent(aIsTrusted, aMsg, w, ePointerEventClass, eReal),
-        mWidth(1),
-        mHeight(1),
-        mIsPrimary(true),
-        mFromTouchEvent(false) {}
+  WidgetPointerEvent(bool aIsTrusted, EventMessage aMsg, nsIWidget* w,
+                     const WidgetEventTime* aTime)
+      : WidgetMouseEvent(aIsTrusted, aMsg, w, ePointerEventClass, eReal,
+                         eNormal, aTime) {
+    if (aMsg == eContextMenu) {
+      mButton = (mContextMenuTrigger == eNormal) ? MouseButton::eSecondary
+                                                 : MouseButton::ePrimary;
+    }
+  }
+
+  WidgetPointerEvent(bool aIsTrusted, EventMessage aMsg, nsIWidget* w,
+                     ContextMenuTrigger aContextMenuTrigger = eNormal,
+                     const WidgetEventTime* aTime = nullptr)
+      : WidgetMouseEvent(aIsTrusted, aMsg, w, ePointerEventClass, eReal,
+                         aContextMenuTrigger, aTime) {
+    if (aMsg == eContextMenu) {
+      mButton = (mContextMenuTrigger == eNormal) ? MouseButton::eSecondary
+                                                 : MouseButton::ePrimary;
+    }
+  }
 
   explicit WidgetPointerEvent(const WidgetMouseEvent& aEvent)
-      : WidgetMouseEvent(aEvent),
-        mWidth(1),
-        mHeight(1),
-        mIsPrimary(true),
-        mFromTouchEvent(false) {
+      : WidgetMouseEvent(aEvent) {
     mClass = ePointerEventClass;
   }
 
@@ -713,17 +853,17 @@ class WidgetPointerEvent : public WidgetMouseEvent {
     MOZ_ASSERT(mClass == ePointerEventClass,
                "Duplicate() must be overridden by sub class");
     // Not copying widget, it is a weak reference.
-    WidgetPointerEvent* result =
-        new WidgetPointerEvent(false, mMessage, nullptr);
+    WidgetPointerEvent* result = new WidgetPointerEvent(
+        false, mMessage, nullptr, mContextMenuTrigger, this);
     result->AssignPointerEventData(*this, true);
     result->mFlags = mFlags;
     return result;
   }
 
-  uint32_t mWidth;
-  uint32_t mHeight;
-  bool mIsPrimary;
-  bool mFromTouchEvent;
+  double mWidth = 1.0;
+  double mHeight = 1.0;
+  bool mIsPrimary = true;
+  bool mFromTouchEvent = false;
 
   // XXX Not tested by test_assign_event_data.html
   void AssignPointerEventData(const WidgetPointerEvent& aEvent,
