@@ -10,35 +10,51 @@
 #include <unistd.h>
 
 #include "mozilla/Types.h"
+#include "AsyncDBus.h"
 #include "nsGtkUtils.h"
 #include "nsIFileURL.h"
 #include "nsIGIOService.h"
 #include "nsIURI.h"
 #include "nsIWidget.h"
 #include "nsIFile.h"
+#include "nsIStringBundle.h"
+#include "mozilla/Components.h"
 #include "mozilla/Preferences.h"
+#include "mozilla/dom/Promise.h"
 
 #include "nsArrayEnumerator.h"
-#include "nsMemory.h"
 #include "nsEnumeratorUtils.h"
 #include "nsNetUtil.h"
 #include "nsReadableUtils.h"
 #include "MozContainer.h"
-#include "gfxPlatformGtk.h"
+#include "WidgetUtilsGtk.h"
 
+#include "gfxPlatform.h"
 #include "nsFilePicker.h"
 
+#undef LOG
+#ifdef MOZ_LOGGING
+#  include "mozilla/Logging.h"
+#  include "nsTArray.h"
+#  include "Units.h"
+extern mozilla::LazyLogModule gWidgetLog;
+#  define LOG(args) MOZ_LOG(gWidgetLog, mozilla::LogLevel::Debug, args)
+#else
+#  define LOG(args)
+#endif /* MOZ_LOGGING */
+
 using namespace mozilla;
+using mozilla::dom::Promise;
 
 #define MAX_PREVIEW_SIZE 180
-// bug 1184009
-#define MAX_PREVIEW_SOURCE_SIZE 4096
+// bug 1935858
+#define MAX_PREVIEW_SOURCE_SIZE 8192
 
 nsIFile* nsFilePicker::mPrevDisplayDirectory = nullptr;
 
 void nsFilePicker::Shutdown() { NS_IF_RELEASE(mPrevDisplayDirectory); }
 
-static GtkFileChooserAction GetGtkFileChooserAction(int16_t aMode) {
+static GtkFileChooserAction GetGtkFileChooserAction(nsIFilePicker::Mode aMode) {
   GtkFileChooserAction action;
 
   switch (aMode) {
@@ -122,7 +138,7 @@ static void UpdateFilePreviewWidget(GtkFileChooser* file_chooser,
   // nice
   gint x_padding =
       (MAX_PREVIEW_SIZE + 6 - gdk_pixbuf_get_width(preview_pixbuf)) / 2;
-  gtk_misc_set_padding(GTK_MISC(preview_widget), x_padding, 0);
+  g_object_set(preview_widget, "padding", x_padding, 0, nullptr);
 
   gtk_image_set_from_pixbuf(preview_widget, preview_pixbuf);
   g_object_unref(preview_pixbuf);
@@ -156,19 +172,8 @@ static nsAutoCString MakeCaseInsensitiveShellGlob(const char* aPattern) {
 NS_IMPL_ISUPPORTS(nsFilePicker, nsIFilePicker)
 
 nsFilePicker::nsFilePicker()
-    : mSelectedType(0),
-      mRunning(false),
-      mAllowURLs(false),
-      mFileChooserDelegate(nullptr) {
-  nsCOMPtr<nsIGIOService> giovfs = do_GetService(NS_GIOSERVICE_CONTRACTID);
-  // Due to Bug 1635718 always use portal for file dialog on Wayland.
-  if (gfxPlatformGtk::GetPlatform()->IsWaylandDisplay()) {
-    mUseNativeFileChooser =
-        Preferences::GetBool("widget.use-xdg-desktop-portal", true);
-  } else {
-    giovfs->ShouldUseFlatpakPortal(&mUseNativeFileChooser);
-  }
-}
+    : mUseNativeFileChooser(
+          widget::ShouldUsePortal(widget::PortalKind::FilePicker)) {}
 
 nsFilePicker::~nsFilePicker() = default;
 
@@ -176,7 +181,7 @@ void ReadMultipleFiles(gpointer filename, gpointer array) {
   nsCOMPtr<nsIFile> localfile;
   nsresult rv =
       NS_NewNativeLocalFile(nsDependentCString(static_cast<char*>(filename)),
-                            false, getter_AddRefs(localfile));
+                            getter_AddRefs(localfile));
   if (NS_SUCCEEDED(rv)) {
     nsCOMArray<nsIFile>& files = *static_cast<nsCOMArray<nsIFile>*>(array);
     files.AppendObject(localfile);
@@ -226,6 +231,71 @@ void nsFilePicker::InitNative(nsIWidget* aParent, const nsAString& aTitle) {
   mTitle.Assign(aTitle);
 }
 
+NS_IMETHODIMP
+nsFilePicker::IsModeSupported(nsIFilePicker::Mode aMode, JSContext* aCx,
+                              Promise** aRetPromise) {
+#ifdef MOZ_ENABLE_DBUS
+  if (!widget::ShouldUsePortal(widget::PortalKind::FilePicker) ||
+      aMode != nsIFilePicker::modeGetFolder) {
+    return nsBaseFilePicker::IsModeSupported(aMode, aCx, aRetPromise);
+  }
+
+  const char kFreedesktopPortalName[] = "org.freedesktop.portal.Desktop";
+  const char kFreedesktopPortalPath[] = "/org/freedesktop/portal/desktop";
+  const char kFreedesktopPortalFileChooser[] =
+      "org.freedesktop.portal.FileChooser";
+
+  MOZ_ASSERT(aCx);
+  MOZ_ASSERT(aRetPromise);
+
+  nsIGlobalObject* globalObject = xpc::CurrentNativeGlobal(aCx);
+  if (NS_WARN_IF(!globalObject)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  ErrorResult result;
+  RefPtr<Promise> retPromise = Promise::Create(globalObject, result);
+  if (NS_WARN_IF(result.Failed())) {
+    return result.StealNSResult();
+  }
+
+  widget::CreateDBusProxyForBus(
+      G_BUS_TYPE_SESSION,
+      GDBusProxyFlags(G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS),
+      /* aInterfaceInfo = */ nullptr, kFreedesktopPortalName,
+      kFreedesktopPortalPath, kFreedesktopPortalFileChooser)
+      ->Then(
+          GetCurrentSerialEventTarget(), __func__,
+          [retPromise](RefPtr<GDBusProxy>&& aProxy) {
+            const char kFreedesktopPortalVersionProperty[] = "version";
+            // Folder selection was added in version 3 of xdg-desktop-portal
+            const uint32_t kFreedesktopPortalMinimumVersion = 3;
+            uint32_t foundVersion = 0;
+
+            RefPtr<GVariant> property =
+                dont_AddRef(g_dbus_proxy_get_cached_property(
+                    aProxy, kFreedesktopPortalVersionProperty));
+
+            if (property) {
+              foundVersion = g_variant_get_uint32(property);
+              LOG(("Found portal version: %u", foundVersion));
+            }
+
+            retPromise->MaybeResolve(foundVersion >=
+                                     kFreedesktopPortalMinimumVersion);
+          },
+          [retPromise](GUniquePtr<GError>&& aError) {
+            g_printerr("Failed to create DBUS proxy: %s\n", aError->message);
+            retPromise->MaybeReject(NS_ERROR_FAILURE);
+          });
+
+  retPromise.forget(aRetPromise);
+  return NS_OK;
+#else
+  return nsBaseFilePicker::IsModeSupported(aMode, aCx, aRetPromise);
+#endif
+}
+
 NS_IMETHODIMP
 nsFilePicker::AppendFilters(int32_t aFilterMask) {
   mAllowURLs = !!(aFilterMask & filterAllowURLs);
@@ -297,7 +367,9 @@ nsFilePicker::GetFile(nsIFile** aFile) {
   *aFile = nullptr;
   nsCOMPtr<nsIURI> uri;
   nsresult rv = GetFileURL(getter_AddRefs(uri));
-  if (!uri) return rv;
+  if (!uri) {
+    return rv;
+  }
 
   nsCOMPtr<nsIFileURL> fileURL(do_QueryInterface(uri, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -327,24 +399,21 @@ nsFilePicker::GetFiles(nsISimpleEnumerator** aFiles) {
   return NS_ERROR_FAILURE;
 }
 
-nsresult nsFilePicker::Show(int16_t* aReturn) {
-  NS_ENSURE_ARG_POINTER(aReturn);
-
-  nsresult rv = Open(nullptr);
-  if (NS_FAILED(rv)) return rv;
-
-  while (mRunning) {
-    g_main_context_iteration(nullptr, TRUE);
-  }
-
-  *aReturn = mResult;
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsFilePicker::Open(nsIFilePickerShownCallback* aCallback) {
   // Can't show two dialogs concurrently with the same filepicker
-  if (mRunning) return NS_ERROR_NOT_AVAILABLE;
+  if (mFileChooser) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  if (MaybeBlockFilePicker(aCallback)) {
+    return NS_OK;
+  }
+
+  // Don't attempt to open a real file-picker in headless mode.
+  if (gfxPlatform::IsHeadless()) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
 
   NS_ConvertUTF16toUTF8 title(mTitle);
 
@@ -393,6 +462,10 @@ nsFilePicker::Open(nsIFilePickerShownCallback* aCallback) {
       gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(file_chooser),
                                         defaultName.get());
       break;
+
+    default:
+      /* no additional setup needed */
+      break;
   }
 
   nsCOMPtr<nsIFile> defaultPath;
@@ -486,7 +559,7 @@ nsFilePicker::Open(nsIFilePickerShownCallback* aCallback) {
   gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(file_chooser),
                                                  TRUE);
 
-  mRunning = true;
+  mFileChooser = file_chooser;
   mCallback = aCallback;
   NS_ADDREF_THIS();
   g_signal_connect(file_chooser, "response", G_CALLBACK(OnResponse), this);
@@ -507,10 +580,54 @@ void nsFilePicker::OnDestroy(GtkWidget* file_chooser, gpointer user_data) {
                                               GTK_RESPONSE_CANCEL);
 }
 
-void nsFilePicker::Done(void* file_chooser, gint response) {
-  mRunning = false;
+bool nsFilePicker::WarnForNonReadableFile(void* file_chooser) {
+  nsCOMPtr<nsIFile> file;
+  GetFile(getter_AddRefs(file));
+  if (!file) {
+    return false;
+  }
 
-  int16_t result;
+  bool isReadable = false;
+  file->IsReadable(&isReadable);
+  if (isReadable) {
+    return false;
+  }
+
+  nsCOMPtr<nsIStringBundleService> stringService =
+      mozilla::components::StringBundle::Service();
+  if (!stringService) {
+    return false;
+  }
+
+  nsCOMPtr<nsIStringBundle> filepickerBundle;
+  nsresult rv = stringService->CreateBundle(
+      "chrome://global/locale/filepicker.properties",
+      getter_AddRefs(filepickerBundle));
+  if (NS_FAILED(rv)) {
+    return false;
+  }
+
+  nsAutoString errorMessage;
+  rv = filepickerBundle->GetStringFromName("selectedFileNotReadableError",
+                                           errorMessage);
+  if (NS_FAILED(rv)) {
+    return false;
+  }
+
+  GtkDialogFlags flags = GTK_DIALOG_DESTROY_WITH_PARENT;
+  auto* cancel_dialog = gtk_message_dialog_new(
+      GTK_WINDOW(file_chooser), flags, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+      "%s", NS_ConvertUTF16toUTF8(errorMessage).get());
+  gtk_dialog_run(GTK_DIALOG(cancel_dialog));
+  gtk_widget_destroy(cancel_dialog);
+
+  return true;
+}
+
+void nsFilePicker::Done(void* file_chooser, gint response) {
+  mFileChooser = nullptr;
+
+  nsIFilePicker::ResultCode result;
   switch (response) {
     case GTK_RESPONSE_OK:
     case GTK_RESPONSE_ACCEPT:
@@ -522,7 +639,13 @@ void nsFilePicker::Done(void* file_chooser, gint response) {
         if (file) {
           bool exists = false;
           file->Exists(&exists);
-          if (exists) result = nsIFilePicker::returnReplace;
+          if (exists) {
+            result = nsIFilePicker::returnReplace;
+          }
+        }
+      } else if (mMode == nsIFilePicker::modeOpen) {
+        if (WarnForNonReadableFile(file_chooser)) {
+          result = nsIFilePicker::returnCancel;
         }
       }
       break;
@@ -569,8 +692,6 @@ void nsFilePicker::Done(void* file_chooser, gint response) {
   if (mCallback) {
     mCallback->Done(result);
     mCallback = nullptr;
-  } else {
-    mResult = result;
   }
   NS_RELEASE_THIS();
 }
@@ -587,16 +708,13 @@ void* nsFilePicker::GtkFileChooserNew(const gchar* title, GtkWindow* parent,
     return (*sGtkFileChooserNativeNewPtr)(title, parent, action, accept_label,
                                           nullptr);
   }
-  if (accept_label == nullptr) {
-    accept_label = (action == GTK_FILE_CHOOSER_ACTION_SAVE) ? GTK_STOCK_SAVE
-                                                            : GTK_STOCK_OPEN;
+  if (!accept_label) {
+    accept_label = g_dgettext(
+        "gtk30", action == GTK_FILE_CHOOSER_ACTION_SAVE ? "_Save" : "_Open");
   }
-  GtkWidget* file_chooser = gtk_file_chooser_dialog_new(
-      title, parent, action, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-      accept_label, GTK_RESPONSE_ACCEPT, nullptr);
-  gtk_dialog_set_alternative_button_order(
-      GTK_DIALOG(file_chooser), GTK_RESPONSE_ACCEPT, GTK_RESPONSE_CANCEL, -1);
-  return file_chooser;
+  return gtk_file_chooser_dialog_new(
+      title, parent, action, g_dgettext("gtk30", "_Cancel"),
+      GTK_RESPONSE_CANCEL, accept_label, GTK_RESPONSE_ACCEPT, nullptr);
 }
 
 void nsFilePicker::GtkFileChooserShow(void* file_chooser) {
@@ -605,7 +723,7 @@ void nsFilePicker::GtkFileChooserShow(void* file_chooser) {
   if (mUseNativeFileChooser && sGtkNativeDialogShowPtr != nullptr) {
     const char* portalEnvString = g_getenv("GTK_USE_PORTAL");
     bool setPortalEnv =
-        (portalEnvString && atoi(portalEnvString) == 0) || !portalEnvString;
+        (portalEnvString && *portalEnvString == '0') || !portalEnvString;
     if (setPortalEnv) {
       setenv("GTK_USE_PORTAL", "1", true);
     }
@@ -644,3 +762,5 @@ void nsFilePicker::GtkFileChooserSetModal(void* file_chooser,
     }
   }
 }
+
+#undef LOG
