@@ -16,7 +16,7 @@
 class nsIWidget;
 class nsIFile;
 
-class nsFilePicker : public nsBaseFilePicker {
+class nsFilePicker final : public nsBaseFilePicker {
  public:
   nsFilePicker();
 
@@ -24,7 +24,6 @@ class nsFilePicker : public nsBaseFilePicker {
 
   // nsIFilePicker (less what's in nsBaseFilePicker)
   NS_IMETHOD Open(nsIFilePickerShownCallback* aCallback) override;
-  NS_IMETHOD Close() override;
   NS_IMETHOD IsModeSupported(nsIFilePicker::Mode, JSContext*,
                              mozilla::dom::Promise**) override;
   NS_IMETHOD AppendFilters(int32_t aFilterMask) override;
@@ -48,7 +47,6 @@ class nsFilePicker : public nsBaseFilePicker {
  protected:
   virtual ~nsFilePicker();
 
-  nsresult Show(nsIFilePicker::ResultCode* aReturn) override;
   void ReadValuesFromFileChooser(void* file_chooser);
 
   bool WarnForNonReadableFile(void* file_chooser);
@@ -62,9 +60,8 @@ class nsFilePicker : public nsBaseFilePicker {
   nsCOMPtr<nsIFilePickerShownCallback> mCallback;
   nsCOMArray<nsIFile> mFiles;
 
-  int16_t mSelectedType;
-  nsIFilePicker::ResultCode mResult;
-  bool mAllowURLs;
+  int16_t mSelectedType = 0;
+  bool mAllowURLs = false;
   nsCString mFileURL;
   nsString mTitle;
   nsString mDefault;
@@ -84,7 +81,7 @@ class nsFilePicker : public nsBaseFilePicker {
   void GtkFileChooserSetModal(void* file_chooser, GtkWindow* parent_widget,
                               gboolean modal);
 
-  GtkFileChooserWidget* mFileChooserDelegate;
+  GtkFileChooserWidget* mFileChooserDelegate = nullptr;
   bool mUseNativeFileChooser;
 
   /**
