@@ -16,11 +16,14 @@
 #include "nsGkAtoms.h"
 #include "nsIDOMWindowUtils.h"
 
+#include "mozilla/AutoRestore.h"
 #include "mozilla/MiscEvents.h"
 #include "mozilla/MouseEvents.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/dom/WheelEventBinding.h"
 #include "mozilla/StaticPrefs_mousewheel.h"
+#include "mozilla/StaticPrefs_widget.h"
+#include "mozilla/widget/WinRegistry.h"
 
 #include <psapi.h>
 
@@ -54,17 +57,84 @@ bool MouseScrollHandler::Device::SetPoint::sMightBeUsing = false;
 // bug 605648 comment 90.
 #define DEFAULT_TIMEOUT_DURATION 1500
 
+/******************************************************************************
+ *
+ * SynthesizingEvent: declarations
+ *
+ ******************************************************************************/
+
+// Maintains state displaced by test-initiated synthesized events. Not
+// instantiated under ordinary release-mode operation.
+class MouseScrollHandler::SynthesizingEvent {
+ public:
+  SynthesizingEvent()
+      : mWnd(nullptr),
+        mMessage(0),
+        mWParam(0),
+        mLParam(0),
+        mStatus(NOT_SYNTHESIZING) {}
+
+  ~SynthesizingEvent() {}
+
+  static SynthesizingEvent* GetActiveInstance();
+
+  nsresult Synthesize(const POINTS& aCursorPoint, HWND aWnd, UINT aMessage,
+                      WPARAM aWParam, LPARAM aLParam,
+                      const BYTE (&aKeyStates)[256]);
+
+  void NotifyMessageReceived(nsWindow* aExpectedWindow, UINT msg, WPARAM wParam,
+                             LPARAM lParam);
+
+  void NotifyMessageHandlingFinished();
+
+  const POINTS& GetCursorPoint() const { return mCursorPoint; }
+
+ private:
+  POINTS mCursorPoint;
+  HWND mWnd;
+  UINT mMessage;
+  WPARAM mWParam;
+  LPARAM mLParam;
+  BYTE mKeyState[256];
+  BYTE mOriginalKeyState[256];
+
+  enum Status {
+    NOT_SYNTHESIZING,
+    SENDING_MESSAGE,
+  };
+  Status mStatus;
+
+  const char* GetStatusName() {
+    switch (mStatus) {
+      case NOT_SYNTHESIZING:
+        return "NOT_SYNTHESIZING";
+      case SENDING_MESSAGE:
+        return "SENDING_MESSAGE";
+      default:
+        return "Unknown";
+    }
+  }
+
+  void Finish();
+};  // SynthesizingEvent
+
 /******************************************************************************
  *
  * MouseScrollHandler
  *
  ******************************************************************************/
 
+/// Convenience alias.
+/* static */
+auto MouseScrollHandler::GetActiveSynthEvent() -> SynthesizingEvent* {
+  return SynthesizingEvent::GetActiveInstance();
+}
+
 /* static */
 POINTS
 MouseScrollHandler::GetCurrentMessagePos() {
-  if (SynthesizingEvent::IsSynthesizing()) {
-    return sInstance->mSynthesizingEvent->GetCursorPoint();
+  if (auto* synth = GetActiveSynthEvent()) {
+    return synth->GetCursorPoint();
   }
   DWORD pos = ::GetMessagePos();
   return MAKEPOINTS(pos);
@@ -90,8 +160,7 @@ MouseScrollHandler* MouseScrollHandler::GetInstance() {
   return sInstance;
 }
 
-MouseScrollHandler::MouseScrollHandler()
-    : mIsWaitingInternalMessage(false), mSynthesizingEvent(nullptr) {
+MouseScrollHandler::MouseScrollHandler() {
   MOZ_LOG(gMouseScrollLog, LogLevel::Info,
           ("MouseScroll: Creating an instance, this=%p, sInstance=%p", this,
            sInstance));
@@ -101,8 +170,6 @@ MouseScrollHandler::~MouseScrollHandler() {
   MOZ_LOG(gMouseScrollLog, LogLevel::Info,
           ("MouseScroll: Destroying an instance, this=%p, sInstance=%p", this,
            sInstance));
-
-  delete mSynthesizingEvent;
 }
 
 /* static */
@@ -112,7 +179,7 @@ void MouseScrollHandler::MaybeLogKeyState() {
   }
   BYTE keyboardState[256];
   if (::GetKeyboardState(keyboardState)) {
-    for (size_t i = 0; i < ArrayLength(keyboardState); i++) {
+    for (size_t i = 0; i < std::size(keyboardState); i++) {
       if (keyboardState[i]) {
         MOZ_LOG(gMouseScrollLog, LogLevel::Debug,
                 ("    Current key state: keyboardState[0x%02zX]=0x%02X (%s)", i,
@@ -131,23 +198,59 @@ void MouseScrollHandler::MaybeLogKeyState() {
   }
 }
 
-/* static */
-bool MouseScrollHandler::NeedsMessage(UINT aMsg) {
-  switch (aMsg) {
-    case WM_SETTINGCHANGE:
-    case WM_MOUSEWHEEL:
-    case WM_MOUSEHWHEEL:
-    case WM_HSCROLL:
-    case WM_VSCROLL:
-    case MOZ_WM_MOUSEVWHEEL:
-    case MOZ_WM_MOUSEHWHEEL:
-    case MOZ_WM_HSCROLL:
-    case MOZ_WM_VSCROLL:
-    case WM_KEYDOWN:
-    case WM_KEYUP:
-      return true;
+bool MouseScrollHandler::ProcessMouseMessage(UINT msg, WPARAM wParam,
+                                             LPARAM lParam,
+                                             MSGResult& aResult) {
+  // Select the appropriate message handler.
+  using HandlerT =
+      bool (MouseScrollHandler::*)(nsWindow*, UINT, WPARAM, LPARAM);
+  HandlerT const handler = [&]() -> HandlerT {
+    switch (msg) {
+      case WM_MOUSEWHEEL:
+      case WM_MOUSEHWHEEL:
+        return &MouseScrollHandler::HandleMouseWheelMessage;
+      case WM_VSCROLL:
+      case WM_HSCROLL:
+        if (lParam || mUserPrefs.IsScrollMessageHandledAsWheelMessage()) {
+          return &MouseScrollHandler::HandleScrollMessageAsMouseWheelMessage;
+        }
+        return &MouseScrollHandler::HandleScrollMessageAsItself;
+      default:
+        MOZ_ASSERT(false, "wrong message type in ProcessMouseMessage");
+        return nullptr;
+    }
+  }();
+  if (!handler) {
+    return false;
   }
-  return false;
+
+  // Find the appropriate nsWindow to handle this message. (This is not
+  // necessarily the window to which the message was sent!)
+  nsWindow* const destWindow = FindTargetWindow(msg, wParam, lParam);
+
+  // Emit a warning if the received message is unexpected, given the
+  // synthesis-state.
+  if (auto* synth = GetActiveSynthEvent()) {
+    synth->NotifyMessageReceived(destWindow, msg, wParam, lParam);
+  }
+
+  if (!destWindow) {
+    // Not over our window; return without consuming. (This will not recurse.)
+    aResult.mConsumed = false;
+    return true;
+  }
+
+  // Actually handle the message.
+  aResult.mConsumed =
+      (GetInstance()->*handler)(destWindow, msg, wParam, lParam);
+  aResult.mResult = 0;
+
+  // Reset the synthesis-state, if necessary.
+  if (auto* synth = GetActiveSynthEvent()) {
+    synth->NotifyMessageHandlingFinished();
+  }
+
+  return true;
 }
 
 /* static */
@@ -169,41 +272,9 @@ bool MouseScrollHandler::ProcessMessage(nsWindow* aWidget, UINT msg,
 
     case WM_MOUSEWHEEL:
     case WM_MOUSEHWHEEL:
-      GetInstance()->ProcessNativeMouseWheelMessage(aWidget, msg, wParam,
-                                                    lParam);
-      sInstance->mSynthesizingEvent->NotifyNativeMessageHandlingFinished();
-      // We don't need to call next wndproc for WM_MOUSEWHEEL and
-      // WM_MOUSEHWHEEL.  We should consume them always.  If the messages
-      // would be handled by our window again, it caused making infinite
-      // message loop.
-      aResult.mConsumed = true;
-      aResult.mResult = (msg != WM_MOUSEHWHEEL);
-      return true;
-
     case WM_HSCROLL:
     case WM_VSCROLL:
-      aResult.mConsumed = GetInstance()->ProcessNativeScrollMessage(
-          aWidget, msg, wParam, lParam);
-      sInstance->mSynthesizingEvent->NotifyNativeMessageHandlingFinished();
-      aResult.mResult = 0;
-      return true;
-
-    case MOZ_WM_MOUSEVWHEEL:
-    case MOZ_WM_MOUSEHWHEEL:
-      GetInstance()->HandleMouseWheelMessage(aWidget, msg, wParam, lParam);
-      sInstance->mSynthesizingEvent->NotifyInternalMessageHandlingFinished();
-      // Doesn't need to call next wndproc for internal wheel message.
-      aResult.mConsumed = true;
-      return true;
-
-    case MOZ_WM_HSCROLL:
-    case MOZ_WM_VSCROLL:
-      GetInstance()->HandleScrollMessageAsMouseWheelMessage(aWidget, msg,
-                                                            wParam, lParam);
-      sInstance->mSynthesizingEvent->NotifyInternalMessageHandlingFinished();
-      // Doesn't need to call next wndproc for internal scroll message.
-      aResult.mConsumed = true;
-      return true;
+      return GetInstance()->ProcessMouseMessage(msg, wParam, lParam, aResult);
 
     case WM_KEYDOWN:
     case WM_KEYUP:
@@ -291,7 +362,7 @@ nsresult MouseScrollHandler::SynthesizeNativeMouseScrollEvent(
   }
 
   if (!sInstance->mSynthesizingEvent) {
-    sInstance->mSynthesizingEvent = new SynthesizingEvent();
+    sInstance->mSynthesizingEvent = MakeUnique<SynthesizingEvent>();
   }
 
   POINTS pts;
@@ -334,9 +405,9 @@ ModifierKeyState MouseScrollHandler::GetModifierKeyState(UINT aMessage) {
   // Assume the Control key is down if the Elantech touchpad has sent the
   // mis-ordered WM_KEYDOWN/WM_MOUSEWHEEL messages.  (See the comment in
   // MouseScrollHandler::Device::Elantech::HandleKeyMessage().)
-  if ((aMessage == MOZ_WM_MOUSEVWHEEL || aMessage == WM_MOUSEWHEEL) &&
-      !result.IsControl() && Device::Elantech::IsZooming()) {
-    // XXX Do we need to unset MODIFIER_SHIFT, MODIFIER_ALT, MODIFIER_OS too?
+  if (aMessage == WM_MOUSEWHEEL && !result.IsControl() &&
+      Device::Elantech::IsZooming()) {
+    // XXX Do we need to unset MODIFIER_SHIFT, MODIFIER_ALT, MODIFIER_META too?
     //     If one of them are true, the default action becomes not zooming.
     result.Unset(MODIFIER_ALTGRAPH);
     result.Set(MODIFIER_CONTROL);
@@ -361,22 +432,14 @@ MouseScrollHandler::ComputeMessagePos(UINT aMessage, WPARAM aWParam,
   return point;
 }
 
-void MouseScrollHandler::ProcessNativeMouseWheelMessage(nsWindow* aWidget,
-                                                        UINT aMessage,
-                                                        WPARAM aWParam,
-                                                        LPARAM aLParam) {
-  if (SynthesizingEvent::IsSynthesizing()) {
-    mSynthesizingEvent->NativeMessageReceived(aWidget, aMessage, aWParam,
-                                              aLParam);
-  }
-
+nsWindow* MouseScrollHandler::FindTargetWindow(UINT aMessage, WPARAM aWParam,
+                                               LPARAM aLParam) {
   POINT point = ComputeMessagePos(aMessage, aWParam, aLParam);
 
   MOZ_LOG(gMouseScrollLog, LogLevel::Info,
-          ("MouseScroll::ProcessNativeMouseWheelMessage: aWidget=%p, "
+          ("MouseScroll::FindTargetWindow: "
            "aMessage=%s, wParam=0x%08zX, lParam=0x%08" PRIXLPTR
            ", point: { x=%ld, y=%ld }",
-           aWidget,
            aMessage == WM_MOUSEWHEEL    ? "WM_MOUSEWHEEL"
            : aMessage == WM_MOUSEHWHEEL ? "WM_MOUSEHWHEEL"
            : aMessage == WM_VSCROLL     ? "WM_VSCROLL"
@@ -386,10 +449,14 @@ void MouseScrollHandler::ProcessNativeMouseWheelMessage(nsWindow* aWidget,
 
   HWND underCursorWnd = ::WindowFromPoint(point);
   if (!underCursorWnd) {
+    // This is unsurprising: Windows ordinarily sends wheel messages to the
+    // focused window, regardless of cursor position. (Nowadays, this is
+    // configurable in Windows' settings, but we've always deliberately
+    // overridden this behavior in Gecko; see bug 168354.)
     MOZ_LOG(gMouseScrollLog, LogLevel::Info,
-            ("MouseScroll::ProcessNativeMouseWheelMessage: "
-             "No window is not found under the cursor"));
-    return;
+            ("MouseScroll::FindTargetWindow: "
+             "No window was found under the cursor"));
+    return nullptr;
   }
 
   if (Device::Elantech::IsPinchHackNeeded() &&
@@ -401,100 +468,46 @@ void MouseScrollHandler::ProcessNativeMouseWheelMessage(nsWindow* aWidget,
     underCursorWnd = WinUtils::FindOurWindowAtPoint(point);
     if (!underCursorWnd) {
       MOZ_LOG(gMouseScrollLog, LogLevel::Info,
-              ("MouseScroll::ProcessNativeMouseWheelMessage: "
+              ("MouseScroll::FindTargetWindow: "
                "Our window is not found under the Elantech helper window"));
-      return;
+      return nullptr;
     }
   }
 
-  // Handle most cases first.  If the window under mouse cursor is our window
-  // except plugin window (MozillaWindowClass), we should handle the message
-  // on the window.
-  if (WinUtils::IsOurProcessWindow(underCursorWnd)) {
-    nsWindow* destWindow = WinUtils::GetNSWindowPtr(underCursorWnd);
-    if (!destWindow) {
-      MOZ_LOG(gMouseScrollLog, LogLevel::Info,
-              ("MouseScroll::ProcessNativeMouseWheelMessage: "
-               "Found window under the cursor isn't managed by nsWindow..."));
-      HWND wnd = ::GetParent(underCursorWnd);
-      for (; wnd; wnd = ::GetParent(wnd)) {
-        destWindow = WinUtils::GetNSWindowPtr(wnd);
-        if (destWindow) {
-          break;
-        }
-      }
-      if (!wnd) {
-        MOZ_LOG(
-            gMouseScrollLog, LogLevel::Info,
-            ("MouseScroll::ProcessNativeMouseWheelMessage: Our window which is "
-             "managed by nsWindow is not found under the cursor"));
-        return;
-      }
-    }
-
-    MOZ_ASSERT(destWindow, "destWindow must not be NULL");
-
-    // Some odd touchpad utils sets focus to window under the mouse cursor.
-    // this emulates the odd behavior for debug.
-    if (mUserPrefs.ShouldEmulateToMakeWindowUnderCursorForeground() &&
-        (aMessage == WM_MOUSEWHEEL || aMessage == WM_MOUSEHWHEEL) &&
-        ::GetForegroundWindow() != destWindow->GetWindowHandle()) {
-      ::SetForegroundWindow(destWindow->GetWindowHandle());
-    }
-
-    MOZ_LOG(gMouseScrollLog, LogLevel::Info,
-            ("MouseScroll::ProcessNativeMouseWheelMessage: Succeeded, "
-             "Posting internal message to an nsWindow (%p)...",
-             destWindow));
-    mIsWaitingInternalMessage = true;
-    UINT internalMessage = WinUtils::GetInternalMessage(aMessage);
-    ::PostMessage(destWindow->GetWindowHandle(), internalMessage, aWParam,
-                  aLParam);
-    return;
+  // If the window under the mouse cursor is not in our process, we assume it's
+  // another application's window, and discard the message.
+  if (!WinUtils::IsOurProcessWindow(underCursorWnd)) {
+    return nullptr;
   }
 
-  // If the window under cursor is not in our process, it means:
-  // 1. The window may be a plugin window (GeckoPluginWindow or its descendant).
-  // 2. The window may be another application's window.
-  HWND pluginWnd = WinUtils::FindOurProcessWindow(underCursorWnd);
-  if (!pluginWnd) {
-    // If there is no plugin window in ancestors of the window under cursor,
-    // the window is for another applications (case 2).
-    // We don't need to handle this message.
-    MOZ_LOG(gMouseScrollLog, LogLevel::Info,
-            ("MouseScroll::ProcessNativeMouseWheelMessage: "
-             "Our window is not found under the cursor"));
-    return;
-  }
-
-  // If the window is a part of plugin, we should post the message to it.
-  MOZ_LOG(
-      gMouseScrollLog, LogLevel::Info,
-      ("MouseScroll::ProcessNativeMouseWheelMessage: Succeeded, "
-       "Redirecting the message to a window which is a plugin child window"));
-  ::PostMessage(underCursorWnd, aMessage, aWParam, aLParam);
-}
-
-bool MouseScrollHandler::ProcessNativeScrollMessage(nsWindow* aWidget,
-                                                    UINT aMessage,
-                                                    WPARAM aWParam,
-                                                    LPARAM aLParam) {
-  if (aLParam || mUserPrefs.IsScrollMessageHandledAsWheelMessage()) {
-    // Scroll message generated by Thinkpad Trackpoint Driver or similar
-    // Treat as a mousewheel message and scroll appropriately
-    ProcessNativeMouseWheelMessage(aWidget, aMessage, aWParam, aLParam);
-    // Always consume the scroll message if we try to emulate mouse wheel
-    // action.
-    return true;
-  }
-
-  if (SynthesizingEvent::IsSynthesizing()) {
-    mSynthesizingEvent->NativeMessageReceived(aWidget, aMessage, aWParam,
-                                              aLParam);
+  // Otherwise, (try to) handle this message on the nsWindow it's associated
+  // with.
+  if (nsWindow* destWindow = WinUtils::GetNSWindowPtr(underCursorWnd)) {
+    return destWindow;
   }
 
   MOZ_LOG(gMouseScrollLog, LogLevel::Info,
-          ("MouseScroll::ProcessNativeScrollMessage: aWidget=%p, "
+          ("MouseScroll::FindTargetWindow: "
+           "Window found under the cursor isn't an nsWindow..."));
+  HWND wnd = ::GetParent(underCursorWnd);
+  for (; wnd; wnd = ::GetParent(wnd)) {
+    if (nsWindow* destWindow = WinUtils::GetNSWindowPtr(wnd)) {
+      return destWindow;
+    }
+  }
+
+  MOZ_LOG(gMouseScrollLog, LogLevel::Info,
+          ("MouseScroll::FindTargetWindow: "
+           "    ...and doesn't have any nsWindow ancestors"));
+  return nullptr;
+}
+
+bool MouseScrollHandler::HandleScrollMessageAsItself(nsWindow* aWidget,
+                                                     UINT aMessage,
+                                                     WPARAM aWParam,
+                                                     LPARAM aLParam) {
+  MOZ_LOG(gMouseScrollLog, LogLevel::Info,
+          ("MouseScroll::HandleScrollMessageAsItself: aWidget=%p, "
            "aMessage=%s, wParam=0x%08zX, lParam=0x%08" PRIXLPTR,
            aWidget, aMessage == WM_VSCROLL ? "WM_VSCROLL" : "WM_HSCROLL",
            aWParam, aLParam));
@@ -537,39 +550,46 @@ bool MouseScrollHandler::ProcessNativeScrollMessage(nsWindow* aWidget,
     default:
       return false;
   }
-  // XXX If this is a plugin window, we should dispatch the event from
-  //     parent window.
+
   aWidget->DispatchContentCommandEvent(&commandEvent);
   return true;
 }
 
-void MouseScrollHandler::HandleMouseWheelMessage(nsWindow* aWidget,
+bool MouseScrollHandler::HandleMouseWheelMessage(nsWindow* aWidget,
                                                  UINT aMessage, WPARAM aWParam,
                                                  LPARAM aLParam) {
-  MOZ_ASSERT((aMessage == MOZ_WM_MOUSEVWHEEL || aMessage == MOZ_WM_MOUSEHWHEEL),
+  // for logging only
+  const char* const msgName [[maybe_unused]] = [&]() {
+    switch (aMessage) {
+      case WM_MOUSEWHEEL:
+        return "WM_MOUSEWHEEL";
+      case WM_MOUSEHWHEEL:
+        return "WM_MOUSEHWHEEL";
+      default:
+        return "err";
+    }
+  }();
+
+  MOZ_ASSERT((aMessage == WM_MOUSEWHEEL || aMessage == WM_MOUSEHWHEEL),
              "HandleMouseWheelMessage must be called with "
-             "MOZ_WM_MOUSEVWHEEL or MOZ_WM_MOUSEHWHEEL");
+             "WM_MOUSEWHEEL or WM_MOUSEHWHEEL");
 
-  MOZ_LOG(
-      gMouseScrollLog, LogLevel::Info,
-      ("MouseScroll::HandleMouseWheelMessage: aWidget=%p, "
-       "aMessage=MOZ_WM_MOUSE%sWHEEL, aWParam=0x%08zX, aLParam=0x%08" PRIXLPTR,
-       aWidget, aMessage == MOZ_WM_MOUSEVWHEEL ? "V" : "H", aWParam, aLParam));
-
-  mIsWaitingInternalMessage = false;
+  MOZ_LOG(gMouseScrollLog, LogLevel::Info,
+          ("MouseScroll::HandleMouseWheelMessage: aWidget=%p, "
+           "aMessage=%s, aWParam=0x%08zX, aLParam=0x%08" PRIXLPTR,
+           aWidget, msgName, aWParam, aLParam));
 
   // If it's not allowed to cache system settings, we need to reset the cache
   // before handling the mouse wheel message.
   mSystemSettings.TrustedScrollSettingsDriver();
 
-  EventInfo eventInfo(aWidget, WinUtils::GetNativeMessage(aMessage), aWParam,
-                      aLParam);
+  EventInfo eventInfo(aWidget, aMessage, aWParam, aLParam);
   if (!eventInfo.CanDispatchWheelEvent()) {
     MOZ_LOG(
         gMouseScrollLog, LogLevel::Info,
         ("MouseScroll::HandleMouseWheelMessage: Cannot dispatch the events"));
     mLastEventInfo.ResetTransaction();
-    return;
+    return false;
   }
 
   // Discard the remaining delta if current wheel message and last one are
@@ -583,9 +603,6 @@ void MouseScrollHandler::HandleMouseWheelMessage(nsWindow* aWidget,
 
   ModifierKeyState modKeyState = GetModifierKeyState(aMessage);
 
-  // Grab the widget, it might be destroyed by a DOM event handler.
-  RefPtr<nsWindow> kungFuDethGrip(aWidget);
-
   WidgetWheelEvent wheelEvent(true, eWheel, aWidget);
   if (mLastEventInfo.InitWheelEvent(aWidget, wheelEvent, modKeyState,
                                     aLParam)) {
@@ -598,29 +615,40 @@ void MouseScrollHandler::HandleMouseWheelMessage(nsWindow* aWidget,
               ("MouseScroll::HandleMouseWheelMessage: The window was destroyed "
                "by eWheel event"));
       mLastEventInfo.ResetTransaction();
-      return;
     }
-  } else {
-    MOZ_LOG(gMouseScrollLog, LogLevel::Info,
-            ("MouseScroll::HandleMouseWheelMessage: eWheel event is not "
-             "dispatched"));
+    return true;
   }
+
+  MOZ_LOG(gMouseScrollLog, LogLevel::Info,
+          ("MouseScroll::HandleMouseWheelMessage: eWheel event was not "
+           "dispatched"));
+  return false;
 }
 
-void MouseScrollHandler::HandleScrollMessageAsMouseWheelMessage(
+bool MouseScrollHandler::HandleScrollMessageAsMouseWheelMessage(
     nsWindow* aWidget, UINT aMessage, WPARAM aWParam, LPARAM aLParam) {
-  MOZ_ASSERT((aMessage == MOZ_WM_VSCROLL || aMessage == MOZ_WM_HSCROLL),
-             "HandleScrollMessageAsMouseWheelMessage must be called with "
-             "MOZ_WM_VSCROLL or MOZ_WM_HSCROLL");
+  // for logging only
+  const char* const msgName [[maybe_unused]] = [&]() {
+    switch (aMessage) {
+      case WM_VSCROLL:
+        return "WM_VSCROLL";
+      case WM_HSCROLL:
+        return "WM_HSCROLL";
+      default:
+        return "err";
+    }
+  }();
 
-  mIsWaitingInternalMessage = false;
+  MOZ_ASSERT((aMessage == WM_VSCROLL || aMessage == WM_HSCROLL),
+             "HandleScrollMessageAsMouseWheelMessage must be called with "
+             "WM_VSCROLL or WM_HSCROLL");
 
   ModifierKeyState modKeyState = GetModifierKeyState(aMessage);
 
   WidgetWheelEvent wheelEvent(true, eWheel, aWidget);
   double& delta =
-      (aMessage == MOZ_WM_VSCROLL) ? wheelEvent.mDeltaY : wheelEvent.mDeltaX;
-  int32_t& lineOrPageDelta = (aMessage == MOZ_WM_VSCROLL)
+      (aMessage == WM_VSCROLL) ? wheelEvent.mDeltaY : wheelEvent.mDeltaX;
+  int32_t& lineOrPageDelta = (aMessage == WM_VSCROLL)
                                  ? wheelEvent.mLineOrPageDeltaY
                                  : wheelEvent.mLineOrPageDeltaX;
 
@@ -645,7 +673,7 @@ void MouseScrollHandler::HandleScrollMessageAsMouseWheelMessage(
       break;
 
     default:
-      return;
+      return false;
   }
   modKeyState.InitInputEvent(wheelEvent);
 
@@ -658,18 +686,18 @@ void MouseScrollHandler::HandleScrollMessageAsMouseWheelMessage(
   MOZ_LOG(
       gMouseScrollLog, LogLevel::Info,
       ("MouseScroll::HandleScrollMessageAsMouseWheelMessage: aWidget=%p, "
-       "aMessage=MOZ_WM_%sSCROLL, aWParam=0x%08zX, aLParam=0x%08" PRIXLPTR ", "
+       "aMessage=%s, aWParam=0x%08zX, aLParam=0x%08" PRIXLPTR ", "
        "wheelEvent { mRefPoint: { x: %d, y: %d }, mDeltaX: %f, mDeltaY: %f, "
        "mLineOrPageDeltaX: %d, mLineOrPageDeltaY: %d, "
        "isShift: %s, isControl: %s, isAlt: %s, isMeta: %s }",
-       aWidget, (aMessage == MOZ_WM_VSCROLL) ? "V" : "H", aWParam, aLParam,
-       wheelEvent.mRefPoint.x.value, wheelEvent.mRefPoint.y.value,
-       wheelEvent.mDeltaX, wheelEvent.mDeltaY, wheelEvent.mLineOrPageDeltaX,
-       wheelEvent.mLineOrPageDeltaY, GetBoolName(wheelEvent.IsShift()),
-       GetBoolName(wheelEvent.IsControl()), GetBoolName(wheelEvent.IsAlt()),
-       GetBoolName(wheelEvent.IsMeta())));
+       aWidget, msgName, aWParam, aLParam, wheelEvent.mRefPoint.x.value,
+       wheelEvent.mRefPoint.y.value, wheelEvent.mDeltaX, wheelEvent.mDeltaY,
+       wheelEvent.mLineOrPageDeltaX, wheelEvent.mLineOrPageDeltaY,
+       GetBoolName(wheelEvent.IsShift()), GetBoolName(wheelEvent.IsControl()),
+       GetBoolName(wheelEvent.IsAlt()), GetBoolName(wheelEvent.IsMeta())));
 
   aWidget->DispatchWheelEvent(&wheelEvent);
+  return true;
 }
 
 /******************************************************************************
@@ -803,30 +831,6 @@ bool MouseScrollHandler::LastEventInfo::InitWheelEvent(
     // If the scroll delta mode isn't per line scroll, we shouldn't allow to
     // override the system scroll speed setting.
     aWheelEvent.mAllowToOverrideSystemScrollSpeed = false;
-  } else if (!MouseScrollHandler::sInstance->mSystemSettings
-                  .IsOverridingSystemScrollSpeedAllowed()) {
-    // If the system settings are customized by either the user or
-    // the mouse utility, we shouldn't allow to override the system scroll
-    // speed setting.
-    aWheelEvent.mAllowToOverrideSystemScrollSpeed = false;
-  } else {
-    // For suppressing too fast scroll, we should ensure that the maximum
-    // overridden delta value should be less than overridden scroll speed
-    // with default scroll amount.
-    double defaultScrollAmount = mIsVertical
-                                     ? SystemSettings::DefaultScrollLines()
-                                     : SystemSettings::DefaultScrollChars();
-    double maxDelta = WidgetWheelEvent::ComputeOverriddenDelta(
-        defaultScrollAmount, mIsVertical);
-    if (maxDelta != defaultScrollAmount) {
-      double overriddenDelta =
-          WidgetWheelEvent::ComputeOverriddenDelta(Abs(delta), mIsVertical);
-      if (overriddenDelta > maxDelta) {
-        // Suppress to fast scroll since overriding system scroll speed with
-        // current delta value causes too big delta value.
-        aWheelEvent.mAllowToOverrideSystemScrollSpeed = false;
-      }
-    }
   }
 
   MOZ_LOG(
@@ -1002,12 +1006,6 @@ void MouseScrollHandler::SystemSettings::TrustedScrollSettingsDriver() {
   // XXX We're not sure about other touchpad drivers...
 }
 
-bool MouseScrollHandler::SystemSettings::
-    IsOverridingSystemScrollSpeedAllowed() {
-  return mScrollLines == DefaultScrollLines() &&
-         mScrollChars == DefaultScrollChars();
-}
-
 /******************************************************************************
  *
  * UserPrefs
@@ -1157,10 +1155,9 @@ void MouseScrollHandler::Device::SynTP::Init() {
   sMinorVersion = -1;
 
   wchar_t buf[40];
-  bool foundKey = WinUtils::GetRegistryKey(
-      HKEY_LOCAL_MACHINE, L"Software\\Synaptics\\SynTP\\Install",
-      L"DriverVersion", buf, sizeof buf);
-  if (!foundKey) {
+  if (!WinRegistry::GetString(
+          HKEY_LOCAL_MACHINE, u"Software\\Synaptics\\SynTP\\Install"_ns,
+          u"DriverVersion"_ns, buf, WinRegistry::kLegacyWinUtilsStringFlags)) {
     MOZ_LOG(gMouseScrollLog, LogLevel::Info,
             ("MouseScroll::Device::SynTP::Init(): "
              "SynTP driver is not found"));
@@ -1204,16 +1201,12 @@ void MouseScrollHandler::Device::Elantech::Init() {
 int32_t MouseScrollHandler::Device::Elantech::GetDriverMajorVersion() {
   wchar_t buf[40];
   // The driver version is found in one of these two registry keys.
-  bool foundKey = WinUtils::GetRegistryKey(HKEY_CURRENT_USER,
-                                           L"Software\\Elantech\\MainOption",
-                                           L"DriverVersion", buf, sizeof buf);
-  if (!foundKey) {
-    foundKey =
-        WinUtils::GetRegistryKey(HKEY_CURRENT_USER, L"Software\\Elantech",
-                                 L"DriverVersion", buf, sizeof buf);
-  }
-
-  if (!foundKey) {
+  if (!WinRegistry::GetString(
+          HKEY_CURRENT_USER, u"Software\\Elantech\\MainOption"_ns,
+          u"DriverVersion"_ns, buf, WinRegistry::kLegacyWinUtilsStringFlags) &&
+      !WinRegistry::GetString(HKEY_CURRENT_USER, u"Software\\Elantech"_ns,
+                              u"DriverVersion"_ns, buf,
+                              WinRegistry::kLegacyWinUtilsStringFlags)) {
     return 0;
   }
 
@@ -1246,7 +1239,7 @@ bool MouseScrollHandler::Device::Elantech::IsHelperWindow(HWND aWnd) {
 
   bool result = false;
   wchar_t path[256] = {L'\0'};
-  if (::GetProcessImageFileNameW(hProcess, path, ArrayLength(path))) {
+  if (::GetProcessImageFileNameW(hProcess, path, std::size(path))) {
     int pathLength = lstrlenW(path);
     if (pathLength >= filenameSuffixLength) {
       if (lstrcmpiW(path + pathLength - filenameSuffixLength, filenameSuffix) ==
@@ -1399,10 +1392,9 @@ void MouseScrollHandler::Device::Apoint::Init() {
   sMinorVersion = -1;
 
   wchar_t buf[40];
-  bool foundKey =
-      WinUtils::GetRegistryKey(HKEY_LOCAL_MACHINE, L"Software\\Alps\\Apoint",
-                               L"ProductVer", buf, sizeof buf);
-  if (!foundKey) {
+  if (!WinRegistry::GetString(HKEY_LOCAL_MACHINE, u"Software\\Alps\\Apoint"_ns,
+                              u"ProductVer"_ns, buf,
+                              WinRegistry::kLegacyWinUtilsStringFlags)) {
     MOZ_LOG(gMouseScrollLog, LogLevel::Info,
             ("MouseScroll::Device::Apoint::Init(): "
              "Apoint driver is not found"));
@@ -1429,19 +1421,20 @@ void MouseScrollHandler::Device::Apoint::Init() {
 
 /* static */
 bool MouseScrollHandler::Device::TrackPoint::IsDriverInstalled() {
-  if (WinUtils::HasRegistryKey(HKEY_CURRENT_USER,
-                               L"Software\\Lenovo\\TrackPoint")) {
+  if (WinRegistry::HasKey(HKEY_CURRENT_USER,
+                          u"Software\\Lenovo\\TrackPoint"_ns)) {
     MOZ_LOG(gMouseScrollLog, LogLevel::Info,
             ("MouseScroll::Device::TrackPoint::IsDriverInstalled(): "
              "Lenovo's TrackPoint driver is found"));
     return true;
   }
 
-  if (WinUtils::HasRegistryKey(HKEY_CURRENT_USER,
-                               L"Software\\Alps\\Apoint\\TrackPoint")) {
+  if (WinRegistry::HasKey(HKEY_CURRENT_USER,
+                          u"Software\\Alps\\Apoint\\TrackPoint"_ns)) {
     MOZ_LOG(gMouseScrollLog, LogLevel::Info,
             ("MouseScroll::Device::TrackPoint::IsDriverInstalled(): "
              "Alps's TrackPoint driver is found"));
+    return true;
   }
 
   return false;
@@ -1455,8 +1448,8 @@ bool MouseScrollHandler::Device::TrackPoint::IsDriverInstalled() {
 
 /* static */
 bool MouseScrollHandler::Device::UltraNav::IsObsoleteDriverInstalled() {
-  if (WinUtils::HasRegistryKey(HKEY_CURRENT_USER,
-                               L"Software\\Lenovo\\UltraNav")) {
+  if (WinRegistry::HasKey(HKEY_CURRENT_USER,
+                          u"Software\\Lenovo\\UltraNav"_ns)) {
     MOZ_LOG(gMouseScrollLog, LogLevel::Info,
             ("MouseScroll::Device::UltraNav::IsObsoleteDriverInstalled(): "
              "Lenovo's UltraNav driver is found"));
@@ -1464,15 +1457,15 @@ bool MouseScrollHandler::Device::UltraNav::IsObsoleteDriverInstalled() {
   }
 
   bool installed = false;
-  if (WinUtils::HasRegistryKey(HKEY_CURRENT_USER,
-                               L"Software\\Synaptics\\SynTPEnh\\UltraNavUSB")) {
+  if (WinRegistry::HasKey(HKEY_CURRENT_USER,
+                          u"Software\\Synaptics\\SynTPEnh\\UltraNavUSB"_ns)) {
     MOZ_LOG(gMouseScrollLog, LogLevel::Info,
             ("MouseScroll::Device::UltraNav::IsObsoleteDriverInstalled(): "
              "Synaptics's UltraNav (USB) driver is found"));
     installed = true;
-  } else if (WinUtils::HasRegistryKey(
+  } else if (WinRegistry::HasKey(
                  HKEY_CURRENT_USER,
-                 L"Software\\Synaptics\\SynTPEnh\\UltraNavPS2")) {
+                 u"Software\\Synaptics\\SynTPEnh\\UltraNavPS2"_ns)) {
     MOZ_LOG(gMouseScrollLog, LogLevel::Info,
             ("MouseScroll::Device::UltraNav::IsObsoleteDriverInstalled(): "
              "Synaptics's UltraNav (PS/2) driver is found"));
@@ -1539,16 +1532,21 @@ bool MouseScrollHandler::Device::SetPoint::IsGetMessagePosResponseValid(
 
 /******************************************************************************
  *
- * SynthesizingEvent
+ * SynthesizingEvent: implementation
  *
  ******************************************************************************/
 
 /* static */
-bool MouseScrollHandler::SynthesizingEvent::IsSynthesizing() {
-  return MouseScrollHandler::sInstance &&
-         MouseScrollHandler::sInstance->mSynthesizingEvent &&
-         MouseScrollHandler::sInstance->mSynthesizingEvent->mStatus !=
-             NOT_SYNTHESIZING;
+MouseScrollHandler::SynthesizingEvent*
+MouseScrollHandler::SynthesizingEvent::GetActiveInstance() {
+  if (auto* outer = MouseScrollHandler::sInstance) {
+    if (auto* self = outer->mSynthesizingEvent.get()) {
+      if (self->mStatus != NOT_SYNTHESIZING) {
+        return self;
+      }
+    }
+  }
+  return nullptr;
 }
 
 nsresult MouseScrollHandler::SynthesizingEvent::Synthesize(
@@ -1558,11 +1556,11 @@ nsresult MouseScrollHandler::SynthesizingEvent::Synthesize(
       gMouseScrollLog, LogLevel::Info,
       ("MouseScrollHandler::SynthesizingEvent::Synthesize(): aCursorPoint: { "
        "x: %d, y: %d }, aWnd=0x%p, aMessage=0x%04X, aWParam=0x%08zX, "
-       "aLParam=0x%08" PRIXLPTR ", IsSynthesized()=%s, mStatus=%s",
+       "aLParam=0x%08" PRIXLPTR ", synthesizing=%s, mStatus=%s",
        aCursorPoint.x, aCursorPoint.y, aWnd, aMessage, aWParam, aLParam,
-       GetBoolName(IsSynthesizing()), GetStatusName()));
+       GetBoolName(!!GetActiveInstance()), GetStatusName()));
 
-  if (IsSynthesizing()) {
+  if (mStatus != NOT_SYNTHESIZING) {
     return NS_ERROR_NOT_AVAILABLE;
   }
 
@@ -1573,15 +1571,14 @@ nsresult MouseScrollHandler::SynthesizingEvent::Synthesize(
   // unexpected message which were not sent from here.
   mCursorPoint = aCursorPoint;
 
-  mWnd = aWnd;
-  mMessage = aMessage;
-  mWParam = aWParam;
-  mLParam = aLParam;
-
   memcpy(mKeyState, aKeyStates, sizeof(mKeyState));
   ::SetKeyboardState(mKeyState);
 
   mStatus = SENDING_MESSAGE;
+  mWnd = aWnd;
+  mMessage = aMessage;
+  mWParam = aWParam;
+  mLParam = aLParam;
 
   // Don't assume that aWnd is always managed by nsWindow.  It might be
   // a plugin window.
@@ -1590,58 +1587,39 @@ nsresult MouseScrollHandler::SynthesizingEvent::Synthesize(
   return NS_OK;
 }
 
-void MouseScrollHandler::SynthesizingEvent::NativeMessageReceived(
-    nsWindow* aWidget, UINT aMessage, WPARAM aWParam, LPARAM aLParam) {
-  if (mStatus == SENDING_MESSAGE && mMessage == aMessage &&
-      mWParam == aWParam && mLParam == aLParam) {
-    mStatus = NATIVE_MESSAGE_RECEIVED;
-    if (aWidget && aWidget->GetWindowHandle() == mWnd) {
-      return;
-    }
-    // Otherwise, the message may not be sent by us.
+void MouseScrollHandler::SynthesizingEvent::NotifyMessageReceived(
+    nsWindow* aWindow, UINT msg, WPARAM wParam, LPARAM lParam) {
+  MOZ_ASSERT(mStatus != NOT_SYNTHESIZING);
+
+  // check that the received message is as expected
+  HWND handle = aWindow ? aWindow->GetWindowHandle() : nullptr;
+  nsWindow* widget [[maybe_unused]] = WinUtils::GetNSWindowPtr(mWnd);
+
+  if (mStatus == SENDING_MESSAGE && aWindow == widget && mWnd == handle &&
+      mMessage == msg && mWParam == wParam && mLParam == lParam) {
+    // all is well; do nothing
+    MOZ_LOG(
+        gMouseScrollLog, LogLevel::Debug,
+        ("MouseScrollHandler::SynthesizingEvent::NotifyMessageReceived(): OK"));
+    return;
   }
 
+  // log values: [{received} vs. {expected}]
   MOZ_LOG(gMouseScrollLog, LogLevel::Info,
-          ("MouseScrollHandler::SynthesizingEvent::NativeMessageReceived(): "
-           "aWidget=%p, aWidget->GetWindowHandle()=0x%p, mWnd=0x%p, "
-           "aMessage=0x%04X, aWParam=0x%08zX, aLParam=0x%08" PRIXLPTR
-           ", mStatus=%s",
-           aWidget, aWidget ? aWidget->GetWindowHandle() : nullptr, mWnd,
-           aMessage, aWParam, aLParam, GetStatusName()));
+          ("MouseScrollHandler::SynthesizingEvent::NotifyMessageReceived(): "
+           "handle=[0x%08zu vs. 0x%08zu], widget=[%p vs. %p], "
+           "msg=[0x%04X vs. 0x%04X], wParam=[0x%08zX vs. 0x%08zX], "
+           "lParam=[0x%08" PRIXLPTR "vs. 0x%08" PRIXLPTR "], mStatus=%s",
+           size_t(handle), size_t(mWnd), aWindow, widget, msg, mMessage, wParam,
+           mWParam, lParam, mLParam, GetStatusName()));
 
-  // We failed to receive our sent message, we failed to do the job.
-  Finish();
-
-  return;
+  // We probably shouldn't get here in normal operation, but we do during
+  // testing. (See failures on bug 1945257.) Fall through without further
+  // action.
 }
 
-void MouseScrollHandler::SynthesizingEvent::
-    NotifyNativeMessageHandlingFinished() {
-  if (!IsSynthesizing()) {
-    return;
-  }
-
-  MOZ_LOG(gMouseScrollLog, LogLevel::Info,
-          ("MouseScrollHandler::SynthesizingEvent::"
-           "NotifyNativeMessageHandlingFinished(): IsWaitingInternalMessage=%s",
-           GetBoolName(MouseScrollHandler::IsWaitingInternalMessage())));
-
-  if (MouseScrollHandler::IsWaitingInternalMessage()) {
-    mStatus = INTERNAL_MESSAGE_POSTED;
-    return;
-  }
-
-  // If the native message handler didn't post our internal message,
-  // we our job is finished.
-  // TODO: When we post the message to plugin window, there is remaning job.
-  Finish();
-}
-
-void MouseScrollHandler::SynthesizingEvent::
-    NotifyInternalMessageHandlingFinished() {
-  if (!IsSynthesizing()) {
-    return;
-  }
+void MouseScrollHandler::SynthesizingEvent::NotifyMessageHandlingFinished() {
+  MOZ_ASSERT(mStatus != NOT_SYNTHESIZING);
 
   MOZ_LOG(gMouseScrollLog, LogLevel::Info,
           ("MouseScrollHandler::SynthesizingEvent::"
@@ -1651,9 +1629,7 @@ void MouseScrollHandler::SynthesizingEvent::
 }
 
 void MouseScrollHandler::SynthesizingEvent::Finish() {
-  if (!IsSynthesizing()) {
-    return;
-  }
+  MOZ_ASSERT(mStatus != NOT_SYNTHESIZING);
 
   MOZ_LOG(gMouseScrollLog, LogLevel::Info,
           ("MouseScrollHandler::SynthesizingEvent::Finish()"));
@@ -1662,7 +1638,14 @@ void MouseScrollHandler::SynthesizingEvent::Finish() {
   ::SetKeyboardState(mOriginalKeyState);
 
   mStatus = NOT_SYNTHESIZING;
+  mWnd = nullptr;
+  mMessage = 0;
+  mWParam = 0;
+  mLParam = 0;
 }
 
 }  // namespace widget
 }  // namespace mozilla
+
+// Restore access to GetMessagePos for unified builds.
+#undef GetMessagePos
